@@ -1,8 +1,14 @@
 #!/bin/sh
 
-ANDROID_SDK_PATH=/cygdrive/c/Program\ Files/Android/android-sdk
-ANDROID_NDK_PATH=/cygdrive/c/Users/${USER}/workspace/android-ndk-r8
-export JAVA_HOME=c:\\Program\ Files\\Java\\jdk1.7.0_02
+if test "$ANDROID_SDK_PATH" = ""; then
+	ANDROID_SDK_PATH=/cygdrive/c/Program\ Files/Android/android-sdk
+fi
+if test "$ANDROID_NDK_PATH" = ""; then
+	ANDROID_NDK_PATH=/cygdrive/c/Users/${USER}/workspace/android-ndk-r8
+fi
+if test "$JAVA_HOME" = ""; then
+	export JAVA_HOME=c:\\Program\ Files\\Java\\jdk1.8.0_51
+fi
 
 # Requires android-11 or later.
 "${ANDROID_SDK_PATH}/tools/android.bat" update project --path . --target android-11
