@@ -10,6 +10,7 @@
 
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_conf_io.h>
+#include  <kiklib/kik_mem.h>
 
 #include  "../x_window.h"
 #include  "../../common/c_animgif.c"
@@ -416,6 +417,7 @@ process_mouse_event(
 			_display.button_state = 0 ;
 			break ;
 
+		case  7 /* AMOTION_EVENT_ACTION_HOVER_MOVE */:
 		case  AMOTION_EVENT_ACTION_MOVE:
 			xev.type = MotionNotify ;
 			break ;
@@ -447,7 +449,9 @@ process_mouse_event(
 
 		if( xev.type == ButtonPress)
 		{
-			if( xev.x + xev.y + 20 >= _disp.width + _disp.height)
+			/* 800x600 display => (800+600) / 64 = 21 */
+			if( xev.x + xev.y + (_disp.width + _disp.height) / 64 >=
+			    _disp.width + _disp.height)
 			{
 				if( click_num == 0)
 				{
