@@ -37,7 +37,7 @@ public class MLActivity extends NativeActivity
 	}
 
 	private native void  visibleFrameChanged( int  yoffset , int  width , int  height) ;
-	private native void  commitText( String  str) ;
+	private native void  commitTextLock( String  str) ;
 	private native void  commitTextNoLock( String  str) ;
 	private native void  preeditText( String  str) ;
 	private native String  convertToTmpPath( String  path) ;
@@ -46,7 +46,8 @@ public class MLActivity extends NativeActivity
 	private String  keyString ;
 	private View  contentView ;
 	private int  inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE ;
-	private int  imeOptions = EditorInfo.IME_FLAG_NO_ENTER_ACTION | EditorInfo.IME_ACTION_DONE ;
+	private int  imeOptions = EditorInfo.IME_FLAG_NO_ENTER_ACTION | EditorInfo.IME_ACTION_DONE |
+								EditorInfo.IME_FLAG_NO_FULLSCREEN ;
 	private ClipboardManager  clipMan ;
 
 	private class TextInputConnection extends BaseInputConnection
@@ -61,7 +62,7 @@ public class MLActivity extends NativeActivity
 		{
 			super.commitText( text , newCursorPosition) ;
 
-			commitText( text.toString()) ;
+			commitTextLock( text.toString()) ;
 
 			return  true ;
 		}
