@@ -5,6 +5,7 @@
 #ifdef  BUILTIN_IMAGELIB
 
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
+#include  <kiklib/kik_mem.h>	/* alloca */
 
 #include  "c_sixel.c"
 #include  "c_regis.c"
@@ -320,7 +321,7 @@ create_cardinals_from_sixel(
 #endif  /* BUILTIN_IMAGELIB */
 
 
-#ifdef  USE_X11
+#ifdef  USE_XLIB
 
 /* seek the closest color */
 static int
@@ -334,8 +335,8 @@ closest_color_index(
 {
 	int  closest = 0 ;
 	int  i ;
-	u_long  min = 0xffffff ;
-	u_long  diff ;
+	u_int  min = 0xffffff ;
+	u_int  diff ;
 	int  diff_r , diff_g , diff_b ;
 
 	for( i = 0 ; i < len ; i++)
@@ -413,4 +414,4 @@ msb(
 	return  nth ;
 }
 
-#endif	/* USE_X11 */
+#endif	/* USE_XLIB */
