@@ -5,6 +5,7 @@
 #include <string.h>	/* memcpy */
 #include <sys/stat.h>	/* fstat */
 #include <kiklib/kik_util.h>	/* K_MIN */
+#include <kiklib/kik_mem.h>	/* alloca */
 
 
 #ifdef  SIXEL_1BPP
@@ -772,7 +773,7 @@ body:
 
 					for( x = 0 ; x < rep ; x ++)
 					{
-					#if  defined(GDK_PIXBUF_VERSION)
+					#if  defined(GDK_PIXBUF_VERSION) || defined(USE_QUARTZ)
 						/* RGBA */
 						pixels[((pix_y + y) * width + pix_x + x) *
 							PIXEL_SIZE] =
