@@ -6,6 +6,7 @@
 
 #include  <kiklib/kik_mem.h>	/* malloc/alloca/free */
 #include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep kik_snprintf*/
+#include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
 
 #include  <x_im.h>
 
@@ -17,6 +18,9 @@
 #define  IM_WNN_DEBUG  1
 #endif
 
+/* Don't forget to modify digit_to_wstr() if MAX_DIGIT_NUM is changed. */
+#define  MAX_DIGIT_NUM  3	/* 999 */
+#define  CAND_WINDOW_ROWS  5
 
 typedef struct im_wnn
 {
@@ -50,6 +54,47 @@ static mkf_parser_t *  parser_wchar = NULL ;
 
 /* --- static functions --- */
 
+static wchar *
+digit_to_wstr(
+	wchar *  dst ,
+	int  digit
+	)
+{
+	int  num ;
+
+	if( digit >= 1000)
+	{
+		/* Ignore it because MAX_DIGIT_NUM is 3 (=999). */
+		digit -= (digit / 1000) * 1000 ;
+	}
+
+	if( digit >= 100)
+	{
+		num = digit / 100 ;
+		*(dst++) = num + 0x30 ;
+		digit -= num * 100 ;
+	}
+	else if( ((digit - 1) / CAND_WINDOW_ROWS + 1) * CAND_WINDOW_ROWS >= 100)
+	{
+		*(dst++) = ' ' ;
+	}
+
+	if( digit >= 10)
+	{
+		num = digit / 10 ;
+		*(dst++) = num + 0x30 ;
+		digit -= num * 10 ;
+	}
+	else if( ((digit - 1) / CAND_WINDOW_ROWS + 1) * CAND_WINDOW_ROWS >= 10)
+	{
+		*(dst++) = ' ' ;
+	}
+
+	*(dst++) = digit + 0x30 ;
+
+	return  dst ;
+}
+
 static void
 wchar_parser_set_str(
 	mkf_parser_t *  parser ,
@@ -151,15 +196,12 @@ preedit(
 	int  rev_pos ,
 	int  rev_len ,
 	char *  candidateword ,	/* wchar */
-	size_t  candidateword_len
+	size_t  candidateword_len ,
+	char *  pos
 	)
 {
 	int  x ;
 	int  y ;
-	mkf_char_t  ch ;
-	ml_char_t *  p ;
-	u_int  num_of_chars ;
-	u_int  len ;
 
 	if( preedit == NULL)
 	{
@@ -175,7 +217,12 @@ preedit(
 	}
 	else
 	{
+		mkf_char_t  ch ;
+		ml_char_t *  p ;
+		u_int  num_of_chars ;
+		u_int  len ;
 		u_char *  tmp = NULL ;
+		size_t  pos_len ;
 
 		wnn->im.preedit.cursor_offset = rev_pos ;
 
@@ -187,8 +234,10 @@ preedit(
 			num_of_chars ++ ;
 		}
 
+		pos_len = strlen(pos) ;
+
 		if( ( p = realloc( wnn->im.preedit.chars ,
-				sizeof(ml_char_t) * num_of_chars)) == NULL)
+				sizeof(ml_char_t) * (num_of_chars + pos_len))) == NULL)
 		{
 			return ;
 		}
@@ -202,7 +251,7 @@ preedit(
 		}
 
 		(*syms->ml_str_init)( wnn->im.preedit.chars = p ,
-				wnn->im.preedit.num_of_chars = num_of_chars) ;
+				wnn->im.preedit.num_of_chars = (num_of_chars + pos_len)) ;
 		wnn->im.preedit.filled_len = 0 ;
 
 		(*wnn->parser_term->init)( wnn->parser_term) ;
@@ -276,6 +325,13 @@ preedit(
 			wnn->im.preedit.filled_len ++ ;
 		}
 
+		for( ; pos_len > 0 ; pos_len --)
+		{
+			(*syms->ml_char_set)( p++ , *(pos++) , US_ASCII , 0 , 0 ,
+				ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 1 , 0 , 0) ;
+			wnn->im.preedit.filled_len ++ ;
+		}
+
 		if( tmp)
 		{
 			free( tmp) ;
@@ -322,7 +378,7 @@ candidate:
 			{
 			#ifdef  DEBUG
 				kik_warn_printf( KIK_DEBUG_TAG
-					" x_im_candidate_screen_new() failed.\n") ;
+					" x_im_status_screen_new() failed.\n") ;
 			#endif
 
 				return ;
@@ -353,17 +409,14 @@ candidate:
 
 static void
 commit(
-	void *  p ,
+	im_wnn_t *  wnn ,
 	const char *  str ,
 	size_t  len
 	)
 {
-	im_wnn_t *  wnn ;
 	u_char  conv_buf[256] ;
 	size_t  filled_len ;
 
-	wnn = (im_wnn_t*) p ;
-
 	(*parser_wchar->init)( parser_wchar) ;
 	(*parser_wchar->set_str)( parser_wchar , (u_char*)str , len) ;
 
@@ -407,12 +460,12 @@ insert_char(
 		{ 0xa4d0 , 0xa4d3 , 0xa4d6 , 0xa4d9 , 0xa4dc } , /* b */
 		{ 0xa4ab , 0xa4ad , 0xa4af , 0xa4b1 , 0xa4b3 } , /* c */
 		{ 0xa4c0 , 0xa4c2 , 0xa4c5 , 0xa4c7 , 0xa4c9 } , /* d */
-		{ 0xa4e3 , 0 ,      0xa4e5 , 0      , 0xa4e7 } , /* xy */
-		{ 0 ,      0 ,      0 ,      0 ,      0 ,    } ,
+		{ 0xa4e3 , 0xa4a3 , 0xa4e5 , 0xa4a7 , 0xa4e7 } , /* xy */
+		{ 0 ,      0 ,      0xa4d5 , 0 ,      0 ,    } , /* f */
 		{ 0xa4ac , 0xa4ae , 0xa4b0 , 0xa4b2 , 0xa4b4 } , /* g */
 		{ 0xa4cf , 0xa4d2 , 0xa4d5 , 0xa4d8 , 0xa4db } , /* h */
-		{ 0 ,      0 ,      0 ,      0 ,      0 ,    } ,
-		{ 0 ,      0 ,      0 ,      0 ,      0 ,    } ,
+		{ 0xa4e3 , 0 ,      0xa4e5 , 0xa4a7 , 0xa4e7 } , /* ch/sh */
+		{ 0 ,      0xa4b8 , 0 ,      0 ,      0 ,    } , /* j */
 		{ 0xa4ab , 0xa4ad , 0xa4af , 0xa4b1 , 0xa4b3 } , /* k */
 		{ 0xa4a1 , 0xa4a3 , 0xa4a5 , 0xa4a7 , 0xa4a9 } , /* l */
 		{ 0xa4de , 0xa4df , 0xa4e0 , 0xa4e1 , 0xa4e2 } , /* m */
@@ -433,7 +486,7 @@ insert_char(
 	static wchar  sign_table1[] =
 	{
 	                 0xa1aa , 0xa1c9 , 0xa1f4 , 0xa1f0 , 0xa1f3 , 0xa1f5 , 0xa1c7 ,
-		0xa1ca , 0xa1cb , 0xa1f6 , 0xa1dc , 0xa1a4 , 0xa1bd , 0xa1a3 , 0xa1bf ,
+		0xa1ca , 0xa1cb , 0xa1f6 , 0xa1dc , 0xa1a4 , 0xa1bc , 0xa1a3 , 0xa1bf ,
 		0xa3b0 , 0xa3b1 , 0xa3b2 , 0xa3b3 , 0xa3b4 , 0xa3b5 , 0xa3b6 , 0xa3b7 ,
 		0xa3b8 , 0xa3b9 , 0xa1a7 , 0xa1a8 , 0xa1e3 , 0xa1e1 , 0xa1e4 , 0xa1a9 ,
 		0xa1f7 ,
@@ -453,30 +506,83 @@ insert_char(
 		jcDeleteChar( wnn->convbuf , 1) ;
 	}
 
-	if( key_char == 'a')
+	if( key_char == 'a' || key_char == 'i' || key_char == 'u' ||
+	    key_char == 'e' || key_char == 'o')
 	{
-		wch = kana_table[wnn->dan].a ;
-		wnn->dan = 0 ;
-	}
-	else if( key_char == 'i')
-	{
-		wch = kana_table[wnn->dan].i ;
-		wnn->dan = 0 ;
-	}
-	else if( key_char == 'u')
-	{
-		wch = kana_table[wnn->dan].u ;
-		wnn->dan = 0 ;
-	}
-	else if( key_char == 'e')
-	{
-		wch = kana_table[wnn->dan].e ;
-		wnn->dan = 0 ;
-	}
-	else if( key_char == 'o')
-	{
-		wch = kana_table[wnn->dan].o ;
-		wnn->dan = 0 ;
+		if( wnn->dan == 'f' - 'a')
+		{
+			if( key_char != 'u')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d5) ;	/* hu */
+				wnn->dan = 'x' - 'a' ;
+			}
+		}
+		else if( wnn->dan == 'j' - 'a')
+		{
+			if( key_char != 'i')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b8) ;	/* zi */
+				wnn->dan = 'e' - 'a' ;
+			}
+		}
+
+		if( key_char == 'a')
+		{
+			wch = kana_table[wnn->dan].a ;
+			wnn->dan = 0 ;
+		}
+		else if( key_char == 'i')
+		{
+			if( wnn->dan == 'i' - 'a')
+			{
+				wnn->dan = 0 ;
+
+				return  0 ;	/* shi/chi */
+			}
+
+			wch = kana_table[wnn->dan].i ;
+			wnn->dan = 0 ;
+		}
+		else if( key_char == 'u')
+		{
+			if( wnn->dan == 'j' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b8) ;	/* zi */
+				wnn->dan = 'e' - 'a' ;
+			}
+			wch = kana_table[wnn->dan].u ;
+			wnn->dan = 0 ;
+		}
+		else if( key_char == 'e')
+		{
+			if( wnn->dan == 'f' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d5) ;	/* hu */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'j' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b8) ;	/* zi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			wch = kana_table[wnn->dan].e ;
+			wnn->dan = 0 ;
+		}
+		else /* if( key_char == 'o') */
+		{
+			if( wnn->dan == 'f' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d5) ;	/* hu */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'j' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b8) ;	/* zi */
+				wnn->dan = 'e' - 'a' ;
+			}
+			wch = kana_table[wnn->dan].o ;
+			wnn->dan = 0 ;
+		}
 	}
 	else if( ( '!' <= key_char && key_char <= '@') ||
 	         ( '[' <= key_char && key_char <= '_') ||
@@ -503,10 +609,26 @@ insert_char(
 	}
 	else
 	{
-		if( wnn->dan == 'n' - 'a' && key_char == 'n')
+		if( wnn->dan == 'n' - 'a' &&
+		    key_char != 'a' && key_char != 'i' && key_char != 'u' &&
+		    key_char != 'e' && key_char != 'o' && key_char != 'y')
 		{
-			wch = 0xa4f3 ;	/* n */
-			wnn->dan = 0 ;
+			if( key_char == 'n')
+			{
+				wch = 0xa4f3 ;	/* n */
+				wnn->dan = 0 ;
+			}
+			else
+			{
+				jcInsertChar( wnn->convbuf , 0xa4f3) ;
+				wch = key_char ;
+				wnn->dan = key_char - 'a' ;
+			}
+		}
+		else if( key_char == wnn->dan + 'a')
+		{
+			jcInsertChar( wnn->convbuf , 0xa4c3) ;
+			wch = key_char ;
 		}
 		else if( key_char == 'y')
 		{
@@ -520,6 +642,56 @@ insert_char(
 				jcInsertChar( wnn->convbuf , 0xa4ae) ;	/* gi */
 				wnn->dan = 'x' - 'a' ;
 			}
+			else if( wnn->dan == 's' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b7) ;	/* si */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'z' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b8) ;	/* zi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 't' - 'a' || wnn->dan == 'c' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4c1) ;	/* ti */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'd' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4c2) ;	/* di */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'n' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4cb) ;	/* ni */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'h' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d2) ;	/* hi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'b' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d3) ;	/* bi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'p' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4d4) ;	/* pi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'm' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4df) ;	/* mi */
+				wnn->dan = 'x' - 'a' ;
+			}
+			else if( wnn->dan == 'r' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4ea) ;	/* ri */
+				wnn->dan = 'x' - 'a' ;
+			}
 
 			if( wnn->dan == 'x' - 'a')
 			{
@@ -546,6 +718,30 @@ insert_char(
 				goto  normal ;
 			}
 		}
+		else if( key_char == 'h')
+		{
+			if( wnn->dan == 'c' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4c1) ;	/* ti */
+				wnn->dan = 'i' - 'a' ;
+			}
+			else if( wnn->dan == 's' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4b7) ;	/* si */
+				wnn->dan = 'i' - 'a' ;
+			}
+			else if( wnn->dan == 'd' - 'a')
+			{
+				jcInsertChar( wnn->convbuf , 0xa4c7) ;	/* di */
+				wnn->dan = 'e' - 'a' ;
+			}
+			else
+			{
+				goto  normal ;
+			}
+
+			wch = 'h' ;
+		}
 		else
 		{
 		normal:
@@ -555,7 +751,15 @@ insert_char(
 			}
 
 			wch = key_char ;
-			wnn->dan = key_char - 'a' ;
+
+			if( 'a' <= key_char && key_char <= 'z')
+			{
+				wnn->dan = key_char - 'a' ;
+			}
+			else
+			{
+				wnn->dan = 0 ;
+			}
 		}
 	}
 
@@ -583,7 +787,7 @@ fix(
 	{
 		wnn->dan = 0 ;
 		wnn->is_cand = 0 ;
-		preedit( wnn , "" , 0 , 0 , 0 , "" , 0) ;
+		preedit( wnn , "" , 0 , 0 , 0 , "" , 0 , "") ;
 		commit( wnn , wnn->convbuf->displayBuf ,
 			(wnn->convbuf->displayEnd - wnn->convbuf->displayBuf) * 2) ;
 		jcFix( wnn->convbuf) ;
@@ -650,12 +854,12 @@ switch_mode(
 
 	if( ( wnn->is_enabled = ( ! wnn->is_enabled)))
 	{
-		preedit( wnn , NULL , 0 , 0 , 0 , NULL , 0) ;
+		preedit( wnn , NULL , 0 , 0 , 0 , NULL , 0 , "") ;
 	}
 	else
 	{
 		jcClear( wnn->convbuf) ;
-		preedit( wnn , "" , 0 , 0 , 0 , "" , 0) ;
+		preedit( wnn , "" , 0 , 0 , 0 , "" , 0 , "") ;
 	}
 
 	return  1 ;
@@ -674,6 +878,7 @@ key_event(
 	wchar *  cand = NULL ;
 	size_t  cand_len = 0 ;
 	int  ret = 0 ;
+	char *  pos = "" ;
 
 	wnn = (im_wnn_t*) im ;
 
@@ -694,7 +899,7 @@ key_event(
 	{
 		ret = fix( wnn) ;
 	}
-	else if( ksym == XK_BackSpace || ksym == XK_Delete)
+	else if( ksym == XK_BackSpace || ksym == XK_Delete || key_char == 0x08 /* Ctrl+h */)
 	{
 		if( wnn->im.preedit.filled_len > 0)
 		{
@@ -796,28 +1001,30 @@ key_event(
 				if( jcCandidateInfo( wnn->convbuf , 0 , &ncand , &curcand) == 0 &&
 				    ( ! wnn->is_cand ||
 				      ( ksym == XK_Up ?
-				          (curcand % 5 == 4 || curcand == ncand - 1) :
-				          (curcand % 5 == 0))))
+				          (curcand % CAND_WINDOW_ROWS == CAND_WINDOW_ROWS - 1 ||
+					   curcand == ncand - 1) :
+				          (curcand % CAND_WINDOW_ROWS == 0))))
 				{
 					wchar  tmp[1024] ;
 					wchar *  src ;
 					wchar *  dst ;
 					int  count ;
-					int  beg = curcand - curcand % 5 ;
+					int  beg = curcand - curcand % CAND_WINDOW_ROWS ;
 
 					wnn->is_cand = 1 ;
 
-					for( count = 0 ; count < 5 ; count++)
+					for( count = 0 ; count < CAND_WINDOW_ROWS ; count++)
 					{
 						if( jcGetCandidate( wnn->convbuf , beg + count ,
 							tmp , sizeof(tmp)) == 0)
 						{
+							cand_len += (MAX_DIGIT_NUM + 1) ;
 							for( src = tmp ; *src ; src++)
 							{
 								cand_len ++ ;
 							}
 
-							if( count < 4 &&
+							if( count < CAND_WINDOW_ROWS - 1 &&
 							    beg + count < ncand - 1)
 							{
 								cand_len ++ ;	/* '\n' */
@@ -829,18 +1036,23 @@ key_event(
 					{
 						dst = cand ;
 
-						for( count = 0 ; count < 5 ; count++)
+						for( count = 0 ; count < CAND_WINDOW_ROWS ;
+						     count++)
 						{
 							if( jcGetCandidate( wnn->convbuf ,
 								beg + count ,
 								tmp , sizeof(tmp)) == 0)
 							{
+								dst = digit_to_wstr( dst ,
+									beg + count + 1) ;
+								*(dst++) = ' ' ;
+
 								for( src = tmp ; *src ; src++)
 								{
 									*(dst++) = *src ;
 								}
 
-								if( count < 4 &&
+								if( count < CAND_WINDOW_ROWS - 1 &&
 								    beg + count < ncand - 1)
 								{
 									*(dst++) = '\n' ;
@@ -849,6 +1061,11 @@ key_event(
 						}
 					}
 				}
+
+				if( ( pos = alloca( 4 + DIGIT_STR_LEN(int) * 2 + 1)))
+				{
+					sprintf( pos , " [%d/%d]" , curcand + 1 , ncand) ;
+				}
 			}
 		}
 		else if( ksym == XK_Right)
@@ -893,13 +1110,13 @@ key_event(
 			wnn->convbuf->displayBuf ,
 			wnn->convbuf->clauseInfo[wnn->convbuf->curLCEnd].dispp -
 			wnn->convbuf->clauseInfo[wnn->convbuf->curLCStart].dispp ,
-			cand , cand_len * sizeof(wchar)) ;
+			cand , cand_len * sizeof(wchar) , pos) ;
 	}
 	else
 	{
 		preedit( wnn , wnn->convbuf->displayBuf ,
 			(wnn->convbuf->displayEnd - wnn->convbuf->displayBuf) * 2 ,
-			jcDotOffset( wnn->convbuf) , 0 , (char*)kana , sizeof(kana)) ;
+			jcDotOffset( wnn->convbuf) , 0 , (char*)kana , sizeof(kana) , pos) ;
 	}
 
 	return  ret ;
