@@ -284,8 +284,13 @@ kik_pty_fork(
 		}
 	}
 
+#ifdef  B38400
+	cfsetispeed( &tio , B38400) ;
+	cfsetospeed( &tio , B38400) ;
+#else
 	cfsetispeed( &tio , B9600) ;
 	cfsetospeed( &tio , B9600) ;
+#endif
 
 	if( tcsetattr( *master , TCSANOW , &tio) < 0)
 	{
