@@ -1,9 +1,11 @@
 /*
- *	$Id: kik_time.c,v 1.6 2007/01/24 20:47:09 mfabian Exp $
+ *	$Id$
  */
 
 #include  "kik_time.h"
 
+#ifndef  REMOVE_FUNCS_MLTERM_UNUSE
+
 #include  <string.h>	/* strncmp()/memset() */
 #include  <stdio.h>
 #include  <ctype.h>	/* isdigit() */
@@ -408,3 +410,5 @@ kik_time_int_month_to_abbrev_string(
 		return  NULL ;
 	}
 }
+
+#endif /* REMOVE_FUNCS_MLTERM_UNUSE */
