@@ -6,10 +6,6 @@
                            finger doesn't (WHY ??)
  */
 
-#if 1
-#define UTMPX
-#endif
-
 #include "kik_utmp.h"
 
 #include <stdio.h>	/* NULL */
@@ -17,7 +13,7 @@
 #include <string.h>	/* strncmp */
 #include <time.h>	/* time */
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
 #include <utmpx.h>	/* getut*, setut*, etc */
 #else
 #include <utmp.h>	/* getut*, setut*, etc */
@@ -31,8 +27,11 @@
 #include "kik_privilege.h"
 #include "kik_debug.h"
 
+#ifdef _PATH_UTMP_UPDATE
+#include <errno.h>
+#endif
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
 #define LINE_WIDTH 32
 #else
 #define LINE_WIDTH 12
@@ -44,6 +43,27 @@ struct  kik_utmp
         char  ut_pos[4];
 };
 
+/* --- static functions --- */
+
+static const char *
+get_pw_name(void)
+{
+  struct passwd * pwent;
+
+  if (
+      ((pwent = getpwuid(getuid())) == NULL) ||
+      (pwent->pw_name == NULL)
+     )
+  {
+    return "?" ;
+  }
+  else
+  {
+    return pwent->pw_name ;
+  }
+}
+
+
 /* --- global functions --- */
 
 kik_utmp_t
@@ -53,40 +73,25 @@ kik_utmp_new(
 	int  pty_fd
 	)
 {
-#ifdef UTMPX
+#ifdef USE_UTMPX
   struct utmpx	ut;
 #else
   struct utmp	ut;
 #endif
-
   kik_utmp_t      utmp;
-  struct passwd * pwent;
-  char          * pw_name;
+  const char    * pw_name;
   char          * tty_num;
-
   struct timeval timenow;
 
   gettimeofday(&timenow, NULL);
 
-  if ( (utmp = malloc(sizeof(*utmp))) == NULL )
+  if ( (utmp = calloc(1, sizeof(*utmp))) == NULL )
   {
     return NULL;
   }
 
   memset( &ut, 0, sizeof(ut) );
 
-  if (
-      ((pwent = getpwuid(getuid())) == NULL) ||
-      (pwent->pw_name == NULL)
-     )
-  {
-    pw_name = "?" ;
-  }
-  else
-  {
-    pw_name = pwent->pw_name ;
-  }
-
   if( (strncmp(tty, "/dev/", 5)) == 0)
   {
     /* skip /dev/ prefix */
@@ -120,6 +125,7 @@ kik_utmp_new(
     return NULL;
   }
 
+  pw_name = get_pw_name();
   strncpy( ut.ut_user, pw_name, K_MIN(sizeof(ut.ut_user), strlen(pw_name)) );
   memcpy( ut.ut_id, tty_num, K_MIN(sizeof(ut.ut_id), strlen(tty_num)) );
   memcpy( ut.ut_line, tty, K_MIN(sizeof(ut.ut_line), strlen(tty)) );
@@ -127,16 +133,18 @@ kik_utmp_new(
   ut.ut_pid	= getpid();
   ut.ut_type	= USER_PROCESS;
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
   ut.ut_tv.tv_sec	= timenow.tv_sec;
   ut.ut_tv.tv_usec	= timenow.tv_usec;
 #else
   ut.ut_time	= time(NULL);
 #endif
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
   memcpy(ut.ut_host, host, K_MIN(sizeof(ut.ut_host), strlen(host)));
+#if ! defined(__FreeBSD__) && ! defined(__APPLE__)
   ut.ut_session = getsid(0) ;
+#endif
 #endif
 
   memcpy( utmp->ut_line, tty,
@@ -148,18 +156,24 @@ kik_utmp_new(
   kik_priv_restore_egid();
 
   /* reset the input stream to the beginning of the file */
-#ifdef UTMPX
+#ifdef USE_UTMPX
   setutxent();
 #else
   setutent();
 #endif
 
   /* insert new entry */
-#ifdef UTMPX
+#ifdef USE_UTMPX
   if ( !pututxline(&ut) )
   {
-    free(utmp);
-    utmp = NULL;
+#ifdef _PATH_UTMP_UPDATE
+    /* NetBSD calls waitpid() for utmp_update which is executed as root(SUID) and fails(ECHILD). */
+    if (errno != ECHILD)
+#endif
+    {
+      free(utmp);
+      utmp = NULL;
+    }
   }
 #else
   /* pututline doesn't return value in XPG2 and SVID2 */
@@ -175,7 +189,7 @@ kik_utmp_new(
   }
 #endif
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
   endutxent();
 #else
   endutent();
@@ -192,24 +206,26 @@ kik_utmp_delete(
 		kik_utmp_t utmp
 	       )
 {
-#ifdef UTMPX
+#ifdef USE_UTMPX
   struct utmpx	ut;
 #else
   struct utmp	ut;
 #endif
+  const char *  pw_name;
 
   memset( &ut, 0, sizeof(ut) );
 
+  pw_name = get_pw_name();
+  strncpy( ut.ut_user, pw_name, K_MIN(sizeof(ut.ut_user), strlen(pw_name)));
   memcpy( ut.ut_id, utmp->ut_pos,
 	  K_MIN(sizeof(ut.ut_id), sizeof(utmp->ut_pos)) );
   memcpy( ut.ut_line, utmp->ut_line,
 	  K_MIN(sizeof(ut.ut_line), sizeof(utmp->ut_line)) );
-  memset( ut.ut_user, 0, sizeof(ut.ut_user));
 
   ut.ut_pid	= getpid();
   ut.ut_type	= DEAD_PROCESS;
 
-#ifdef UTMPX
+#ifdef USE_UTMPX
   ut.ut_tv.tv_sec  = 0;
   ut.ut_tv.tv_usec = 0;
 #else
@@ -220,7 +236,7 @@ kik_utmp_delete(
   kik_priv_restore_egid();
 
   /* reset the input stream to the beginning of the file */
-#ifdef UTMPX
+#ifdef USE_UTMPX
   setutxent();
   pututxline(&ut);
   endutxent();
