@@ -26,6 +26,8 @@
 #endif
 
 #ifdef  USE_WIN32API
+#include  <stdlib.h>	/* _fmode */
+#include  <fcntl.h>	/* _O_BINARY */
 static char *  dummy_argv[] = { "mlterm" , NULL } ;
 #define  argv  (__argv ? __argv : dummy_argv)
 #define  argc  __argc
@@ -155,6 +157,10 @@ main(
 	main_tid = GetCurrentThreadId() ;
 #endif
 
+#ifdef  USE_WIN32API
+	_fmode = _O_BINARY ;
+#endif
+
 	check_console() ;
 
 	/* normal user */
@@ -163,6 +169,15 @@ main(
 
 	kik_set_sys_conf_dir( CONFIG_PATH) ;
 
+#if  (defined(__NetBSD__) || defined(__OpenBSD__)) && defined(USE_FRAMEBUFFER)
+       /*
+	* XXX
+	* It performs well to read as large amount of data as possible
+	* on framebuffer on old machines.
+	*/
+	ml_set_timeout_read_pty( 0xffff) ;	/* 65535 sec */
+#endif
+
 	if( ! main_loop_init( argc , argv))
 	{
 	#ifdef  DEBUG
