@@ -15,7 +15,7 @@
 #include  <x_font.h>	/* x_use_cp932_ucs_fot_xft */
 #include  <x_screen_manager.h>
 #include  <x_event_source.h>
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 #include  <xlib/x_xim.h>
 #endif
 
@@ -123,7 +123,7 @@ main_loop_init(
 	x_main_config_t  main_config ;
 	kik_conf_t *  conf ;
 	char *  value ;
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	int  use_xim ;
 #endif
 	u_int  max_screens_multiple ;
@@ -166,18 +166,16 @@ main_loop_init(
 		"show version message") ;
 	kik_conf_add_opt( conf , 'R' , "fsrange" , 0 , "font_size_range" , 
 		"font size range for GUI configurator [6-30]") ;
-#if  ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	kik_conf_add_opt( conf , 'Y' , "decsp" , 1 , "compose_dec_special_font" ,
 		"compose dec special font [false]") ;
+	kik_conf_add_opt( conf , 'i' , "xim" , 1 , "use_xim" ,
+		"use XIM (X Input Method) [true]") ;
 #endif
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	kik_conf_add_opt( conf , 'c' , "cp932" , 1 , "use_cp932_ucs_for_xft" , 
 		"use CP932-Unicode mapping table for JISX0208 [false]") ;
 #endif
-#if  ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
-	kik_conf_add_opt( conf , 'i' , "xim" , 1 , "use_xim" , 
-		"use XIM (X Input Method) [true]") ;
-#endif
 #ifndef  USE_WIN32API
 	kik_conf_add_opt( conf , 'j' , "daemon" , 0 , "daemon_mode" ,
 	#ifdef  USE_WIN32GUI
@@ -196,10 +194,8 @@ main_loop_init(
 	kik_conf_add_opt( conf , '\0' , "keepalive" , 0 , "ssh_keepalive_interval" ,
 		"interval seconds to send keepalive. [0 = not send]") ;
 #endif
-	kik_conf_add_opt( conf , '\0' , "metaprefix" , 0 , "mod_meta_prefix" ,
-		"prefix characters in pressing meta key if mod_meta_mode = esc") ;
 	kik_conf_add_opt( conf , '\0' , "deffont" , 0 , "default_font" , "default font") ;
-#ifndef  USE_FRAMEBUFFER
+#ifdef  SUPPORT_POINT_SIZE_FONT
 	kik_conf_add_opt( conf , '\0' , "point" , 1 , "use_point_size" ,
 		"treat fontsize as point instead of pixel") ;
 #endif
@@ -264,7 +260,7 @@ main_loop_init(
 	}
 #endif
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	use_xim = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_xim")))
@@ -276,9 +272,7 @@ main_loop_init(
 	}
 
 	x_xim_init( use_xim) ;
-#endif
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	if( ( value = kik_conf_get_value( conf , "compose_dec_special_font")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -390,11 +384,6 @@ main_loop_init(
 #endif
 #endif
 
-	if( ( value = kik_conf_get_value( conf , "mod_meta_prefix")))
-	{
-		x_set_mod_meta_prefix( kik_str_unescape( value)) ;
-	}
-
 	x_main_config_init( &main_config , conf , argc , argv) ;
 
 	if( ( value = kik_conf_get_value( conf , "default_font")))
@@ -407,7 +396,7 @@ main_loop_init(
 			"ISO10646_UCS4_1" , value , 0) ;
 	}
 
-#ifndef  USE_FRAMEBUFFER
+#ifdef  SUPPORT_POINT_SIZE_FONT
 	if( ( value = kik_conf_get_value( conf , "use_point_size")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -448,9 +437,11 @@ main_loop_final(void)
 
 	ml_free_word_separators() ;
 
+	x_free_mod_meta_prefix() ;
+
 	kik_set_msg_log_file_name( NULL) ;
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	x_xim_final() ;
 #endif
 
@@ -490,6 +481,12 @@ main_loop_start(void)
 
 			break ;
 		}
+	#ifdef  USE_CONSOLE
+		else if( ml_get_all_terms( NULL) == 0)
+		{
+			break ;
+		}
+	#endif
 
 		kik_alloca_end_stack_frame() ;
 	}
