@@ -15,10 +15,10 @@
 
 #else
 
-mkf_map_func( "mkf_zh", mkf_map_big5_to_ucs4, 16)
-mkf_map_func( "mkf_zh", mkf_map_hkscs_to_ucs4, 16)
+mkf_map_func( zh, mkf_map_big5_to_ucs4, 16)
+mkf_map_func( zh, mkf_map_hkscs_to_ucs4, 16)
 
-mkf_map_func( "mkf_zh", mkf_map_ucs4_to_big5, 32)
-mkf_map_func( "mkf_zh", mkf_map_ucs4_to_hkscs, 32)
+mkf_map_func( zh, mkf_map_ucs4_to_big5, 32)
+mkf_map_func( zh, mkf_map_ucs4_to_hkscs, 32)
 
 #endif
