@@ -4,7 +4,6 @@
 
 #include  "ml_iscii.h"
 
-#include  <ctype.h>		/* isdigit */
 #include  <kiklib/kik_str.h>	/* kik_snprintf */
 #include  <kiklib/kik_dlfcn.h>
 #include  <kiklib/kik_mem.h>
@@ -48,8 +47,6 @@ struct  ml_isciikey_state
 #include  <table/malayalam.table>
 #include  <table/oriya.table>
 #include  <table/punjabi.table>
-#include  <table/roman.table>
-#include  <table/tamil.table>
 #include  <table/telugu.table>
 #endif
 
@@ -72,7 +69,6 @@ static struct
 	{ iscii_malayalam_table , sizeof( iscii_malayalam_table) / sizeof( struct tabl) , } ,
 	{ iscii_oriya_table , sizeof( iscii_oriya_table) / sizeof( struct tabl) , } ,
 	{ iscii_punjabi_table , sizeof( iscii_punjabi_table) / sizeof( struct tabl) , } ,
-	{ iscii_roman_table , sizeof( iscii_roman_table) / sizeof( struct tabl) , } ,
 	{ iscii_tamil_table , sizeof( iscii_tamil_table) / sizeof( struct tabl) , } ,
 	{ iscii_telugu_table , sizeof( iscii_telugu_table) / sizeof( struct tabl) , } ,
 #else
@@ -127,7 +123,6 @@ static char *  iscii_table_files[] =
 	"ind_malayalam" ,
 	"ind_oriya" ,
 	"ind_punjabi" ,
-	"ind_roman" ,
 	"ind_tamil" ,
 	"ind_telugu" ,
 } ;
@@ -252,19 +247,7 @@ ml_iscii_shape(
 ml_iscii_state_t
 ml_iscii_new(void)
 {
-	ml_iscii_state_t  state ;
-	
-	if( ! ( state = malloc( sizeof( *state))))
-	{
-		return  NULL ;
-	}
-
-	state->num_of_chars_array = NULL ;
-	state->size = 0 ;
-
-	state->has_iscii = 0 ;
-
-	return  state ;
+	return  calloc( 1 , sizeof( struct ml_iscii_state)) ;
 }
 
 int
@@ -288,8 +271,8 @@ ml_iscii(
 	int  dst_pos ;
 	int  src_pos ;
 	u_char *  iscii_buf ;
-	u_int  iscii_buf_len ;
 	u_char *  font_buf ;
+	u_int8_t *  num_of_chars_array ;
 	u_int  font_buf_len ;
 	u_int  prev_font_filled ;
 	u_int  iscii_filled ;
@@ -297,20 +280,18 @@ ml_iscii(
 	mkf_charset_t  prev_cs ;
 	int  has_ucs ;
 
-	iscii_buf_len = src_len * 4 + 1 ;
-	if( ( iscii_buf = alloca( iscii_buf_len)) == NULL)
+	if( ( iscii_buf = alloca( src_len * MAX_COMB_SIZE + 1)) == NULL)
 	{
 		return  0 ;
 	}
 
-	font_buf_len = src_len * 4 + 1 ;
+	font_buf_len = src_len * MAX_COMB_SIZE + 1 ;
 	if( ( font_buf = alloca( font_buf_len)) == NULL)
 	{
 		return  0 ;
 	}
 
-	if( ( state->num_of_chars_array = realloc( state->num_of_chars_array ,
-						font_buf_len * sizeof(u_int8_t))) == NULL)
+	if( ( num_of_chars_array = alloca( font_buf_len * sizeof(u_int8_t))) == NULL)
 	{
 		return  0 ;
 	}
@@ -337,24 +318,29 @@ ml_iscii(
 			u_int  num ;
 
 			iscii_buf[iscii_filled ++] = ml_char_code( src + src_pos) ;
-			if( ( comb = ml_get_combining_chars( src + src_pos , &num)))
+			comb = ml_get_combining_chars( src + src_pos , &num) ;
+			for( ; num > 0 ; num--)
 			{
-				for( count = 0 ; count < num ; count++)
-				{
-					iscii_buf[iscii_filled ++] = ml_char_code( comb + count) ;
-				}
+				iscii_buf[iscii_filled ++] = ml_char_code( comb++) ;
 			}
 			iscii_buf[iscii_filled] = '\0' ;
+
 			font_filled = ml_iscii_shape( cs , font_buf , font_buf_len , iscii_buf) ;
 
 			if( font_filled < prev_font_filled)
 			{
-				dst_pos -= (prev_font_filled - font_filled) ;
-
-				for( count = 1 ; count <= prev_font_filled - font_filled ; count++)
+				if( font_filled == 0)
 				{
-					state->num_of_chars_array[dst_pos] +=
-						state->num_of_chars_array[dst_pos + count] ;
+					return  0 ;
+				}
+
+				count = prev_font_filled - font_filled ;
+				dst_pos -= count ;
+
+				for( ; count > 0 ; count--)
+				{
+					num_of_chars_array[dst_pos] +=
+						num_of_chars_array[dst_pos + count] ;
 				}
 
 				prev_font_filled = font_filled ; /* goto to the next if block */
@@ -362,15 +348,16 @@ ml_iscii(
 
 			if( dst_pos >= 0 && font_filled == prev_font_filled)
 			{
-				state->num_of_chars_array[dst_pos] ++ ;
+				num_of_chars_array[dst_pos] ++ ;
 			}
 			else
 			{
-				state->num_of_chars_array[++dst_pos] = 1 ;
+				num_of_chars_array[++dst_pos] = 1 ;
 
-				for( count = 1 ; count < font_filled - prev_font_filled ; count++)
+				for( count = font_filled - prev_font_filled ;
+				     count > 1 ; count--)
 				{
-					state->num_of_chars_array[++dst_pos] = 0 ;
+					num_of_chars_array[++dst_pos] = 0 ;
 				}
 			}
 
@@ -385,11 +372,36 @@ ml_iscii(
 				has_ucs = 1 ;
 			}
 
-			state->num_of_chars_array[++dst_pos] = 1 ;
+			num_of_chars_array[++dst_pos] = 1 ;
 		}
 	}
 
-	state->size = dst_pos + 1 ;
+	if( state->size != dst_pos + 1)
+	{
+		void *  p ;
+
+		if( ! ( p = realloc( state->num_of_chars_array ,
+				K_MAX(dst_pos + 1,src_len) * sizeof(*num_of_chars_array))))
+		{
+			return  0 ;
+		}
+
+	#ifdef  __DEBUG
+		if( p != state->num_of_chars_array)
+		{
+			kik_debug_printf( KIK_DEBUG_TAG
+				" REALLOC array %d(%p) -> %d(%p)\n" ,
+				state->size , state->num_of_chars_array ,
+				dst_pos + 1 , p) ;
+		}
+	#endif
+
+		state->num_of_chars_array = p ;
+		state->size = dst_pos + 1 ;
+	}
+
+	memcpy( state->num_of_chars_array , num_of_chars_array ,
+		state->size * sizeof(*num_of_chars_array)) ;
 
 	return  (! state->has_iscii && has_ucs) ? -1 : 1 ;
 }
@@ -397,12 +409,19 @@ ml_iscii(
 int
 ml_iscii_copy(
 	ml_iscii_state_t  dst ,
-	ml_iscii_state_t  src
+	ml_iscii_state_t  src ,
+	int  optimize
 	)
 {
 	u_int8_t *  p ;
 
-	if( src->size == 0)
+	if( optimize && ! src->has_iscii)
+	{
+		ml_iscii_delete( dst) ;
+
+		return  -1 ;
+	}
+	else if( src->size == 0)
 	{
 		free( dst->num_of_chars_array) ;
 		p = NULL ;
