@@ -16,10 +16,10 @@
 
 #define  IS_UNICODE_AREA_CS(attr)  ((attr) & (0x1 << 17))
 
-/* Combination of UNICODE_AREA, IS_ITALIC, IS_BOLD, IS_FULLWIDTH and CHARSET */
+/* Combination of IS_ITALIC, IS_BOLD, IS_FULLWIDTH, CS_REVISION_1 and CHARSET(UNICODE AREA) */
 #define  MLFONT(attr) \
 	IS_UNICODE_AREA_CS(attr) ? \
-	((((attr) >> 5) & 0xe00) | ISO10646_UCS4_1 | (((attr) << 7) & 0x1ff000)) : \
+	((((attr) >> 5) & 0xf00) | ISO10646_UCS4_1 | (((attr) << 7) & 0xff000)) : \
 	(((attr) >> 5) & 0xfff)
 
 #define  IS_VISIBLE(attr)  ((attr) & (0x1 << 19))
@@ -29,7 +29,7 @@
 #define  IS_FULLWIDTH(attr)  ((attr) & (0x1 << 14))
 #define  CHARSET(attr) \
 	IS_UNICODE_AREA_CS(attr) ? \
-	ISO10646_UCS4_1 : \
+	(ISO10646_UCS4_1 | (((attr) >> 5) & 0x100)) : \
 	(((attr) >> 5) & 0x1ff)
 
 #define  IS_REVERSED(attr)  ((attr) & (0x1 << 4))
@@ -66,6 +66,8 @@ static struct
 
 } *  unicode_areas ;
 static u_int  num_of_unicode_areas ;
+static u_int  unicode_area_min ;
+static u_int  unicode_area_max ;
 
 
 /* --- static functions --- */
@@ -87,6 +89,140 @@ get_comb_size(
 	return  size ;
 }
 
+static ml_char_t *
+new_comb(
+	ml_char_t *  ch ,
+	u_int *  comb_size_ptr
+	)
+{
+	ml_char_t *  multi_ch ;
+	u_int  comb_size ;
+
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		if( IS_ZEROWIDTH(ch->u.ch.attr))
+		{
+			/*
+			 * Zero width characters must not be combined to
+			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
+			 */
+			return  NULL ;
+		}
+
+		if( ( multi_ch = malloc( sizeof( ml_char_t) * 2)) == NULL)
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
+		#endif
+
+			return  NULL ;
+		}
+
+	#ifndef  __GLIBC__
+		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
+		{
+			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
+			                "Character combining is not supported.\n") ;
+
+			return  NULL ;
+		}
+	#endif
+
+		ml_char_init( multi_ch) ;
+		ml_char_copy( multi_ch , ch) ;
+
+		comb_size = 1 ;
+
+		if( comb_size > 10)
+		{
+			abort() ;
+		}
+	}
+	else
+	{
+		if( IS_ZEROWIDTH(ch->u.multi_ch->u.ch.attr))
+		{
+			/*
+			 * Zero width characters must not be combined to
+			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
+			 */
+			return  NULL ;
+		}
+
+		if( ( comb_size = get_comb_size( ch->u.multi_ch)) >= MAX_COMB_SIZE)
+		{
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG
+			" This char is already combined by %d chars, so no more combined.\n",
+			comb_size) ;
+		#endif
+
+			return  NULL ;
+		}
+
+		if( ( multi_ch = realloc( ch->u.multi_ch ,
+			sizeof( ml_char_t) * (comb_size + 2))) == NULL)
+		{
+			return  NULL ;
+		}
+
+	#ifndef  __GLIBC__
+		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
+		{
+			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
+			                "Character combining is not supported.\n") ;
+
+			return  0 ;
+		}
+	#endif
+
+		comb_size ++ ;
+	}
+
+	SET_COMB_TRAILING( (multi_ch[comb_size - 1]).u.ch.attr) ;
+
+	ch->u.multi_ch = multi_ch ;
+	USE_MULTI_CH(ch->u.ch.attr) ;	/* necessary for 64bit big endian */
+
+	*comb_size_ptr = comb_size ;
+
+	return  multi_ch + comb_size ;
+}
+
+#ifdef  USE_NORMALIZE
+#include  <kiklib/kik_mem.h>
+int  ml_normalize( u_int16_t *  str , int  num) ;
+
+static void
+normalize(
+	ml_char_t *  ch ,
+	u_int  comb_size
+	)
+{
+	u_int16_t *  str ;
+
+	if( ( str = alloca( sizeof(*str) * (comb_size + 1))))
+	{
+		ml_char_t *  multi_ch ;
+		u_int  count ;
+
+		multi_ch = ch->u.multi_ch ;
+		for( count = 0 ; count < comb_size + 1 ; count++)
+		{
+			str[count] = ml_char_code( multi_ch + count) ;
+		}
+
+		if( ml_normalize( str , comb_size + 1) == 1)
+		{
+			*ch = *multi_ch ;
+			ch->u.ch.code = str[0] ;
+			UNSET_COMB_TRAILING(ch->u.ch.attr) ;
+			free( multi_ch) ;
+		}
+	}
+}
+#endif
+
 
 /* --- global functions --- */
 
@@ -102,8 +238,8 @@ ml_set_use_multi_col_char(
 
 ml_font_t
 ml_char_get_unicode_area_font(
-	u_int32_t  min ,
-	u_int32_t  max
+	u_int32_t  min ,	/* min <= max */
+	u_int32_t  max		/* min <= max */
 	)
 {
 	u_int  idx ;
@@ -118,7 +254,7 @@ ml_char_get_unicode_area_font(
 		}
 	}
 
-	if( num_of_unicode_areas == 511 /* Max is 2^9-1 */ ||
+	if( num_of_unicode_areas == 255 /* Max is 2^8-1 */ ||
 	    ! ( p = realloc( unicode_areas ,
 			sizeof(*unicode_areas) * (num_of_unicode_areas + 1))))
 	{
@@ -127,6 +263,24 @@ ml_char_get_unicode_area_font(
 		return  UNKNOWN_CS ;
 	}
 
+	if( num_of_unicode_areas == 0)
+	{
+		unicode_area_min = min ;
+		unicode_area_max = max ;
+	}
+	else
+	{
+		if( unicode_area_min > min)
+		{
+			unicode_area_min = min ;
+		}
+
+		if( unicode_area_max < max)
+		{
+			unicode_area_max = max ;
+		}
+	}
+
 	unicode_areas = p ;
 	unicode_areas[num_of_unicode_areas].min = min ;
 	unicode_areas[num_of_unicode_areas++].max = max ;
@@ -197,8 +351,12 @@ ml_char_set(
 
 	ch->u.ch.code = code ;
 
-	if( unicode_areas && cs == ISO10646_UCS4_1)
+	if( code <= unicode_area_max && cs == ISO10646_UCS4_1 && unicode_area_min <= code)
 	{
+		/*
+		 * If code == 0, unicode_area_max == 0 and unicode_area_min == 0,
+		 * enter this block unexpectedly, but harmless.
+		 */
 		for( idx = num_of_unicode_areas ; idx > 0 ; idx --)
 		{
 			if( unicode_areas[idx - 1].min <= code &&
@@ -310,7 +468,7 @@ ml_char_reverse_attr(
 	}
 }
 
-int
+ml_char_t *
 ml_char_combine(
 	ml_char_t *  ch ,
 	u_int32_t  code ,
@@ -326,7 +484,8 @@ ml_char_combine(
 	int  is_blinking
 	)
 {
-	ml_char_t *  multi_ch ;
+	ml_char_t *  comb ;
+	u_int  comb_size ;
 
 	/*
 	 * This check should be excluded, because characters whose is_comb flag
@@ -336,120 +495,71 @@ ml_char_combine(
 #if  0
 	if( ! is_comb)
 	{
-		return  0 ;
+		return  NULL ;
 	}
 #endif
 
-	if( IS_SINGLE_CH(ch->u.ch.attr))
+#ifdef  DEBUG
+	if( ! IS_ISCII(ml_char_cs( ch)) && IS_ISCII(cs))
 	{
-		if( IS_ZEROWIDTH(ch->u.ch.attr))
-		{
-			/*
-			 * Zero width characters must not be combined to
-			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
-			 */
-			return  0 ;
-		}
-
-		if( ( multi_ch = malloc( sizeof( ml_char_t) * 2)) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
-		#endif
-
-			return  0 ;
-		}
-		
-#if !defined(__GLIBC__)
-		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
-		{
-			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
-			                "Character combining is not supported.\n") ;
-
-			return  0 ;
-		}
+		kik_debug_printf( KIK_DEBUG_TAG " combining iscii char to non-iscii char.\n") ;
+	}
 #endif
 
-		ml_char_init( multi_ch) ;
-		ml_char_copy( multi_ch , ch) ;
-		SET_COMB_TRAILING( multi_ch->u.ch.attr) ;
-
-		ml_char_init( multi_ch + 1) ;
-		if( ml_char_set( multi_ch + 1 , code , cs , is_fullwidth , is_comb ,
-			fg_color , bg_color , is_bold , is_italic , underline_style ,
-			is_crossed_out , is_blinking) == 0)
-		{
-			return  0 ;
-		}
-	}
-	else
+	if( ! ( comb = new_comb( ch , &comb_size)))
 	{
-		u_int  comb_size ;
-
-		if( IS_ZEROWIDTH(ch->u.multi_ch->u.ch.attr))
-		{
-			/*
-			 * Zero width characters must not be combined to
-			 * show string like U+09b0 + U+200c + U+09cd + U+09af correctly.
-			 */
-			return  0 ;
-		}
-
-		if( ( comb_size = get_comb_size( ch->u.multi_ch)) >= MAX_COMB_SIZE)
-		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG
-			" This char is already combined by %d chars, so no more combined.\n",
-			comb_size) ;
-		#endif
-		
-			return  0 ;
-		}
-		
-		if( ( multi_ch = realloc( ch->u.multi_ch ,
-			sizeof( ml_char_t) * (comb_size + 2))) == NULL)
-		{
-			return  0 ;
-		}
-
-	#ifndef  __GLIBC__
-		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
-		{
-			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
-			                "Character combining is not supported.\n") ;
-
-			return  0 ;
-		}
-	#endif
-
-		SET_COMB_TRAILING( multi_ch[comb_size].u.ch.attr) ;
-		ml_char_init( multi_ch + comb_size + 1) ;
-		if( ml_char_set( multi_ch + comb_size + 1 , code , cs , is_fullwidth ,
-			is_comb , fg_color , bg_color , is_bold , is_italic , underline_style ,
-			is_crossed_out , is_blinking) == 0)
-		{
-			return  0 ;
-		}
+		return  NULL ;
 	}
 
-	ch->u.multi_ch = multi_ch ;
-	USE_MULTI_CH(ch->u.ch.attr) ;	/* necessary for 64bit big endian */
+	ml_char_init( comb) ;
+	if( ml_char_set( comb , code , cs , is_fullwidth , is_comb ,
+		fg_color , bg_color , is_bold , is_italic , underline_style ,
+		is_crossed_out , is_blinking) == 0)
+	{
+		return  NULL ;
+	}
 
-	return  1 ;
+#ifdef  USE_NORMALIZE
+	normalize( ch , comb_size) ;
+#endif
+
+	return  comb ;
 }
 
-int
+ml_char_t *
 ml_char_combine_simple(
 	ml_char_t *  ch ,
-	ml_char_t *  comb
+	ml_char_t *  src
 	)
 {
-	return  ml_char_combine( ch , ml_char_code( comb) , CHARSET(comb->u.ch.attr) ,
-			IS_FULLWIDTH(comb->u.ch.attr) , IS_COMB(comb->u.ch.attr) ,
-			comb->u.ch.fg_color , comb->u.ch.bg_color ,
-			IS_BOLD(comb->u.ch.attr) , IS_ITALIC(comb->u.ch.attr) ,
-			UNDERLINE_STYLE(comb->u.ch.attr) , IS_CROSSED_OUT(comb->u.ch.attr) ,
-			IS_BLINKING(comb->u.ch.attr)) ;
+	ml_char_t *  comb ;
+	u_int  comb_size ;
+
+	/*
+	 * This check should be excluded, because characters whose is_comb flag
+	 * (combining property of mkf) is NULL can be combined
+	 * if ml_is_arabic_combining(them) returns non-NULL.
+	 */
+#if  0
+	if( ! is_comb)
+	{
+		return  NULL ;
+	}
+#endif
+
+	if( ! ( comb = new_comb( ch , &comb_size)))
+	{
+		return  NULL ;
+	}
+
+	*comb = *src ;
+	UNSET_COMB_TRAILING(comb->u.ch.attr) ;
+
+#ifdef  USE_NORMALIZE
+	normalize( ch , comb_size) ;
+#endif
+
+	return  comb ;
 }
 
 ml_char_t *
@@ -622,6 +732,39 @@ ml_char_cs(
 	}
 }
 
+int
+ml_char_set_cs(
+	ml_char_t *  ch ,
+	mkf_charset_t  cs
+	)
+{
+	if( IS_SINGLE_CH(ch->u.ch.attr))
+	{
+		if( IS_UNICODE_AREA_CS(ch->u.ch.attr))
+		{
+			if( cs == ISO10646_UCS4_1_V)
+			{
+				ch->u.ch.attr |= (0x100 << 5) ;
+			}
+			else /* if( cs == ISO10646_UCS4_1) */
+			{
+				ch->u.ch.attr &= ~(0x100 << 5) ;
+			}
+		}
+		else
+		{
+			ch->u.ch.attr &= ~(MAX_CHARSET << 5) ;	/* ~0x1ff (ml_font.h) */
+			ch->u.ch.attr |= (cs << 5) ;
+		}
+	}
+	else
+	{
+		ml_char_set_cs( ch->u.multi_ch , cs) ;
+	}
+
+	return  1 ;
+}
+
 ml_font_t
 ml_char_font(
 	ml_char_t *  ch
@@ -750,6 +893,14 @@ ml_char_set_fg_color(
 		comb_size = get_comb_size( ch->u.multi_ch) ;
 		for( count = 0 ; count < comb_size + 1 ; count ++)
 		{
+		#ifdef  DEBUG
+			if( CHARSET(ch->u.multi_ch[count].u.ch.attr) == ISO10646_UCS4_1_V)
+			{
+				kik_debug_printf( KIK_DEBUG_TAG " Don't change bg_color"
+					" of ISO10646_UCS4_1_V char after shaped.\n") ;
+			}
+		#endif
+
 			ml_char_set_fg_color( ch->u.multi_ch + count , color) ;
 		}
 	}
@@ -790,6 +941,14 @@ ml_char_set_bg_color(
 		comb_size = get_comb_size( ch->u.multi_ch) ;
 		for( count = 0 ; count < comb_size + 1 ; count ++)
 		{
+		#ifdef  DEBUG
+			if( CHARSET(ch->u.multi_ch[count].u.ch.attr) == ISO10646_UCS4_1_V)
+			{
+				kik_debug_printf( KIK_DEBUG_TAG " Don't change bg_color"
+					" of ISO10646_UCS4_1_V char after shaped.\n") ;
+			}
+		#endif
+
 			ml_char_set_bg_color( ch->u.multi_ch + count , color) ;
 		}
 	}
@@ -797,6 +956,90 @@ ml_char_set_bg_color(
 	return  1 ;
 }
 
+int
+ml_char_get_offset(
+	ml_char_t *  ch
+	)
+{
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr) && CHARSET(attr) == ISO10646_UCS4_1_V)
+	{
+		int8_t  offset ;
+
+		offset = ch->u.ch.bg_color ;	/* unsigned -> signed */
+
+		return  offset ;
+	}
+	else
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " ml_char_get_position() accepts "
+			"ISO10646_UCS4_1_V char alone, but received %x charset.\n" ,
+			ml_char_cs( ch)) ;
+	#endif
+
+		return  0 ;
+	}
+}
+
+u_int
+ml_char_get_width(
+	ml_char_t *  ch
+	)
+{
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr) && CHARSET(attr) == ISO10646_UCS4_1_V)
+	{
+		return  ch->u.ch.fg_color ;
+	}
+	else
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " ml_char_get_width() accepts "
+			"ISO10646_UCS4_1_V char alone, but received %x charset.\n" ,
+			ml_char_cs( ch)) ;
+	#endif
+
+		return  0 ;
+	}
+}
+
+int
+ml_char_set_position(
+	ml_char_t *  ch ,
+	u_int8_t  offset ,	/* signed -> unsigned */
+	u_int8_t  width
+	)
+{
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr) && CHARSET(attr) == ISO10646_UCS4_1_V)
+	{
+		ch->u.ch.bg_color = offset ;
+		ch->u.ch.fg_color = width ;
+
+		return  1 ;
+	}
+	else
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " ml_char_set_position() accepts "
+			"ISO10646_UCS4_1_V char alone, but received %x charset.\n" ,
+			ml_char_cs( ch)) ;
+	#endif
+
+		return  0 ;
+	}
+}
+
 int
 ml_char_underline_style(
 	ml_char_t *  ch
@@ -1111,14 +1354,10 @@ ml_char_dump(
 	}
 #endif
 
-	if( ( comb_chars = ml_get_combining_chars( ch , &comb_size)) != NULL)
+	comb_chars = ml_get_combining_chars( ch , &comb_size) ;
+	for( ; comb_size > 0 ; comb_size --)
 	{
-		int  count ;
-
-		for( count = 0 ; count < comb_size ; count ++)
-		{
-			ml_char_dump( &comb_chars[count]) ;
-		}
+		ml_char_dump( comb_chars++) ;
 	}
 }
 
