@@ -14,8 +14,8 @@
 #include  "ml_color.h"
 
 
-#define  MAX_COMB_SIZE	7	/* Used in ml_shape.c,x_screen.c */
-#define  UTF_MAX_SIZE	6
+#define  MAX_COMB_SIZE	7	/* Used in ml_shape.c,x_screen.c,ml_screen.c */
+#define  UTF_MAX_SIZE	6	/* see skk/dict.c */
 /*
  * XXX
  * char prefixes are max 4 bytes.
@@ -70,7 +70,9 @@ typedef struct ml_char
 			 * 1 bit : is_italic(0 or 1)
 			 * 1 bit : is_bold(0 or 1)
 			 * 1 bit : is_fullwidth(0 or 1)
-			 * 9 bit : charset(0x0 - 0x1ff)
+			 * 9 bit : charset(0x0 - 0x1ff) or
+			 *         1 bit: CS_REVISION_1(ISO10646_UCS4_1_V)
+			 *         8 bit: unicode area id
 			 * 1 bit : is_reversed(0 or 1)	... used for X Selection
 			 * 1 bit : is_crossed_out
 			 * 1 bit : is_comb(0 or 1)
@@ -121,7 +123,7 @@ void  ml_char_change_attr( ml_char_t *  ch , int  is_bold , int  is_underlined ,
 void  ml_char_reverse_attr( ml_char_t *  ch , int  bold , int  underlined ,
 	int  blinking , int  reversed) ;
 
-int  ml_char_combine( ml_char_t *  ch , u_int32_t  code , mkf_charset_t  cs ,
+ml_char_t *  ml_char_combine( ml_char_t *  ch , u_int32_t  code , mkf_charset_t  cs ,
 	int  is_fullwidth , int  is_comb , ml_color_t  fg_color , ml_color_t  bg_color ,
 	int  is_bold , int  is_italic , int  underline_style , int  is_crossed_out ,
 	int  is_blinking) ;
@@ -130,7 +132,7 @@ int  ml_char_combine( ml_char_t *  ch , u_int32_t  code , mkf_charset_t  cs ,
 #define  ml_char_combine_picture( ch , id , pos) \
 	ml_char_combine( ch , pos , PICTURE_CHARSET , 0 , 0 ,  id , id , 0 , 0 , 0 , 0 , 0)
 
-int  ml_char_combine_simple( ml_char_t *  ch , ml_char_t *  comb) ;
+ml_char_t *  ml_char_combine_simple( ml_char_t *  ch , ml_char_t *  comb) ;
 
 ml_char_t *  ml_get_base_char( ml_char_t *  ch) ;
 
@@ -153,6 +155,8 @@ int  ml_char_set_code( ml_char_t *  ch , u_int32_t  code) ;
 
 mkf_charset_t  ml_char_cs( ml_char_t *  ch) ;
 
+int  ml_char_set_cs( ml_char_t *  ch , mkf_charset_t  cs) ;
+
 int  ml_char_is_comb( ml_char_t *  ch) ;
 
 ml_font_t  ml_char_font( ml_char_t *  ch) ;
@@ -172,6 +176,12 @@ ml_color_t  ml_char_bg_color( ml_char_t *  ch) ;
 
 int  ml_char_set_bg_color( ml_char_t *  ch , ml_color_t  color) ;
 
+int  ml_char_get_offset( ml_char_t *  ch) ;
+
+u_int  ml_char_get_width( ml_char_t *  ch) ;
+
+int  ml_char_set_position( ml_char_t *  ch , u_int8_t  offset , u_int8_t  width) ;
+
 int  ml_char_underline_style( ml_char_t *  ch) ;
 
 int  ml_char_is_crossed_out( ml_char_t *  ch) ;
