@@ -314,10 +314,64 @@ static u_int8_t color256_rgb_table[][3] =
 
 static char *  color_file = "mlterm/color" ;
 static KIK_MAP( color_rgb)  color_config ;
+static u_int  num_of_changed_256_colors ;
+
+static struct
+{
+	u_int  is_changed: 1 ;
+	u_int  mark: 7 ;
+	u_int8_t  red ;
+	u_int8_t  green ;
+	u_int8_t  blue ;
+
+} *  ext_color_table ;
+static u_int  ext_color_mark = 1 ;
+
+static int  color_distance_threshold = COLOR_DISTANCE_THRESHOLD ;
+static int  use_pseudo_color = 0 ;
 
 
 /* --- static functions --- */
 
+static void
+get_default_rgb(
+	ml_color_t  color ,	/* is 255 or less */
+	u_int8_t *  red ,
+	u_int8_t *  green ,
+	u_int8_t *  blue
+	)
+{
+	if( IS_VTSYS_COLOR(color))
+	{
+		*red = vtsys_color_rgb_table[ color][0] ;
+		*green = vtsys_color_rgb_table[ color][1] ;
+		*blue = vtsys_color_rgb_table[ color][2] ;
+	}
+	else if( color <= 0xe7)
+	{
+		u_int8_t  tmp ;
+
+		tmp = (color - 0x10) % 6 ;
+		*blue = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+
+		tmp = ((color - 0x10) / 6) % 6 ;
+		*green = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+
+		tmp = ((color - 0x10) / 36) % 6 ;
+		*red = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+	}
+	else /* if( color >= 0xe8) */
+	{
+		u_int8_t  tmp ;
+
+		tmp = (color - 0xe8) * 10 + 8 ;
+
+		*blue = tmp ;
+		*green = tmp ;
+		*red = tmp ;
+	}
+}
+
 static KIK_PAIR( color_rgb)
 get_color_rgb_pair(
 	ml_color_t  color
@@ -332,7 +386,7 @@ get_color_rgb_pair(
 
 static int
 color_config_set_rgb(
-	ml_color_t  color ,
+	ml_color_t  color ,	/* is 255 or less */
 	u_int8_t  red ,
 	u_int8_t  green ,
 	u_int8_t  blue ,
@@ -340,6 +394,7 @@ color_config_set_rgb(
 	)
 {
 	KIK_PAIR( color_rgb)  pair ;
+	int  result ;
 	rgb_t  rgb ;
 
 	rgb.red = red ;
@@ -349,21 +404,44 @@ color_config_set_rgb(
 
 	if( ( pair = get_color_rgb_pair( color)))
 	{
+		u_int8_t  r ;
+		u_int8_t  g ;
+		u_int8_t  b ;
+
 		if( pair->value.red == red && pair->value.green == green &&
 			pair->value.blue == blue && pair->value.alpha == alpha)
 		{
 			/* Not changed */
 
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG
+				" color %d rgb(%02x%02x%02x%02x) not changed.\n" ,
+				color , red , green , blue , alpha) ;
+		#endif
+
 			return  0 ;
 		}
 
-		pair->value = rgb ;
+		get_default_rgb( color , &r , &g , &b) ;
+
+		if( r == red && g == green && b == blue && alpha == 0xff)
+		{
+			if( IS_256_COLOR(color))
+			{
+				num_of_changed_256_colors -- ;
+			}
+
+			kik_map_erase_simple( result , color_config , color) ;
+		}
+		else
+		{
+			pair->value = rgb ;
+		}
 
 		return  1 ;
 	}
 	else
 	{
-		int  result ;
 		u_int8_t  r ;
 		u_int8_t  g ;
 		u_int8_t  b ;
@@ -389,7 +467,7 @@ color_config_set_rgb(
 
 			#ifdef  DEBUG
 				kik_debug_printf( KIK_DEBUG_TAG
-					" color %d'rgb(%02x%02x%02x%02x) not changed.\n",
+					" color %d rgb(%02x%02x%02x%02x) not changed.\n",
 					color , red , green , blue , alpha) ;
 			#endif
 
@@ -399,10 +477,12 @@ color_config_set_rgb(
 			else
 			{
 				kik_debug_printf( KIK_DEBUG_TAG
-					" color %d's rgb(%02x%02x%02x) changed => %02x%02x%02x.\n",
+					" color %d rgb(%02x%02x%02x) changed => %02x%02x%02x.\n",
 					color , r , g , b , red , green , blue) ;
 			}
 		#endif
+
+			num_of_changed_256_colors ++ ;
 		}
 
 		kik_map_set( result , color_config , color , rgb) ;
@@ -472,6 +552,11 @@ parse_conf(
 		{
 			int  result ;
 
+			if( IS_256_COLOR(color))
+			{
+				num_of_changed_256_colors -- ;
+			}
+
 			kik_map_erase_simple( result , color_config , color) ;
 
 			return  1 ;
@@ -526,6 +611,47 @@ read_conf(
 
 /* --- global functions --- */
 
+void
+ml_set_color_mode(
+	const char *  mode
+	)
+{
+	if( strcmp( mode , "256") == 0)
+	{
+		color_distance_threshold = COLOR_DISTANCE_THRESHOLD ;
+		use_pseudo_color = 1 ;
+	}
+	else
+	{
+		if( strcmp( mode , "true") == 0)
+		{
+			color_distance_threshold = 40 ; /* 9 + 30 + 1 */
+		}
+		else /* if( strcmp( mode , "high") == 0 */
+		{
+			color_distance_threshold = COLOR_DISTANCE_THRESHOLD ;
+		}
+		use_pseudo_color = 0 ;
+	}
+}
+
+char *
+ml_get_color_mode(void)
+{
+	if( use_pseudo_color)
+	{
+		return  "256" ;
+	}
+	else if( color_distance_threshold == 40)
+	{
+		return  "true" ;
+	}
+	else
+	{
+		return  "high" ;
+	}
+}
+
 int
 ml_color_config_init(void)
 {
@@ -638,7 +764,7 @@ ml_get_color(
 
 	if( sscanf( name, "%d", (int*) &color) == 1)
 	{
-		if( IS_VALID_COLOR_EXCEPT_FG_BG(color))
+		if( IS_VTSYS256_COLOR(color))
 		{
 			return  color ;
 		}
@@ -668,43 +794,30 @@ ml_get_color_rgba(
 	u_int8_t *  alpha	/* can be NULL */
 	)
 {
-	if( ! IS_VALID_COLOR_EXCEPT_FG_BG(color))
+	if( ! IS_VALID_COLOR_EXCEPT_SPECIAL_COLORS(color))
 	{
 		return  0 ;
 	}
 
-	if( color_config && color_config_get_rgb( color , red , green , blue , alpha))
+	if( IS_EXT_COLOR(color))
+	{
+		if( ! ext_color_table ||
+		    ext_color_table[EXT_COLOR_TO_INDEX(color)].mark == 0)
+		{
+			return  0 ;
+		}
+
+		*red = ext_color_table[EXT_COLOR_TO_INDEX(color)].red ;
+		*green = ext_color_table[EXT_COLOR_TO_INDEX(color)].green ;
+		*blue = ext_color_table[EXT_COLOR_TO_INDEX(color)].blue ;
+	}
+	else if( color_config && color_config_get_rgb( color , red , green , blue , alpha))
 	{
 		return  1 ;
 	}
-	else if( IS_VTSYS_COLOR(color))
+	else
 	{
-		*red = vtsys_color_rgb_table[ color][0] ;
-		*green = vtsys_color_rgb_table[ color][1] ;
-		*blue = vtsys_color_rgb_table[ color][2] ;
-	}
-	else if( color <= 0xe7)
-	{
-		u_int8_t  tmp ;
-
-		tmp = (color - 0x10) % 6 ;
-		*blue = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
-
-		tmp = ((color - 0x10) / 6) % 6 ;
-		*green = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
-
-		tmp = ((color - 0x10) / 36) % 6 ;
-		*red = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
-	}
-	else /* if( color >= 0xe8) */
-	{
-		u_int8_t  tmp ;
-
-		tmp = (color - 0xe8) * 10 + 8 ;
-
-		*blue = tmp ;
-		*green = tmp ;
-		*red = tmp ;
+		get_default_rgb( color , red , green , blue) ;
 	}
 
 	if( alpha)
@@ -849,6 +962,101 @@ end:
 	return  1 ;
 }
 
+/*
+ * Return the number of colors which should be searched after this function.
+ */
+u_int
+ml_get_closest_256_color(
+	ml_color_t *  closest ,
+	u_int *  min_diff ,
+	u_int8_t  red ,
+	u_int8_t  green ,
+	u_int8_t  blue ,
+	int  threshold
+	)
+{
+	int  r , g , b ;
+	int  tmp ;
+	ml_color_t  color ;
+	u_int8_t  rgb[3] ;
+	u_int  diff ;
+	int  diff_r , diff_g , diff_b ;
+	int  count ;
+	int  num ;
+
+	if( num_of_changed_256_colors > 0)
+	{
+		kik_debug_printf( "OUCH %d\n" , num_of_changed_256_colors);
+		return  256 ;
+	}
+
+	/*
+	 * 0 - 47 => 0
+	 * 48 - 114 => 1
+	 * 115 - 154 => 2
+	 * ...
+	 */
+	tmp = (red <= 114 ? (red >= 48) : ((red - 55) + 20) / 40) ;
+	r = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+	color = tmp * 36 ;
+	tmp = (green <= 114 ? (green >= 48) : ((green - 55) + 20) / 40) ;
+	g = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+	color += tmp * 6 ;
+	tmp = (blue <= 114 ? (blue >= 48) : ((blue - 55) + 20) / 40) ;
+	b = tmp ? (tmp * 40 + 55) & 0xff : 0 ;
+	color += tmp ;
+	/* lazy color-space conversion */
+	diff_r = red - r ;
+	diff_g = green - g ;
+	diff_b = blue - b ;
+	diff = COLOR_DISTANCE(diff_r,diff_g,diff_b) ;
+	if( diff < *min_diff)
+	{
+		*min_diff = diff ;
+		*closest = color + 0x10 ;
+		if( diff < threshold)
+		{
+			return  0 ;
+		}
+	}
+
+	num = 1 ;
+	rgb[0] = red ;
+
+	if( red != green)
+	{
+		rgb[num++] = green ;
+	}
+
+	if( red != blue && green != blue)
+	{
+		rgb[num++] = blue ;
+	}
+
+	for( count = 0 ; count < num ; count++)
+	{
+		tmp = (rgb[count] >= 233 ?
+			23 : (rgb[count] <= 12 ? 0 : ((rgb[count] - 8) + 5) / 10)) ;
+		r = g = b = tmp * 10 + 8 ;
+		/* lazy color-space conversion */
+		diff_r = red - r ;
+		diff_g = green - g ;
+		diff_b = blue - b ;
+		diff = COLOR_DISTANCE(diff_r,diff_g,diff_b) ;
+		if( diff < *min_diff)
+		{
+			*min_diff = diff ;
+			*closest = tmp + 0xe8 ;
+			if( diff < threshold)
+			{
+				return  0 ;
+			}
+		}
+	}
+
+	return  16 ;
+}
+
 ml_color_t
 ml_get_closest_color(
 	u_int8_t  red ,
@@ -858,9 +1066,27 @@ ml_get_closest_color(
 {
 	ml_color_t  closest = ML_UNKNOWN_COLOR ;
 	ml_color_t  color ;
-	u_long  min = 0xffffff ;
+	u_int  linear_search_max ;
+	u_int  min = 0xffffff ;
+	ml_color_t  oldest_color ;
+	u_int  oldest_mark ;
+	u_int  max = 0 ;
 
-	for( color = 0 ; color < 256 ; color++)
+#ifdef  __DEBUG
+	ml_color_t  hit_closest = ML_UNKNOWN_COLOR ;
+	ml_get_closest_256_color( &hit_closest , &min , red , green , blue) ;
+	min = 0xffffff ;
+
+	for( color = 16 ; color < 256 ; color++)
+#else
+	if( ( linear_search_max = ml_get_closest_256_color( &closest , &min ,
+					red , green , blue , color_distance_threshold)) == 0)
+	{
+		return  closest ;
+	}
+
+	for( color = 0 ; color < linear_search_max ; color++)
+#endif
 	{
 		u_int8_t  r ;
 		u_int8_t  g ;
@@ -869,27 +1095,176 @@ ml_get_closest_color(
 
 		if( ml_get_color_rgba( color , &r , &g , &b , &a) && a == 0xff)
 		{
-			u_long  diff ;
+			u_int  diff ;
 			int  diff_r , diff_g , diff_b ;
 
 			/* lazy color-space conversion */
 			diff_r = red - r ;
 			diff_g = green - g ;
 			diff_b = blue - b ;
-			diff = diff_r * diff_r * 9 + diff_g * diff_g * 30 +
-				diff_b * diff_b ;
+			diff = COLOR_DISTANCE(diff_r,diff_g,diff_b) ;
 			if( diff < min)
 			{
 				min = diff ;
 				closest = color ;
-				/* no one may notice the difference (4[2^3/2]*4*9+4*4*30+4*4) */
-				if( diff < 640)
+				if( diff < color_distance_threshold)
 				{
-					break ;
+				#ifdef  __DEBUG
+					/*
+					 * XXX
+					 * The result of linear search of boundary values
+					 * (115, 195 etc) is different from that of
+					 * ml_get_closest_256_color() of it.
+					 */
+					if( closest != hit_closest)
+					{
+						kik_debug_printf( "ERROR %x %x %x -> %x<=>%x\n" ,
+							red , green , blue ,
+							closest , hit_closest) ;
+					}
+				#endif
+
+					return  closest ;
 				}
 			}
 		}
 	}
 
-	return  closest ;
+#ifdef  __DEBUG
+	/*
+	 * XXX
+	 * The result of linear search of boundary values (115, 195 etc) is
+	 * different from that of ml_get_closest_256_color() of it.
+	 */
+	if( closest != hit_closest)
+	{
+		kik_debug_printf( "ERROR %x %x %x -> %x<=>%x\n" ,
+			red , green , blue , closest , hit_closest) ;
+	}
+#endif
+
+	if( use_pseudo_color ||
+	    ( ! ext_color_table &&
+	      ! ( ext_color_table = calloc( MAX_EXT_COLORS , sizeof(*ext_color_table)))))
+	{
+		return  closest ;
+	}
+
+	if( ( oldest_mark = ext_color_mark / 2) == MAX_EXT_COLORS / 2)
+	{
+		oldest_mark = 1 ;
+	}
+	else
+	{
+		oldest_mark ++ ;
+	}
+
+	if( ext_color_mark == MAX_EXT_COLORS)
+	{
+		ext_color_mark = 2 ;
+	}
+	else
+	{
+		ext_color_mark ++ ;
+	}
+
+	color = 0 ;
+	while( ext_color_table[color].mark)
+	{
+		u_int  diff ;
+		int  diff_r , diff_g , diff_b ;
+
+		/* lazy color-space conversion */
+		diff_r = red - ext_color_table[color].red ;
+		diff_g = green - ext_color_table[color].green ;
+		diff_b = blue - ext_color_table[color].blue ;
+		diff = COLOR_DISTANCE(diff_r,diff_g,diff_b) ;
+		if( diff < min)
+		{
+			min = diff ;
+			if( diff < color_distance_threshold)
+			{
+				/* Set new mark */
+				ext_color_table[color].mark = ext_color_mark / 2 ;
+
+			#ifdef  __DEBUG
+				kik_debug_printf( KIK_DEBUG_TAG " Use cached ext color %x\n" ,
+					INDEX_TO_EXT_COLOR(color)) ;
+			#endif
+
+				return  INDEX_TO_EXT_COLOR(color) ;
+			}
+		}
+
+		if( max == MAX_EXT_COLORS / 2)
+		{
+			/* do nothing */
+		}
+		else if( ext_color_table[color].mark == oldest_mark)
+		{
+			max = MAX_EXT_COLORS / 2 ;
+			oldest_color = color ;
+		}
+		else
+		{
+			if( ext_color_table[color].mark < oldest_mark)
+			{
+				diff = oldest_mark - ext_color_table[color].mark ;
+			}
+			else /* if( ext_color_table[color].mark > oldest_mark) */
+			{
+				diff = oldest_mark + (MAX_EXT_COLORS / 2) -
+					ext_color_table[color].mark ;
+			}
+
+			if( diff > max)
+			{
+				max = diff ;
+				oldest_color = color ;
+			}
+		}
+
+		if( ++color == MAX_EXT_COLORS)
+		{
+			color = oldest_color ;
+			ext_color_table[color].is_changed = 1 ;
+
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " Delete ext color %x mark %x\n" ,
+				INDEX_TO_EXT_COLOR(color) , ext_color_table[color].mark) ;
+		#endif
+
+			break ;
+		}
+	}
+
+	ext_color_table[color].mark = ext_color_mark / 2 ;
+	ext_color_table[color].red = red ;
+	ext_color_table[color].green = green ;
+	ext_color_table[color].blue = blue ;
+
+#ifdef  DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG "New ext color %.2x: r%.2x g%x b%.2x mark %x\n" ,
+		INDEX_TO_EXT_COLOR(color) , red , green , blue ,
+		ext_color_table[color].mark) ;
+#endif
+
+	return  INDEX_TO_EXT_COLOR(color) ;
+}
+
+int
+ml_ext_color_is_changed(
+	ml_color_t  color
+	)
+{
+	if( ext_color_table[EXT_COLOR_TO_INDEX(color)].is_changed)
+	{
+		ext_color_table[EXT_COLOR_TO_INDEX(color)].is_changed = 0 ;
+
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
 }
