@@ -5,12 +5,12 @@
 #include  "ml_line.h"
 
 #include  <string.h>		/* memset */
-#include  <kiklib/kik_mem.h>	/* alloca */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_util.h>	/* K_MIN */
+#include  <kiklib/kik_mem.h>	/* alloca */
 
 #include  "ml_ctl_loader.h"
-#include  "ml_shape.h"
+#include  "ml_ot_layout.h"
 
 
 #ifdef  DEBUG
@@ -27,6 +27,7 @@
 
 #define  ml_line_is_using_bidi( line)  ((line)->ctl_info_type == VINFO_BIDI)
 #define  ml_line_is_using_iscii( line)  ((line)->ctl_info_type == VINFO_ISCII)
+#define  ml_line_is_using_ot_layout( line)  ((line)->ctl_info_type == VINFO_OT_LAYOUT)
 
 #if  0
 #define  __DEBUG
@@ -110,17 +111,18 @@ ml_line_bidi_is_rtl(
 static int
 ml_bidi_copy(
 	ml_bidi_state_t  dst ,
-	ml_bidi_state_t  src
+	ml_bidi_state_t  src ,
+	int  optimize_ctl_info
 	)
 {
-	int (*func)( ml_bidi_state_t , ml_bidi_state_t) ;
+	int (*func)( ml_bidi_state_t , ml_bidi_state_t , int) ;
 
 	if( ! (func = ml_load_ctl_bidi_func( ML_BIDI_COPY)))
 	{
 		return  0 ;
 	}
 
-	return  (*func)( dst , src) ;
+	return  (*func)( dst , src , optimize_ctl_info) ;
 }
 
 static int
@@ -138,21 +140,6 @@ ml_bidi_reset(
 	return  (*func)( state) ;
 }
 
-static int
-ml_line_bidi_need_shape(
-	ml_line_t *  line
-	)
-{
-	int (*func)( ml_line_t *) ;
-
-	if( ! ( func = ml_load_ctl_bidi_func( ML_LINE_BIDI_NEED_SHAPE)))
-	{
-		return  0 ;
-	}
-
-	return  (*func)( line) ;
-}
-
 static int
 ml_line_bidi_render(
 	ml_line_t *  line ,
@@ -220,17 +207,18 @@ ml_line_set_use_iscii(
 static int
 ml_iscii_copy(
 	ml_iscii_state_t  dst ,
-	ml_iscii_state_t  src
+	ml_iscii_state_t  src ,
+	int  optimize_ctl_info
 	)
 {
-	int (*func)( ml_iscii_state_t , ml_iscii_state_t) ;
-	
+	int (*func)( ml_iscii_state_t , ml_iscii_state_t , int) ;
+
 	if( ! (func = ml_load_ctl_iscii_func( ML_ISCII_COPY)))
 	{
 		return  0 ;
 	}
 
-	return  (*func)( dst , src) ;
+	return  (*func)( dst , src , optimize_ctl_info) ;
 }
 
 static int
@@ -248,21 +236,6 @@ ml_iscii_reset(
 	return  (*func)( state) ;
 }
 
-static int
-ml_line_iscii_need_shape(
-	ml_line_t *  line
-	)
-{
-	int (*func)( ml_line_t *) ;
-
-	if( ! ( func = ml_load_ctl_iscii_func( ML_LINE_ISCII_NEED_SHAPE)))
-	{
-		return  0 ;
-	}
-
-	return  (*func)( line) ;
-}
-
 static int
 ml_line_iscii_render(
 	ml_line_t *  line
@@ -315,8 +288,7 @@ ml_line_iscii_logical(
 #define  ml_line_bidi_convert_visual_char_index_to_logical( line , char_index)  (char_index)
 #define  ml_line_bidi_copy_logical_str( line , dst , beg , len)  (0)
 #define  ml_line_bidi_is_rtl( line)  (0)
-#define  ml_line_bidi_need_shape( line)  (0)
-#define  ml_bidi_copy( dst , src)  (0)
+#define  ml_bidi_copy( dst , src , optimize)  (0)
 #define  ml_bidi_reset( state)  (0)
 #define  ml_line_bidi_render( line , bidi_mode , separators)  (0)
 #define  ml_line_bidi_visual( line)  (0)
@@ -327,8 +299,7 @@ int  ml_line_set_use_bidi( ml_line_t *  line , int  flag) ;
 int  ml_line_bidi_convert_visual_char_index_to_logical( ml_line_t *  line , int  char_index) ;
 int  ml_line_bidi_copy_logical_str( ml_line_t *  line , ml_char_t *  dst , int  beg , u_int  len) ;
 int  ml_line_bidi_is_rtl( ml_line_t *  line) ;
-int  ml_line_bidi_need_shape( ml_line_t *  line) ;
-int  ml_bidi_copy( ml_bidi_state_t  dst , ml_bidi_state_t  src) ;
+int  ml_bidi_copy( ml_bidi_state_t  dst , ml_bidi_state_t  src , int  optimize) ;
 int  ml_bidi_reset( ml_bidi_state_t  state) ;
 int  ml_line_bidi_convert_logical_char_index_to_visual( ml_line_t *  line , int  char_index ,
 	int *  ltr_rtl_meet_pos) ;
@@ -339,8 +310,7 @@ int  ml_line_bidi_logical( ml_line_t *  line) ;
 
 #ifndef  USE_IND
 #define  ml_line_set_use_iscii( line , flag)  (0)
-#define  ml_line_iscii_need_shape( line)  (0)
-#define  ml_iscii_copy( dst , src)  (0)
+#define  ml_iscii_copy( dst , src , optimize)  (0)
 #define  ml_iscii_reset( state)  (0)
 #define  ml_line_iscii_render( line)  (0)
 #define  ml_line_iscii_visual( line)  (0)
@@ -348,8 +318,7 @@ int  ml_line_bidi_logical( ml_line_t *  line) ;
 #else
 /* Link functions in libctl/ml_*iscii.c */
 int  ml_line_set_use_iscii( ml_line_t *  line , int  flag) ;
-int  ml_line_iscii_need_shape( ml_line_t *  line) ;
-int  ml_iscii_copy( ml_iscii_state_t  dst , ml_iscii_state_t  src) ;
+int  ml_iscii_copy( ml_iscii_state_t  dst , ml_iscii_state_t  src , int  optimize) ;
 int  ml_iscii_reset( ml_iscii_state_t  state) ;
 int  ml_line_iscii_convert_logical_char_index_to_visual( ml_line_t *  line ,
 	int  logical_char_index) ;
@@ -360,6 +329,468 @@ int  ml_line_iscii_logical( ml_line_t *  line) ;
 
 #endif	/* NO_DYNAMIC_LOAD_CTL */
 
+#ifdef  USE_OT_LAYOUT
+
+static int
+ml_line_set_use_ot_layout(
+	ml_line_t *  line ,
+	int  flag
+	)
+{
+	if( flag)
+	{
+		if( ml_line_is_using_ot_layout( line))
+		{
+			return  1 ;
+		}
+		else if( line->ctl_info_type != 0)
+		{
+			return  0 ;
+		}
+
+		if( ( line->ctl_info.ot_layout = ml_ot_layout_new()) == NULL)
+		{
+			return  0 ;
+		}
+
+		line->ctl_info_type = VINFO_OT_LAYOUT ;
+	}
+	else
+	{
+		if( ml_line_is_using_ot_layout( line))
+		{
+			ml_ot_layout_delete( line->ctl_info.ot_layout) ;
+			line->ctl_info_type = 0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+/* The caller should check ml_line_is_using_ot_layout() in advance. */
+static int
+ml_line_ot_layout_visual(
+	ml_line_t *  line
+	)
+{
+	ml_char_t *  src ;
+	u_int  src_len ;
+	ml_char_t *  dst ;
+	u_int  dst_len ;
+	int  dst_pos ;
+	int  src_pos ;
+
+	if( line->ctl_info.ot_layout->size == 0 ||
+	    ! line->ctl_info.ot_layout->substituted)
+	{
+	#ifdef  __DEBUG
+		kik_warn_printf( KIK_DEBUG_TAG " Not need to visualize.\n") ;
+	#endif
+
+		return  1 ;
+	}
+
+	src_len = line->num_of_filled_chars ;
+	if( ( src = ml_str_alloca( src_len)) == NULL)
+	{
+		return  0 ;
+	}
+
+	ml_str_copy( src , line->chars , src_len) ;
+
+	dst_len = line->ctl_info.ot_layout->size ;
+	if( line->num_of_chars < dst_len)
+	{
+		ml_char_t *  chars ;
+
+		if( ( chars = ml_str_new( dst_len)))
+		{
+			/* XXX => shrunk at ml_screen.c and ml_logical_visual_ctl.c */
+			ml_str_delete( line->chars , line->num_of_chars) ;
+			line->chars = chars ;
+			line->num_of_chars = dst_len ;
+		}
+		else
+		{
+			dst_len = line->num_of_chars ;
+		}
+	}
+
+	dst = line->chars ;
+
+	src_pos = 0 ;
+	for( dst_pos = 0 ; dst_pos < dst_len ; dst_pos++)
+	{
+		if( line->ctl_info.ot_layout->num_of_chars_array[dst_pos] == 0)
+		{
+			/*
+			 * (logical)ACD -> (visual)A0CD -> (shaped)abcd
+			 *          B              B
+			 * (B is combining char)
+			 */
+			ml_char_copy( dst + dst_pos , ml_get_base_char( src + src_pos - 1)) ;
+			ml_char_set_code( dst + dst_pos , 0) ;
+		}
+		else
+		{
+			u_int  count ;
+
+			ml_char_copy( dst + dst_pos , src + (src_pos ++)) ;
+
+			for( count = 1 ;
+			     count < line->ctl_info.ot_layout->num_of_chars_array[dst_pos] ;
+			     count ++)
+			{
+				ml_char_t *  comb ;
+				u_int  num ;
+
+			#ifdef  DEBUG
+				if( ml_char_is_comb( ml_get_base_char( src + src_pos)))
+				{
+					kik_debug_printf( KIK_DEBUG_TAG " illegal ot_layout\n") ;
+				}
+			#endif
+
+				ml_char_combine_simple( dst + dst_pos ,
+					ml_get_base_char( src + src_pos)) ;
+
+				comb = ml_get_combining_chars( src + (src_pos ++) , &num) ;
+				for( ; num > 0 ; num--)
+				{
+				#ifdef  DEBUG
+					if( ! ml_char_is_comb( comb))
+					{
+						kik_debug_printf( KIK_DEBUG_TAG
+							" illegal ot_layout\n") ;
+					}
+				#endif
+					ml_char_combine_simple( dst + dst_pos , comb ++) ;
+				}
+			}
+		}
+	}
+
+#ifdef  DEBUG
+	if( src_pos != src_len)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG "ml_line_ot_layout_visual() failed: %d -> %d\n" ,
+			src_len , src_pos) ;
+	}
+#endif
+
+	ml_str_final( src , src_len) ;
+
+	line->num_of_filled_chars = dst_pos ;
+
+	return  1 ;
+}
+
+/* The caller should check ml_line_is_using_ot_layout() in advance. */
+static int
+ml_line_ot_layout_logical(
+	ml_line_t *  line
+	)
+{
+	ml_char_t *  src ;
+	u_int  src_len ;
+	ml_char_t *  dst ;
+	int  src_pos ;
+
+	if( line->ctl_info.ot_layout->size == 0 ||
+	    ! line->ctl_info.ot_layout->substituted)
+	{
+	#ifdef  __DEBUG
+		kik_warn_printf( KIK_DEBUG_TAG " Not need to logicalize.\n") ;
+	#endif
+
+		return  1 ;
+	}
+
+	src_len = line->num_of_filled_chars ;
+	if( ( src = ml_str_alloca( src_len)) == NULL)
+	{
+		return  0 ;
+	}
+
+	ml_str_copy( src , line->chars , src_len) ;
+	dst = line->chars ;
+
+	for( src_pos = 0 ; src_pos < line->ctl_info.ot_layout->size ; src_pos++)
+	{
+		ml_char_t *  comb ;
+		u_int  num ;
+
+		if( line->ctl_info.ot_layout->num_of_chars_array[src_pos] == 0)
+		{
+			continue ;
+		}
+
+		if( ml_char_cs( src + src_pos) == ISO10646_UCS4_1_V)
+		{
+			ml_char_set_cs( src + src_pos , ISO10646_UCS4_1) ;
+		}
+
+		if( line->ctl_info.ot_layout->num_of_chars_array[src_pos] == 1)
+		{
+			ml_char_copy( dst , src + src_pos) ;
+		}
+		else
+		{
+			ml_char_copy( dst , ml_get_base_char( src + src_pos)) ;
+
+			comb = ml_get_combining_chars( src + src_pos , &num) ;
+			for( ; num > 0 ; num-- , comb++)
+			{
+				if( ml_char_cs( comb) == ISO10646_UCS4_1_V)
+				{
+					ml_char_set_cs( comb , ISO10646_UCS4_1) ;
+					kik_debug_printf( "%x\n" , ml_char_cs( comb)) ;
+				}
+
+				if( ml_char_is_comb( comb))
+				{
+					ml_char_combine_simple( dst , comb) ;
+				}
+				else
+				{
+					ml_char_copy( ++ dst , comb) ;
+				}
+			}
+		}
+
+		dst++ ;
+	}
+
+	ml_str_final( src , src_len) ;
+
+	line->num_of_filled_chars = dst - line->chars ;
+
+	return  1 ;
+}
+
+/* The caller should check ml_line_is_using_ot_layout() in advance. */
+static int
+ml_line_ot_layout_convert_logical_char_index_to_visual(
+	ml_line_t *  line ,
+	int  logical_char_index
+	)
+{
+	int  visual_char_index ;
+
+	if( ml_line_is_empty(line))
+	{
+		return  0 ;
+	}
+
+	if( line->ctl_info.ot_layout->size == 0 ||
+	    ! line->ctl_info.ot_layout->substituted)
+	{
+	#ifdef  __DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " logical char_index is same as visual one.\n") ;
+	#endif
+		return  logical_char_index ;
+	}
+
+	for( visual_char_index = 0 ; visual_char_index < line->ctl_info.ot_layout->size ;
+	     visual_char_index++)
+	{
+		if( logical_char_index == 0 ||
+		    ( logical_char_index -=
+			line->ctl_info.ot_layout->num_of_chars_array[visual_char_index]) < 0)
+		{
+			break ;
+		}
+	}
+
+	return  visual_char_index ;
+}
+
+/* The caller should check ml_line_is_using_ot_layout() in advance. */
+static int
+ml_line_ot_layout_render(
+	ml_line_t *  line ,	/* is always modified */
+	void *  term
+	)
+{
+	int  ret ;
+	int  visual_mod_beg ;
+
+	/*
+	 * Lower case: ASCII
+	 * Upper case: GLYPH INDEX
+	 *    (Logical) AAA == (Visual) BBBBB
+	 * => (Logical) aaa == (Visual) aaa
+	 * In this case ml_line_is_cleared_to_end() returns 0, so "BB" remains on
+	 * the screen unless following ml_line_set_modified().
+	 */
+	visual_mod_beg = ml_line_get_beg_of_modified( line) ;
+	if( line->ctl_info.ot_layout->substituted)
+	{
+		visual_mod_beg = ml_line_ot_layout_convert_logical_char_index_to_visual(
+					line , visual_mod_beg) ;
+	}
+
+	if( ml_line_is_real_modified( line))
+	{
+		line->ctl_info.ot_layout->term = term ;
+
+		if( ( ret = ml_ot_layout( line->ctl_info.ot_layout , line->chars ,
+					line->num_of_filled_chars)) <= 0)
+		{
+			return  ret ;
+		}
+
+		if( line->ctl_info.ot_layout->substituted)
+		{
+			int  beg ;
+
+			if( ( beg = ml_line_ot_layout_convert_logical_char_index_to_visual( line ,
+					ml_line_get_beg_of_modified( line))) < visual_mod_beg)
+			{
+				visual_mod_beg = beg ;
+			}
+		}
+
+		/*
+		 * Conforming line->change_{beg|end}_col to visual mode.
+		 * If this line contains glyph indeces, it should be redrawn to
+		 * the end of line.
+		 */
+		ml_line_set_modified( line , visual_mod_beg , line->num_of_chars) ;
+	}
+	else
+	{
+		ml_line_set_modified( line , visual_mod_beg ,
+			ml_line_ot_layout_convert_logical_char_index_to_visual( line ,
+					ml_line_get_end_of_modified( line))) ;
+	}
+
+	return  1 ;
+}
+
+#endif	/* USE_OT_LAYOUT */
+
+static void
+copy_line(
+	ml_line_t *  dst ,
+	ml_line_t *  src ,
+	int  optimize_ctl_info
+	)
+{
+	u_int  copy_len ;
+
+	copy_len = K_MIN(src->num_of_filled_chars,dst->num_of_chars) ;
+
+	ml_str_copy( dst->chars , src->chars , copy_len) ;
+	dst->num_of_filled_chars = copy_len ;
+
+	dst->change_beg_col = K_MIN(src->change_beg_col,dst->num_of_chars) ;
+	dst->change_end_col = K_MIN(src->change_end_col,dst->num_of_chars) ;
+
+	dst->is_modified = src->is_modified ;
+	dst->is_continued_to_next = src->is_continued_to_next ;
+	dst->size_attr = src->size_attr ;
+
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
+	if( ml_line_is_using_bidi( src))
+	{
+		if( ml_line_is_using_bidi( dst) || ml_line_set_use_bidi( dst , 1))
+		{
+			/*
+			 * Don't use ml_line_bidi_render() here,
+			 * or it is impossible to call this function in visual context.
+			 */
+			if( dst->num_of_chars < src->num_of_filled_chars)
+			{
+			#ifdef  DEBUG
+				kik_debug_printf( KIK_DEBUG_TAG " Not copy ml_bidi.\n") ;
+			#endif
+			}
+			else if( ml_bidi_copy( dst->ctl_info.bidi , src->ctl_info.bidi ,
+					optimize_ctl_info) == -1)
+			{
+				dst->ctl_info_type = 0 ;
+				dst->ctl_info.bidi = NULL ;
+			}
+		}
+	}
+	else if( ml_line_is_using_bidi( dst))
+	{
+		ml_line_set_use_bidi( dst , 0) ;
+	}
+#endif
+
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
+	if( ml_line_is_using_iscii( src))
+	{
+		if( ml_line_is_using_iscii( dst) || ml_line_set_use_iscii( dst , 1))
+		{
+			/*
+			 * Don't use ml_line_iscii_render() here,
+			 * or it is impossible to call this function in visual context.
+			 */
+			if( dst->num_of_chars < src->num_of_filled_chars)
+			{
+			#ifdef  DEBUG
+				kik_debug_printf( KIK_DEBUG_TAG " Not copy ml_iscii.\n") ;
+			#endif
+			}
+			else if( ml_iscii_copy( dst->ctl_info.iscii , src->ctl_info.iscii ,
+				optimize_ctl_info) == -1)
+			{
+				dst->ctl_info_type = 0 ;
+				dst->ctl_info.iscii = NULL ;
+			}
+		}
+	}
+	else if( ml_line_is_using_iscii( dst))
+	{
+		ml_line_set_use_iscii( dst , 0) ;
+	}
+#endif
+
+#ifdef  USE_OT_LAYOUT
+	if( ml_line_is_using_ot_layout( src))
+	{
+		if( ml_line_is_using_ot_layout( dst) || ml_line_set_use_ot_layout( dst , 1))
+		{
+			/*
+			 * Don't use ml_line_ot_layout_render() here,
+			 * or it is impossible to call this function in visual context.
+			 */
+			if( dst->num_of_chars < src->num_of_filled_chars)
+			{
+			#ifdef  DEBUG
+				kik_debug_printf( KIK_DEBUG_TAG " Not copy ml_ot_layout.\n") ;
+			#endif
+			}
+			else if( ml_ot_layout_copy( dst->ctl_info.ot_layout ,
+					src->ctl_info.ot_layout ,
+					optimize_ctl_info) == -1)
+			{
+				dst->ctl_info_type = 0 ;
+				dst->ctl_info.ot_layout = NULL ;
+			}
+		}
+	}
+	else if( ml_line_is_using_ot_layout( dst))
+	{
+		ml_line_set_use_ot_layout( dst , 0) ;
+	}
+#endif
+}
+
+static void
+set_real_modified(
+	ml_line_t *  line ,
+	int  beg_char_index ,
+	int  end_char_index
+	)
+{
+	ml_line_set_modified( line , beg_char_index , end_char_index) ;
+	line->is_modified = 2 ;
+}
+
 
 /* --- global functions --- */
 
@@ -393,7 +824,7 @@ ml_line_clone(
 	)
 {
 	ml_line_init( clone , num_of_chars) ;
-	ml_line_copy( clone , orig) ;
+	copy_line( clone , orig , 1 /* clone->ctl_info can be uncopied. */) ;
 
 	return  1 ;
 }
@@ -411,6 +842,12 @@ ml_line_final(
 	{
 		ml_line_set_use_iscii( line , 0) ;
 	}
+#ifdef  USE_OT_LAYOUT
+	else if( ml_line_is_using_ot_layout( line))
+	{
+		ml_line_set_use_ot_layout( line , 0) ;
+	}
+#endif
 
 	if( line->chars)
 	{
@@ -460,11 +897,15 @@ ml_line_break_boundary(
 		ml_char_copy( line->chars + count , ml_sp_ch()) ;
 	}
 
-	/*
-	 * change_{beg|end}_col is not updated , because space has no glyph.
-	 */
 #if  0
-	ml_line_set_modified( line , END_CHAR_INDEX(line) + 1 , END_CHAR_INDEX(line) + size) ;
+	set_real_modified( line , END_CHAR_INDEX(line) + 1 , END_CHAR_INDEX(line) + size) ;
+#else
+	if( ml_line_is_using_ctl( line) && ! ml_line_is_real_modified( line))
+	{
+		/* ctl_render_line() should be called in ctl_render() in ml_logical_visual.c. */
+		set_real_modified( line , END_CHAR_INDEX(line) + size ,
+			END_CHAR_INDEX(line) + size) ;
+	}
 #endif
 
 	line->num_of_filled_chars += size ;
@@ -514,7 +955,7 @@ ml_line_reset(
 		{
 			if( ! ml_char_equal( line->chars + count , ml_sp_ch()))
 			{
-				ml_line_set_modified( line , 0 , count) ;
+				set_real_modified( line , 0 , count) ;
 
 				break ;
 			}
@@ -525,7 +966,7 @@ ml_line_reset(
 		}
 	}
 #else
-	ml_line_set_modified( line , 0 , END_CHAR_INDEX(line)) ;
+	set_real_modified( line , 0 , END_CHAR_INDEX(line)) ;
 #endif
 	
 	line->num_of_filled_chars = 0 ;
@@ -538,9 +979,16 @@ ml_line_reset(
 	{
 		ml_iscii_reset( line->ctl_info.iscii) ;
 	}
+#ifdef  USE_OT_LAYOUT
+	else if( ml_line_is_using_ot_layout( line))
+	{
+		ml_ot_layout_reset( line->ctl_info.ot_layout) ;
+	}
+#endif
 
 	line->is_continued_to_next = 0 ;
-	
+	line->size_attr = 0 ;
+
 	return 1 ;
 }
 
@@ -564,7 +1012,7 @@ ml_line_clear(
 		{
 			if( ! ml_char_equal( line->chars + count , ml_sp_ch()))
 			{
-				ml_line_set_modified( line , char_index , count) ;
+				set_real_modified( line , char_index , count) ;
 
 				break ;
 			}
@@ -575,12 +1023,13 @@ ml_line_clear(
 		}
 	}
 #else
-	ml_line_set_modified( line , char_index , END_CHAR_INDEX(line)) ;
+	set_real_modified( line , char_index , END_CHAR_INDEX(line)) ;
 #endif
 
 	ml_char_copy( line->chars + char_index , ml_sp_ch()) ;
 	line->num_of_filled_chars = char_index + 1 ;
 	line->is_continued_to_next = 0 ;
+	line->size_attr = 0 ;
 
 	return  1 ;
 }
@@ -763,7 +1212,7 @@ ml_line_overwrite(
 
 	line->num_of_filled_chars = new_len ;
 
-	ml_line_set_modified( line , beg_char_index , beg_char_index + len + padding - 1) ;
+	set_real_modified( line , beg_char_index , beg_char_index + len + padding - 1) ;
 	
 	return  1 ;
 }
@@ -779,12 +1228,12 @@ ml_line_overwrite_all(
 	int  len
 	)
 {
-	ml_line_set_modified( line , 0 , END_CHAR_INDEX(line)) ;
+	set_real_modified( line , 0 , END_CHAR_INDEX(line)) ;
 
 	ml_str_copy( line->chars , chars , len) ;
 	line->num_of_filled_chars = len ;
 
-	ml_line_set_modified( line , 0 , END_CHAR_INDEX(line)) ;
+	set_real_modified( line , 0 , END_CHAR_INDEX(line)) ;
 
 	return  1 ;
 }
@@ -937,7 +1386,7 @@ ml_line_fill(
 	
 	line->num_of_filled_chars = char_index + copy_len ;
 
-	ml_line_set_modified( line , beg , beg + num + left_cols) ;
+	set_real_modified( line , beg , beg + num + left_cols) ;
 
 	return  1 ;
 }
@@ -1058,7 +1507,11 @@ ml_line_set_modified_all(
 	 */
 	line->change_end_col = line->num_of_chars * 2 ;
 
-	line->is_modified = 1 ;
+	/* Don't overwrite if line->is_modified == 2 (real modified) */
+	if( ! line->is_modified)
+	{
+		line->is_modified = 1 ;
+	}
 
 	return  1 ;
 }
@@ -1315,50 +1768,7 @@ ml_line_copy(
 	ml_line_t *  src
 	)
 {
-	u_int  copy_len ;
-
-	copy_len = K_MIN(src->num_of_filled_chars,dst->num_of_chars) ;
-
-	ml_str_copy( dst->chars , src->chars , copy_len) ;
-	dst->num_of_filled_chars = copy_len ;
-
-	dst->change_beg_col = K_MIN(src->change_beg_col,dst->num_of_chars) ;
-	dst->change_end_col = K_MIN(src->change_end_col,dst->num_of_chars) ;
-	
-	dst->is_modified = src->is_modified ;
-	dst->is_continued_to_next = src->is_continued_to_next ;
-
-	if( ml_line_is_using_bidi( src))
-	{
-		if( ml_line_is_using_bidi( dst) || ml_line_set_use_bidi( dst , 1))
-		{
-			/*
-			 * Don't use ml_line_bidi_render() here,
-			 * or it is impossible to call this function in visual context.
-			 */
-			ml_bidi_copy( dst->ctl_info.bidi , src->ctl_info.bidi) ;
-		}
-	}
-	else if( ml_line_is_using_bidi( dst))
-	{
-		ml_line_set_use_bidi( dst , 0) ;
-	}
-
-	if( ml_line_is_using_iscii( src))
-	{
-		if( ml_line_is_using_iscii( dst) || ml_line_set_use_iscii( dst , 1))
-		{
-			/*
-			 * Don't use ml_line_iscii_render() here,
-			 * or it is impossible to call this function in visual context.
-			 */
-			ml_iscii_copy( dst->ctl_info.iscii , src->ctl_info.iscii) ;
-		}
-	}
-	else if( ml_line_is_using_iscii( dst))
-	{
-		ml_line_set_use_iscii( dst , 0) ;
-	}
+	copy_line( dst , src , 0) ;
 
 	return  1 ;
 }
@@ -1481,6 +1891,19 @@ ml_line_get_num_of_filled_chars_except_spaces_with_func(
 	}
 }
 
+void
+ml_line_set_size_attr(
+	ml_line_t *  line ,
+	int  size_attr
+	)
+{
+	if( line->size_attr != size_attr)
+	{
+		line->size_attr = size_attr ;
+		ml_line_set_modified_all( line) ;
+	}
+}
+
 
 int
 ml_line_convert_visual_char_index_to_logical(
@@ -1546,6 +1969,14 @@ ml_line_convert_logical_char_index_to_visual(
 #ifdef  NO_DYNAMIC_LOAD_CTL
 	if( line->ctl_info_type)
 	{
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
+		{
+			char_index = ml_line_ot_layout_convert_logical_char_index_to_visual(
+					line , char_index) ;
+		}
+		else
+	#endif
 		if( ml_line_is_using_bidi( line))
 		{
 		#ifdef  USE_FRIBIDI
@@ -1566,6 +1997,14 @@ ml_line_convert_logical_char_index_to_visual(
 #else
 	if( line->ctl_info_type)
 	{
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
+		{
+			char_index = ml_line_ot_layout_convert_logical_char_index_to_visual(
+					line , char_index) ;
+		}
+		else
+	#endif
 		if( ml_line_is_using_bidi( line))
 		{
 			int (*bidi_func)( ml_line_t * , int , int *) ;
@@ -1576,7 +2015,7 @@ ml_line_convert_logical_char_index_to_visual(
 				char_index = (*bidi_func)( line , char_index , meet_pos) ;
 			}
 		}
-		else /* if( ml_line_is_using_iscii( line) */
+		else /* if( ml_line_is_using_iscii( line)) */
 		{
 			int (*iscii_func)( ml_line_t * , int) ;
 
@@ -1593,77 +2032,6 @@ ml_line_convert_logical_char_index_to_visual(
 }
 
 
-ml_line_t *
-ml_line_shape(
-	ml_line_t *  line
-	)
-{
-	ml_line_t *  orig ;
-	ml_char_t *  shaped ;
-	u_int  (*func)( ml_char_t * , u_int , ml_char_t * , u_int) ;
-
-	if( line->ctl_info_type)
-	{
-		if( ml_line_is_using_bidi( line))
-		{
-			if( ! ml_line_bidi_need_shape( line))
-			{
-				return  NULL ;
-			}
-
-			func = ml_shape_arabic ;
-		}
-		else /* if( ml_line_is_using_iscii( line)) */
-		{
-			if( ! ml_line_iscii_need_shape( line))
-			{
-				return  NULL ;
-			}
-
-			func = ml_shape_iscii ;
-		}
-
-		if( ( orig = malloc( sizeof( ml_line_t))) == NULL)
-		{
-			return  NULL ;
-		}
-
-		ml_line_share( orig , line) ;
-
-		if( ( shaped = ml_str_new( line->num_of_chars)) == NULL)
-		{
-			free( orig) ;
-
-			return  NULL ;
-		}
-
-		line->num_of_filled_chars = (*func)( shaped , line->num_of_chars ,
-							line->chars , line->num_of_filled_chars) ;
-
-		line->chars = shaped ;
-
-		return  orig ;
-	}
-
-	return  NULL ;
-}
-
-int
-ml_line_unshape(
-	ml_line_t *  line ,
-	ml_line_t *  orig
-	)
-{
-	ml_str_delete( line->chars , line->num_of_chars) ;
-	
-	line->chars = orig->chars ;
-	line->num_of_filled_chars = orig->num_of_filled_chars ;
-
-	free( orig) ;
-
-	return  1 ;
-}
-
 int
 ml_line_unuse_ctl(
 	ml_line_t *  line
@@ -1671,6 +2039,13 @@ ml_line_unuse_ctl(
 {
 	if( line->ctl_info_type)
 	{
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
+		{
+			return  ml_line_set_use_ot_layout( line , 0) ;
+		}
+		else
+	#endif
 		if( ml_line_is_using_bidi( line))
 		{
 			return  ml_line_set_use_bidi( line , 0) ;
@@ -1688,12 +2063,20 @@ int
 ml_line_ctl_render(
 	ml_line_t *  line ,
 	ml_bidi_mode_t  bidi_mode ,
-	const char *  separators
+	const char *  separators ,
+	void *  term
 	)
 {
+	int  ret ;
+	int  (*set_use_ctl)( ml_line_t * , int) ;
+
 	if( ! ml_line_is_using_ctl( line))
 	{
-		if( ! ml_line_set_use_bidi( line , 1) && ! ml_line_set_use_iscii( line , 1))
+		if(
+		#ifdef  USE_OT_LAYOUT
+		    (! term || ! ml_line_set_use_ot_layout( line , 1)) &&
+		#endif
+		    ! ml_line_set_use_bidi( line , 1) && ! ml_line_set_use_iscii( line , 1))
 		{
 			return  0 ;
 		}
@@ -1701,47 +2084,144 @@ ml_line_ctl_render(
 
 	if( line->ctl_info_type)
 	{
-		int  ret ;
-
-		if( ml_line_is_using_bidi( line))
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
 		{
-			if( ( ret = ml_line_bidi_render( line , bidi_mode , separators)) < 0
-			#if  ! defined(NO_DYNAMIC_LOAD_CTL)
-			    && ml_load_ctl_iscii_func( ML_LINE_SET_USE_ISCII)
-			#elif  ! defined(USE_FRIBIDI)
-			    && 0
-			#endif
-			    )
+			if( ! term)
 			{
-				ml_line_set_use_bidi( line , 0) ;
-				ml_line_set_use_iscii( line , 1) ;
-
-				return  ml_line_iscii_render( line) ;
+				ret = -1 ;
+			}
+			else if( ( ret = ml_line_ot_layout_render( line , term)) >= 0)
+			{
+				return  ret ;
 			}
 
-			return  ret ;
+			set_use_ctl = ml_line_set_use_ot_layout ;
+
+			if( ret == -1)
+			{
+				goto  render_bidi ;
+			}
+			else /* if( ret == -2) */
+			{
+				goto  render_iscii ;
+			}
+		}
+		else
+	#endif
+		if( ml_line_is_using_bidi( line))
+		{
+			if( ( ret = ml_line_bidi_render( line , bidi_mode , separators)) >= 0)
+			{
+				return  ret ;
+			}
+
+		#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
+			set_use_ctl = ml_line_set_use_bidi ;
+		#else
+			/* Never enter here */
+		#endif
+
+			if( ret == -1)
+			{
+			#ifdef  USE_OT_LAYOUT
+				if( ! term)
+				{
+					return  1 ;
+				}
+
+				goto  render_ot_layout ;
+			#else
+				return  1 ;
+			#endif
+			}
+			else /* if( ret == -2) */
+			{
+				goto  render_iscii ;
+			}
 		}
 		else /* if( ml_line_is_using_iscii( line)) */
 		{
-			if( ( ret = ml_line_iscii_render( line)) < 0
-			#if  ! defined(NO_DYNAMIC_LOAD_CTL)
-			    && ml_load_ctl_bidi_func( ML_LINE_SET_USE_BIDI)
-			#elif  ! defined(USE_FRIBIDI)
-			    && 0
-			#endif
-			    )
+			if( ( ret = ml_line_iscii_render( line)) >= 0)
 			{
-				ml_line_set_use_iscii( line , 0) ;
-				ml_line_set_use_bidi( line , 1) ;
-
-				return  ml_line_bidi_render( line , bidi_mode , separators) ;
+				return  ret ;
 			}
 
-			return  ret ;
+		#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
+			set_use_ctl = ml_line_set_use_iscii ;
+		#else
+			/* Never enter here */
+		#endif
+
+		#ifdef  USE_OT_LAYOUT
+			if( term)
+			{
+				goto  render_ot_layout ;
+			}
+			else
+		#endif
+			{
+				goto  render_bidi ;
+			}
 		}
 	}
 
 	return  0 ;
+
+#ifdef  USE_OT_LAYOUT
+render_ot_layout:
+	(*set_use_ctl)( line , 0) ;
+	ml_line_set_use_ot_layout( line , 1) ;
+
+	if( ( ret = ml_line_ot_layout_render( line , term) != -1))
+	{
+		return  ret ;
+	}
+
+	/* Fall through */
+#endif
+
+render_bidi:
+	if(
+	#if  ! defined(NO_DYNAMIC_LOAD_CTL)
+	    ml_load_ctl_bidi_func( ML_LINE_SET_USE_BIDI)
+	#elif  defined(USE_FRIBIDI)
+	    1
+	#else
+	    0
+	#endif
+	    )
+	{
+		(*set_use_ctl)( line , 0) ;
+		ml_line_set_use_bidi( line , 1) ;
+
+		return  ml_line_bidi_render( line , bidi_mode , separators) ;
+	}
+	else
+	{
+		return  0 ;
+	}
+
+render_iscii:
+	if(
+	#if  ! defined(NO_DYNAMIC_LOAD_CTL)
+	    ml_load_ctl_iscii_func( ML_LINE_SET_USE_ISCII)
+	#elif  defined(USE_ISCII)
+	    1
+	#else
+	    0
+	#endif
+	    )
+	{
+		(*set_use_ctl)( line , 0) ;
+		ml_line_set_use_iscii( line , 1) ;
+
+		return  ml_line_iscii_render( line) ;
+	}
+	else
+	{
+		return  0 ;
+	}
 }
 
 int
@@ -1751,6 +2231,13 @@ ml_line_ctl_visual(
 {
 	if( line->ctl_info_type)
 	{
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
+		{
+			return  ml_line_ot_layout_visual( line) ;
+		}
+		else
+	#endif
 		if( ml_line_is_using_bidi( line))
 		{
 			return  ml_line_bidi_visual( line) ;
@@ -1771,6 +2258,13 @@ ml_line_ctl_logical(
 {
 	if( line->ctl_info_type)
 	{
+	#ifdef  USE_OT_LAYOUT
+		if( ml_line_is_using_ot_layout( line))
+		{
+			return  ml_line_ot_layout_logical( line) ;
+		}
+		else
+	#endif
 		if( ml_line_is_using_bidi( line))
 		{
 			return  ml_line_bidi_logical( line) ;
@@ -1784,6 +2278,7 @@ ml_line_ctl_logical(
 	return  0 ;
 }
 
+
 #ifdef  DEBUG
 
 void
