@@ -9,6 +9,7 @@
 #include  "ml_str.h"
 #include  "ml_bidi.h"	/* ml_bidi_state_t */
 #include  "ml_iscii.h"	/* ml_iscii_state_t */
+#include  "ml_ot_layout.h"	/* ml_ot_layout_state_t */
 
 
 enum
@@ -18,17 +19,18 @@ enum
 	SCROLL         = 0x04
 } ;
 
-
 enum
 {
 	VINFO_BIDI = 0x01 ,
-	VINFO_ISCII = 0x02
+	VINFO_ISCII = 0x02 ,
+	VINFO_OT_LAYOUT = 0x03 ,
 } ;
 
 typedef union  ctl_info
 {
 	ml_bidi_state_t  bidi ;
 	ml_iscii_state_t  iscii ;
+	ml_ot_layout_state_t  ot_layout ;
 
 } ctl_info_t ;
 
@@ -54,15 +56,18 @@ typedef struct  ml_line
 	u_int16_t  change_beg_col ;		/* 0 - 65535 */
 	u_int16_t  change_end_col ;		/* 0 - 65535 */
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND) || defined(USE_FRIBIDI)
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND) || defined(USE_FRIBIDI) || defined(USE_OT_LAYOUT)
 	/* Don't touch from ml_line.c. ctl_info is used by ml_line_bidi.c and ml_line_iscii.c. */
 	ctl_info_t  ctl_info ;
 #endif
 	u_int8_t  ctl_info_type ;
 
-	int8_t  is_modified ;
+	int8_t  is_modified ;		/* 1: need to redraw. 2: was really changed. */
 	int8_t  is_continued_to_next ;
 
+	/* public */
+	int8_t  size_attr ;
+
 } ml_line_t ;
 
 
@@ -101,6 +106,9 @@ int  ml_line_is_cleared_to_end( ml_line_t *  line) ;
 
 int  ml_line_is_modified( ml_line_t *  line) ;
 
+/* XXX Private api for ml_line.c and ml_line_{iscii|bidi}.c */
+#define  ml_line_is_real_modified( line)  (ml_line_is_modified( line) == 2)
+
 int  ml_line_get_beg_of_modified( ml_line_t *  line) ;
 
 int  ml_line_get_end_of_modified( ml_line_t *  line) ;
@@ -141,12 +149,18 @@ u_int  ml_line_get_num_of_filled_chars_except_spaces_with_func(
 #define  ml_line_get_num_of_filled_chars_except_spaces(line) \
 	ml_line_get_num_of_filled_chars_except_spaces_with_func( (line) , ml_char_code_equal)
 
-int  ml_line_get_word_pos( ml_line_t *  line , int *  beg_char_index , int *  end_char_index ,
-	int  char_index) ;
+void  ml_line_set_size_attr( ml_line_t *  line , int  size_attr) ;
 
 
 #define  ml_line_is_using_ctl( line)  ((line)->ctl_info_type)
 
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_OT_LAYOUT)
+#define  ml_line_has_ot_substitute_glyphs( line) \
+	((line)->ctl_info_type == VINFO_OT_LAYOUT && (line)->ctl_info.ot_layout->substituted)
+#else
+#define  ml_line_has_ot_substitute_glyphs( line)  (0)
+#endif
+
 int  ml_line_convert_visual_char_index_to_logical( ml_line_t *  line , int  char_index) ;
 
 int  ml_line_is_rtl( ml_line_t *  line) ;
@@ -164,7 +178,7 @@ int  ml_line_unshape( ml_line_t *  line , ml_line_t *  orig) ;
 int  ml_line_unuse_ctl( ml_line_t *  line) ;
 
 int  ml_line_ctl_render( ml_line_t *  line , ml_bidi_mode_t  bidi_mode ,
-	const char *  separators) ;
+	const char *  separators , void *  term) ;
 
 int  ml_line_ctl_visual( ml_line_t *  line) ;
 
