@@ -67,6 +67,7 @@ typedef struct  ctl_logical_visual
 	int  ltr_rtl_meet_pos ;
 	ml_bidi_mode_t  bidi_mode ;
 	const char *  separators ;
+	void *  term ;
 
 } ctl_logical_visual_t ;
 
@@ -870,7 +871,7 @@ vert_visual_line(
 }
 
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND) || defined(USE_OT_LAYOUT)
 
 /*
  * Ctl logical <=> visual methods
@@ -945,7 +946,8 @@ ctl_render_line(
 	{
 		ml_line_ctl_render( line ,
 			((ctl_logical_visual_t*)logvis)->bidi_mode ,
-			((ctl_logical_visual_t*)logvis)->separators) ;
+			((ctl_logical_visual_t*)logvis)->separators ,
+			((ctl_logical_visual_t*)logvis)->term) ;
 	}
 }
 
@@ -1263,22 +1265,25 @@ ml_get_vertical_mode_name(
 }
 
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND) || defined(USE_OT_LAYOUT)
 
 ml_logical_visual_t *
 ml_logvis_ctl_new(
 	ml_bidi_mode_t  bidi_mode ,
-	const char *  separators
+	const char *  separators ,
+	void *  term
 	)
 {
 	ctl_logical_visual_t *  ctl_logvis ;
 
+#ifndef  USE_OT_LAYOUT
 #ifndef  NO_DYNAMIC_LOAD_CTL
 	if( ! ml_load_ctl_bidi_func( ML_LINE_SET_USE_BIDI) &&
 	    ! ml_load_ctl_iscii_func( ML_LINE_SET_USE_ISCII))
 	{
 		return  NULL ;
 	}
+#endif
 #endif
 
 	if( ( ctl_logvis = calloc( 1 , sizeof( ctl_logical_visual_t))) == NULL)
@@ -1288,6 +1293,7 @@ ml_logvis_ctl_new(
 
 	ctl_logvis->bidi_mode = bidi_mode ;
 	ctl_logvis->separators = separators ;
+	ctl_logvis->term = term ;
 
 	ctl_logvis->logvis.delete = ctl_delete ;
 	ctl_logvis->logvis.init = ctl_init ;
