@@ -5,6 +5,7 @@
 #include  "ml_screen.h"
 
 #include  <stdlib.h>		/* abs */
+#include  <unistd.h>		/* write */
 #include  <sys/time.h>
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>	/* malloc/free */
@@ -25,6 +26,7 @@
 /* --- static variables --- */
 
 static char *  word_separators = " .,:;/|@()[]{}" ;
+static int  regard_uri_as_word = 0 ;
 
 
 /* --- static functions --- */
@@ -136,7 +138,11 @@ receive_scrolled_out_line(
 			line->num_of_chars - ml_screen_get_logical_cols( screen)) ;
 		line->num_of_chars = ml_screen_get_logical_cols( screen) ;
 	}
+	else
 #endif
+	{
+		ml_line_set_size_attr( line , 0) ;
+	}
 
 	if( ml_screen_is_backscrolling( screen) == BSM_STATIC)
 	{
@@ -802,7 +808,7 @@ get_msec_time(void)
 
 int
 ml_set_word_separators(
-	char *  seps
+	const char *  seps
 	)
 {
 	static char *  default_word_separators ;
@@ -844,6 +850,20 @@ ml_get_word_separators(void)
 	return  word_separators ;
 }
 
+void
+ml_set_regard_uri_as_word(
+	int  flag
+	)
+{
+	regard_uri_as_word = flag ;
+}
+
+int
+ml_get_regard_uri_as_word(void)
+{
+	return  regard_uri_as_word ;
+}
+
 
 ml_screen_t *
 ml_screen_new(
@@ -1347,6 +1367,11 @@ ml_screen_backscroll_upward(
 		size = screen->backscroll_rows ;
 	}
 
+	if( size == 0)
+	{
+		return  0 ;
+	}
+
 	screen->backscroll_rows -= size ;
 
 	if( ! screen->screen_listener ||
@@ -1406,6 +1431,11 @@ ml_screen_backscroll_downward(
 		size = ml_get_num_of_logged_lines( &screen->logs) - screen->backscroll_rows ;
 	}
 
+	if( size == 0)
+	{
+		return  0 ;
+	}
+
 	screen->backscroll_rows += size ;
 
 	if( ! screen->screen_listener ||
@@ -1617,6 +1647,60 @@ ml_screen_get_word_region(
 		return  0 ;
 	}
 
+	if( regard_uri_as_word)
+	{
+		char *  orig ;
+		u_int  len ;
+		ml_char_t *  str ;
+
+		orig = word_separators ;
+		word_separators = "\" '`<>|" ;
+		regard_uri_as_word = 0 ;
+
+		/* Not return 0 */
+		ml_screen_get_word_region( screen , beg_char_index , beg_row ,
+			end_char_index , end_row , base_char_index , base_row) ;
+
+		word_separators = orig ;
+		regard_uri_as_word = 1 ;
+
+		if( ( len = check_or_copy_region( screen , NULL , 0 ,
+				*beg_char_index , *beg_row ,
+				*end_char_index , *end_row)) > 0 &&
+		    ( str = ml_str_alloca( len)))
+		{
+			u_int  count ;
+			int  slash_num ;
+
+			check_or_copy_region( screen , str , len , *beg_char_index , *beg_row ,
+				*end_char_index , *end_row) ;
+
+			for( slash_num = 0 , count = 0 ; count < len ; count++)
+			{
+				if( ml_char_cs( str + count) == US_ASCII)
+				{
+					switch( ml_char_code( str + count))
+					{
+					case  '/':
+						if( ++ slash_num == 1)
+						{
+							continue ;
+						}
+
+						/* Fall through */
+
+					case  '@':
+						ml_str_final( str , len) ;
+
+						return  1 ;
+					}
+				}
+			}
+
+			ml_str_final( str , len) ;
+		}
+	}
+
 	if( is_word_separator( ml_char_at( base_line , base_char_index)))
 	{
 		*beg_char_index = base_char_index ;
@@ -2585,3 +2669,58 @@ ml_screen_enable_blinking(
 {
 	screen->has_blinking_char = 1 ;
 }
+
+int
+ml_screen_write_content(
+	ml_screen_t *  screen ,
+	int  fd ,
+	mkf_conv_t *  conv ,
+	int  clear_at_end
+	)
+{
+	int  beg ;
+	int  end ;
+	ml_char_t *  buf ;
+	u_int  num ;
+	u_char  conv_buf[512] ;
+	mkf_parser_t *  ml_str_parser ;
+
+	ml_screen_logical( screen) ;
+
+	beg = - ml_screen_get_num_of_logged_lines( screen) ;
+	end = ml_screen_get_rows( screen) ;
+
+	num = ml_screen_get_region_size( screen , 0 , beg , 0 , end , 0) ;
+
+	if( ( buf = ml_str_alloca( num)) == NULL)
+	{
+		return  0 ;
+	}
+
+	ml_screen_copy_region( screen , buf , num , 0 , beg , 0 , end , 0) ;
+
+	if( ! ( ml_str_parser = ml_str_parser_new()))
+	{
+		return  0 ;
+	}
+
+	(*ml_str_parser->init)( ml_str_parser) ;
+	ml_str_parser_set_str( ml_str_parser , buf , num) ;
+	(*conv->init)( conv) ;
+
+	while( ! ml_str_parser->is_eos &&
+	       ( num = (*conv->convert)( conv , conv_buf ,
+			sizeof( conv_buf) , ml_str_parser)) > 0)
+	{
+		write( fd , conv_buf , num) ;
+	}
+
+	if( clear_at_end)
+	{
+		write( fd , "\n\x1b[1000S\x1b[H" , 11) ;
+	}
+
+	(*ml_str_parser->delete)( ml_str_parser) ;
+
+	return  1 ;
+}
