@@ -8,6 +8,7 @@
 
 #include  <time.h>
 #include  <kiklib/kik_types.h>		/* int8_t */
+#include  <mkf/mkf_conv.h>
 
 #include  "ml_edit.h"
 #include  "ml_logs.h"
@@ -78,17 +79,20 @@ typedef struct  ml_screen
 	ml_bs_mode_t  backscroll_mode ;
 	int8_t  is_backscrolling ;
 
-	int8_t  use_dynamic_comb ;	/* public */
 	int8_t  is_cursor_visible ;
 	int8_t  has_blinking_char ;
 
 } ml_screen_t ;
 
 
-int  ml_set_word_separators( char *  seps) ;
+int  ml_set_word_separators( const char *  seps) ;
 
 char *  ml_get_word_separators(void) ;
 
+void  ml_set_regard_uri_as_word( int  flag) ;
+
+int  ml_get_regard_uri_as_word(void) ;
+
 #define  ml_free_word_separators()  ml_set_word_separators(NULL)
 
 
@@ -369,7 +373,12 @@ int  ml_screen_fill_area( ml_screen_t *  screen , int  code ,
 #define  ml_screen_set_use_rect_attr_select( screen , use) \
 	ml_edit_set_use_rect_attr_select( (screen)->edit , use)
 
+#define  ml_screen_clear_size_attr( screen)   ml_edit_clear_size_attr( (screen)->edit)
+
 void  ml_screen_enable_blinking( ml_screen_t *  screen) ;
 
+int  ml_screen_write_content( ml_screen_t *  screen , int  fd , mkf_conv_t *  conv ,
+		int  clear_at_end) ;
+
 
 #endif
