@@ -10,8 +10,11 @@
 
 #include  "ml_char.h"
 #include  "ml_iscii.h"
+#include  "ml_line.h"	/* ctl_info_t */
 
 
+#define  IS_ARABIC_CHAR(ucode)  (((ucode) & 0xffffff00) == 0x600)
+
 #if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 
 u_int  ml_shape_arabic( ml_char_t *  dst , u_int  dst_len , ml_char_t *  src , u_int  src_len) ;
@@ -35,5 +38,8 @@ u_int  ml_shape_iscii( ml_char_t *  dst , u_int  dst_len , ml_char_t *  src , u_
 
 #endif
 
+u_int  ml_shape_ot_layout( ml_char_t *  dst , u_int  dst_len ,
+		ml_char_t *  src , u_int  src_len , ctl_info_t  ctl_info) ;
+
 
 #endif	/* __ML_SHAPING_H__ */
