@@ -95,20 +95,38 @@ sig_error(
 
 	if( len > 0)
 	{
-		if( fork() > 0)
+		pid_t  pid ;
+
+		pid = fork() ;
+
+		if( pid < 0)
+		{
+			return ;
+		}
+
+		if( pid == 0)
 		{
 			/* child process */
-			kik_setenv( "INHERIT_PTY_LIST" , env , 1) ;
-
-			if( auto_restart_cmd)
+			for( count = 0 ; count < num_of_terms ; count++)
 			{
-				execlp( auto_restart_cmd , auto_restart_cmd , NULL) ;
+				ml_term_write_content( terms[count] ,
+					ml_term_get_slave_fd( terms[count]) ,
+					terms[count]->parser->cc_conv , 1) ;
 			}
 
-			execl( BINDIR "/mlterm" , BINDIR "/mlterm" , NULL) ;
-
-			kik_error_printf( "Failed to restart mlterm.\n") ;
+			exit(0) ;
 		}
+
+		kik_setenv( "INHERIT_PTY_LIST" , env , 1) ;
+
+		if( auto_restart_cmd)
+		{
+			execlp( auto_restart_cmd , auto_restart_cmd , NULL) ;
+		}
+
+		execl( BINDIR "/mlterm" , BINDIR "/mlterm" , NULL) ;
+
+		kik_error_printf( "Failed to restart mlterm.\n") ;
 	}
 
 	exit(1) ;
@@ -198,6 +216,11 @@ ml_term_manager_final(void)
 {
 	int  count ;
 
+#ifdef  USE_OT_LAYOUT
+	ml_set_ot_layout_attr( NULL , OT_SCRIPT) ;
+	ml_set_ot_layout_attr( NULL , OT_FEATURES) ;
+#endif
+
 	kik_remove_sig_child_listener( NULL , sig_child) ;
 	ml_term_final() ;
 
@@ -302,7 +325,8 @@ ml_create_term(
 	int  use_local_echo ,
 	const char *  win_name ,
 	const char *  icon_name ,
-	ml_alt_color_mode_t  alt_color_mode
+	ml_alt_color_mode_t  alt_color_mode ,
+	int  use_ot_layout
 	)
 {
 #if  ! defined(USE_WIN32API) && ! defined(DEBUG)
@@ -342,7 +366,7 @@ ml_create_term(
 						use_ctl , bidi_mode , bidi_separators ,
 						use_dynamic_comb , bs_mode , vertical_mode ,
 						use_local_echo , win_name , icon_name ,
-						alt_color_mode)))
+						alt_color_mode , use_ot_layout)))
 					{
 						ml_term_plug_pty( terms[num_of_terms++] , pty) ;
 						ml_set_pty_winsize( pty , cols , rows , 0 , 0) ;
@@ -386,7 +410,8 @@ ml_create_term(
 				logging_vt_seq , policy , col_size_a , use_char_combining ,
 				use_multi_col_char , use_ctl , bidi_mode , bidi_separators ,
 				use_dynamic_comb , bs_mode , vertical_mode ,
-				use_local_echo , win_name , icon_name , alt_color_mode)))
+				use_local_echo , win_name , icon_name , alt_color_mode ,
+				use_ot_layout)))
 	{
 		return  NULL ;
 	}
