@@ -35,6 +35,7 @@ typedef enum  ml_unicode_policy
 	NOT_USE_UNICODE_BOXDRAW_FONT = 0x4 ,
 	ONLY_USE_UNICODE_BOXDRAW_FONT = 0x8 ,
 	USE_UNICODE_DRCS = 0x10 ,
+	CONVERT_UNICODE_TO_ISCII = 0x20 ,
 
 	UNICODE_POLICY_MAX
 
@@ -151,7 +152,7 @@ typedef struct  ml_config_event_listener
 	void (*saved)(void) ;		/* Event that mlterm/main file was changed. */
 	
 	void (*set_font)( void * , char * , char * , char * , int) ;
-	void (*get_font)( void * , char * , char * , char * , int) ;
+	void (*get_font)( void * , char * , char * , int) ;
 	void (*set_color)( void * , char * , char * , char * , int) ;
 	void (*get_color)( void * , char * , int) ;
 
@@ -196,9 +197,10 @@ typedef struct  ml_vt100_parser
 	ml_write_buffer_t  w_buf ;
 	
 	ml_pty_ptr_t  pty ;
+	ml_pty_hook_t  pty_hook ;
 
 	ml_screen_t *  screen ;
-	ml_termcap_entry_t *  termcap ;
+	ml_termcap_ptr_t  termcap ;
 
 	mkf_parser_t *  cc_parser ;	/* char code parser */
 	mkf_conv_t *  cc_conv ;		/* char code converter */
@@ -269,8 +271,6 @@ typedef struct  ml_vt100_parser
 #endif
 	int8_t  modify_other_keys ;
 
-	int8_t  use_ctl ;
-
 #ifdef  USE_VT52
 	int8_t  is_vt52_mode ;
 #endif
@@ -315,12 +315,18 @@ void  ml_set_use_scp_full( int  use) ;
 #define  ml_set_use_scp_full(use)  (0)
 #endif
 
+void  ml_set_timeout_read_pty( u_long  timeout) ;
+
+void  ml_set_primary_da( char *  da) ;
+
+void  ml_set_secondary_da( char *  da) ;
+
 void  ml_vt100_parser_init(void) ;
 
 void  ml_vt100_parser_final(void) ;
 
 ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen ,
-	ml_termcap_entry_t *  termcap , ml_char_encoding_t  encoding ,
+	ml_termcap_ptr_t  termcap , ml_char_encoding_t  encoding ,
 	int  is_auto_encoding , int  use_auto_detect , int  logging_vt_seq ,
 	ml_unicode_policy_t  policy , u_int  col_size_a ,
 	int  use_char_combining , int  use_multi_col_char ,
@@ -354,7 +360,7 @@ int  ml_vt100_parser_write_loopback( ml_vt100_parser_t *  vt100_parser ,
 /* Must be called in visual context. */
 int  ml_vt100_parser_show_message( ml_vt100_parser_t *  vt100_parser , char *  msg) ;
 
-#ifdef  __ANDROID__
+#if  defined(__ANDROID__) || defined(__APPLE__)
 /* Must be called in visual context. */
 int  ml_vt100_parser_preedit( ml_vt100_parser_t *  vt100_parser ,
 	const u_char *  buf , size_t  len) ;
@@ -404,8 +410,6 @@ int  ml_init_encoding_conv( ml_vt100_parser_t *  vt100_parser) ;
 
 #define  ml_vt100_parser_get_unicode_policy( vt100_parser)  ((vt100_parser)->unicode_policy)
 
-#define  ml_vt100_parser_set_use_ctl( vt100_parser , use)  ((vt100_parser)->use_ctl = (use))
-
 int  ml_set_auto_detect_encodings( char *  encodings) ;
 
 int  ml_convert_to_internal_ch( mkf_char_t *  ch , ml_unicode_policy_t  unicode_policy ,
