@@ -29,6 +29,7 @@
 #include  "mc_ratio.h"
 #include  "mc_radio.h"
 #include  "mc_char_width.h"
+#include  "mc_geometry.h"
 
 
 #if  0
@@ -73,11 +74,10 @@ update(
 		mc_update_alpha() ;
 		mc_update_bgtype() ;
 	}
-	mc_update_color(MC_COLOR_SBFG) ;
-	mc_update_color(MC_COLOR_SBBG) ;
 	mc_update_tabsize() ;
 	mc_update_logsize() ;
 	mc_update_wordsep() ;
+	mc_update_geometry() ;
 	mc_update_font_misc() ;
 	mc_update_space(MC_SPACE_LINE) ;
 	mc_update_space(MC_SPACE_LETTER) ;
@@ -85,13 +85,11 @@ update(
 	mc_update_ratio(MC_RATIO_SCREEN_HEIGHT) ;
 	mc_update_radio(MC_RADIO_MOD_META_MODE) ;
 	mc_update_radio(MC_RADIO_BELL_MODE) ;
-	mc_update_radio(MC_RADIO_SB_MODE) ;
 	mc_update_radio(MC_RADIO_LOG_VTSEQ) ;
 	mc_update_ratio(MC_RATIO_BRIGHTNESS) ;
 	mc_update_ratio(MC_RATIO_CONTRAST) ;
 	mc_update_ratio(MC_RATIO_GAMMA) ;
 	mc_update_ratio(MC_RATIO_FADE) ;
-	mc_update_sb_view_name() ;
 	mc_update_im() ;
 	mc_update_cursor_color() ;
 	mc_update_substitute_color() ;
@@ -105,6 +103,16 @@ update(
 	mc_update_flag_mode(MC_FLAG_LOCALECHO) ;
 	mc_update_flag_mode(MC_FLAG_BLINKCURSOR) ;
 	mc_update_flag_mode(MC_FLAG_STATICBACKSCROLL) ;
+	mc_update_flag_mode(MC_FLAG_EXTSCROLLSHORTCUT) ;
+	mc_update_flag_mode(MC_FLAG_REGARDURIASWORD) ;
+	mc_update_flag_mode(MC_FLAG_OTLAYOUT) ;
+
+	mc_update_radio(MC_RADIO_SB_MODE) ;
+#ifndef  USE_QUARTZ
+	mc_update_color(MC_COLOR_SBFG) ;
+	mc_update_color(MC_COLOR_SBBG) ;
+	mc_update_sb_view_name() ;
+#endif
 
 	mc_flush(io) ;
 
@@ -401,6 +409,24 @@ pty_button_clicked(
 	return  1 ;
 }	
 
+#if  defined(UES_WIN32GUI) || defined(USE_QUARTZ)
+static gboolean
+event(
+	GtkWidget *  widget ,
+	GdkEvent *  event ,
+	gpointer  data
+	)
+{
+	if( event->type == GDK_FOCUS_CHANGE && ! ((GdkEventFocus*)event)->in)
+	{
+		gtk_window_set_keep_above( GTK_WINDOW(widget) , FALSE) ;
+		g_signal_handlers_disconnect_by_func( widget , event , NULL) ;
+	}
+
+	return  FALSE ;
+}
+#endif
+
 /*
  *  ********  Building GUI (lower part, independent buttons)  ********
  */
@@ -625,12 +651,21 @@ show(void)
 	gtk_widget_show( config_widget) ;
 	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
 
+	config_widget = mc_flag_config_widget_new( MC_FLAG_DYNCOMB) ;
+	gtk_widget_show( config_widget) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
+
 
 	config_widget = mc_flag_config_widget_new( MC_FLAG_RECVUCS) ;
 	gtk_widget_show( config_widget) ;
 	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
 
 
+	config_widget = mc_flag_config_widget_new( MC_FLAG_OTLAYOUT) ;
+	gtk_widget_show( config_widget) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
+
+
 	/* contents of the "Font" tab */
 
 	label = gtk_label_new( _("Font")) ;
@@ -769,6 +804,7 @@ show(void)
 	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
 
 
+#ifndef  USE_QUARTZ
 	config_widget = mc_sb_view_config_widget_new() ;
 	gtk_widget_show( config_widget) ;
 	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
@@ -785,6 +821,7 @@ show(void)
 	config_widget = mc_color_config_widget_new( MC_COLOR_SBBG) ;
 	gtk_widget_show( config_widget) ;
 	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
+#endif
 
 
 	/* contents of the "Others" tab */
@@ -810,9 +847,17 @@ show(void)
 	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
 
 
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , FALSE , FALSE , 0) ;
+
+	config_widget = mc_geometry_config_widget_new() ;
+	gtk_widget_show( config_widget) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
+
 	config_widget = mc_wordsep_config_widget_new() ;
 	gtk_widget_show( config_widget) ;
-	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , config_widget , FALSE , FALSE , 0) ;
 
 
 	config_widget = mc_radio_config_widget_new( MC_RADIO_MOD_META_MODE) ;
@@ -835,11 +880,6 @@ show(void)
 	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
 
 
-	config_widget = mc_flag_config_widget_new( MC_FLAG_DYNCOMB) ;
-	gtk_widget_show( config_widget) ;
-	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
-
-
 	hbox = gtk_hbox_new( FALSE , 0) ;
 	gtk_widget_show( hbox) ;
 	gtk_box_pack_start( GTK_BOX(vbox) , hbox , FALSE , FALSE , 0) ;
@@ -864,12 +904,28 @@ show(void)
 	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
 
 
+	config_widget = mc_flag_config_widget_new( MC_FLAG_EXTSCROLLSHORTCUT) ;
+	gtk_widget_show( config_widget) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
+
+
+	config_widget = mc_flag_config_widget_new( MC_FLAG_REGARDURIASWORD) ;
+	gtk_widget_show( config_widget) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , config_widget , FALSE , FALSE , 0) ;
+
+
 	gtk_window_set_position( GTK_WINDOW(window) , GTK_WIN_POS_MOUSE) ;
 #if ! GTK_CHECK_VERSION(2 ,90 ,0)
 	gtk_window_set_policy( GTK_WINDOW(window) , 0 , 0 , 0) ;
 #endif
+
 	gtk_widget_show( window) ;
 
+#if  defined(UES_WIN32GUI) || defined(USE_QUARTZ)
+	gtk_window_set_keep_above( GTK_WINDOW(window) , TRUE) ;
+	g_signal_connect( window , "event" , G_CALLBACK(event) , NULL) ;
+#endif
+
 	gtk_main() ;
 
 	return  1 ;
