@@ -38,6 +38,9 @@ static char *  config_keys[MC_FLAG_MODES] =
 	"use_local_echo" ,
 	"blink_cursor" ,
 	"static_backscroll_mode" ,
+	"use_extended_scroll_shortcut" ,
+	"regard_uri_as_word" ,
+	"use_ot_layout" ,
 } ;
 
 static char *labels[MC_FLAG_MODES] =
@@ -56,6 +59,9 @@ static char *labels[MC_FLAG_MODES] =
 	N_("Local echo"),
 	N_("Blink cursor"),
 	N_("Don't scroll automatically in scrolling back"),
+	N_("Scroll by Shift+Up or Shift+Down"),
+	N_("Select URI by double click"),
+	N_("OpenType Layout"),
 } ;
 
 static GtkWidget *  widgets[MC_FLAG_MODES];
