@@ -12,7 +12,7 @@
 #include  <glib.h>
 #include  <c_intl.h>
 
-#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0)
+#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0) && ! defined(USE_QUARTZ)
 #include  "gtkxlfdsel.h"
 #endif
 
@@ -60,10 +60,20 @@ static cs_info_t  cs_info_table[] =
 	 * UNICODE => ISO10646_UCS4_1 or U+XXXX-XXXX in get_correct_cs().
 	 */
 	{ "UNICODE" , { "iso10646-1" , NULL , NULL , } , } ,
-	{ "UNICODE (FULLWIDTH)" , { "iso10646-1" , NULL , NULL , } , } ,
-	{ "UNICODE (ARABIC)" , { "iso10646-1" , NULL , NULL , } , } ,
-	{ "UNICODE (EMOJI)" , { "iso10646-1" , NULL , NULL , } , } ,
-	{ "UNICODE (HANKAKU KANA)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Full Width)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Emoji)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Hankaku Kana)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Hebrew)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Arabic)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Hindi)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Bengali)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Punjabi)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Gujarati)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Oriya)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Tamil)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Telugu)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Kannada)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (Malayalam)" , { "iso10646-1" , NULL , NULL , } , } ,
 
 	{ "DEC_SPECIAL" , { "iso8859-1" , NULL , NULL , } , } ,
 	{ "ISO8859_1" , { "iso8859-1" , NULL , NULL , } , } ,
@@ -98,7 +108,6 @@ static cs_info_t  cs_info_table[] =
 	{ "ISCII_MALAYALAM" , { NULL , NULL , NULL , } , } ,
 	{ "ISCII_ORIYA" , { NULL , NULL , NULL , } , } ,
 	{ "ISCII_PUNJABI" , { NULL , NULL , NULL , } , } ,
-	{ "ISCII_ROMAN" , { NULL , NULL , NULL , } , } ,
 	{ "ISCII_TAMIL" , { NULL , NULL , NULL , } , } ,
 	{ "ISCII_TELUGU" , { NULL , NULL , NULL , } , } ,
 	{ "VISCII" , { "viscii-1" , NULL , NULL , } , } ,
@@ -197,34 +206,37 @@ reset_fontname_list(void)
  * If you use functions in mc_io.c, use this function instead of direct
  * access to cs_info_t::cs.
  */
-static char *
+static const char *
 get_correct_cs(
 	int  idx
 	)
 {
+	const char *  unicode_names[] =
+	{
+		"ISO10646_UCS4_1" ,
+		"ISO10646_UCS4_1_FULLWIDTH" ,
+		"U+1F000-1F77F" ,	/* Emoji */
+		"U+FF61-FF9F" ,		/* Hankaku Kana */
+		"U+0590-05FF" ,		/* Hebrew */
+		"U+0600-06FF" ,		/* Arabic */
+		"U+0900-097F" ,		/* Hindi */
+		"U+0980-09FF" ,		/* Bengali */
+		"U+0A00-0A7F" ,		/* Punjabi */
+		"U+0A80-0AFF" ,		/* Gujarati */
+		"U+0B00-0B7F" ,		/* Oriya */
+		"U+0B80-0BFF" ,		/* Tamil */
+		"U+0C00-0C7F" ,		/* Telugu */
+		"U+0C80-0CFF" ,		/* Kannada */
+		"U+0D00-0D7F" ,		/* Malayalam */
+	} ;
+
 	if( idx < 0)
 	{
 		return  NULL ;
 	}
-	else if( idx == 1)
+	else if( 1 <= idx && idx <= 15)
 	{
-		return  "ISO10646_UCS4_1" ;
-	}
-	else if( idx == 2)
-	{
-		return  "ISO10646_UCS4_1_FULLWIDTH" ;
-	}
-	else if( idx == 3)
-	{
-		return  "U+0600-06FF" ;	/* Arabic */
-	}
-	else if( idx == 4)
-	{
-		return  "U+1F000-1F77F" ;	/* Emoji */
-	}
-	else if( idx == 5)
-	{
-		return  "U+FF61-FF9F" ;	/* Hankaku Kana */
+		return  unicode_names[idx - 1] ;
 	}
 	else if( idx < sizeof(cs_info_table) / sizeof(cs_info_table[0]))
 	{
@@ -288,7 +300,7 @@ aa_flag_checked(
 			reset_fontname_list() ;
 			gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 				g_locale_to_utf8(
-					mc_get_font_name( get_font_file() , new_fontsize ,
+					mc_get_font_name( get_font_file() ,
 						get_correct_cs( selected_cs)) ,
 					-1 , NULL , NULL , NULL) ) ;
 		}
@@ -318,7 +330,7 @@ xft_flag_checked(
 	reset_fontname_list() ;
 	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
+			mc_get_font_name( get_font_file() ,
 						get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
 
@@ -346,7 +358,7 @@ cairo_flag_checked(
 	reset_fontname_list() ;
 	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
+			mc_get_font_name( get_font_file() ,
 						get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
 
@@ -362,7 +374,7 @@ vcol_flag_checked(
 	reset_fontname_list() ;
 	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
+			mc_get_font_name( get_font_file() ,
 						get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
 
@@ -375,7 +387,7 @@ vertical_mode_changed(void)
 	reset_fontname_list() ;
 	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
+			mc_get_font_name( get_font_file() ,
 						get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
 
@@ -399,13 +411,6 @@ fontsize_selected(
 	g_free( new_fontsize);
 	new_fontsize = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 
-	reset_fontname_list() ;
-	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
-		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
-						get_correct_cs( selected_cs)) ,
-			-1 , NULL , NULL , NULL) ) ;
-	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s font size is selected.\n" , new_fontsize) ;
 #endif
@@ -413,6 +418,100 @@ fontsize_selected(
 	return  1 ;
 }
 
+static void
+specify_width(
+	GtkWidget *  widget ,
+	int  flag
+	)
+{
+	gchar *  fontname ;
+
+	if( ( ( fontname = new_fontname_list[selected_cs]) ||
+	      ( fontname = gtk_entry_get_text(GTK_ENTRY(fontname_entry)))) &&
+	    *fontname)
+	{
+		gchar *  p ;
+		int  percent ;
+		size_t  len ;
+
+		if( ! ( p = strrchr( fontname , ':')) ||
+		    ( percent = atoi( p + 1)) == 0)
+		{
+			if( flag == 0)
+			{
+				return ;
+			}
+
+			len = strlen( fontname) ;
+			percent = (flag < 0 ? 100 : 110) ;
+		}
+		else
+		{
+			len = p - fontname ;
+
+			if( flag == 0)
+			{
+				percent = 0 ;
+			}
+			else
+			{
+				percent += (flag < 0 ? -10 : 10) ;
+
+				if( percent <= 0 || 200 < percent)
+				{
+					return ;
+				}
+			}
+		}
+
+		if( ( p = malloc( len + 5)))
+		{
+			strncpy( p , fontname , len) ;
+			if( percent == 0)
+			{
+				p[len] = '\0' ;
+			}
+			else
+			{
+				sprintf( p + len , ":%d" , percent) ;
+			}
+			free( new_fontname_list[selected_cs]) ;
+			new_fontname_list[selected_cs] = p ;
+
+			dont_change_new_fontname_list = 1 ;
+			gtk_entry_set_text( GTK_ENTRY(fontname_entry) , p) ;
+			dont_change_new_fontname_list = 0 ;
+		}
+	}
+}
+
+static void
+widen_width(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	specify_width( widget , 1) ;
+}
+
+static void
+narrow_width(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	specify_width( widget , -1) ;
+}
+
+static void
+default_width(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	specify_width( widget , 0) ;
+}
+
 static void
 fontcs_changed(void)
 {
@@ -428,7 +527,6 @@ fontcs_changed(void)
 		gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
 			g_locale_to_utf8(
 				mc_get_font_name( get_font_file() ,
-					new_fontsize ,
 					get_correct_cs(selected_cs)) ,
 				-1 , NULL , NULL , NULL) ) ;
 	}
@@ -577,7 +675,7 @@ fontcs_selected(
 	return  0 ;
 }
 
-#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0)
+#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0) && ! defined(USE_QUARTZ)
 
 static gchar *
 get_xlfd_font_name(
@@ -876,7 +974,7 @@ select_font(
 	gpointer  p
 	)
 {
-#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0)
+#if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0) && ! defined(USE_QUARTZ)
 	if( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(xft_flag)) &&
 	    ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(cairo_flag)))
 	{
@@ -930,6 +1028,7 @@ mc_font_config_widget_new(void)
 	GtkWidget *  entry ;
 	GtkWidget *  radio ;
 	GtkWidget *  label ;
+	GtkWidget *  button ;
 	char *  fontlist[] =
 	{
 		"6" , "7" , "8" , "9" , "10" ,
@@ -949,7 +1048,7 @@ mc_font_config_widget_new(void)
 
 	combo = mc_combo_new_with_width(_("Font size (pixels)") , fontlist ,
 		sizeof(fontlist) / sizeof(fontlist[0]) , new_fontsize , 1 ,
-		50 , &entry) ;
+		30 , &entry) ;
 	g_signal_connect( entry , "changed" , G_CALLBACK(fontsize_selected) , NULL) ;
 	gtk_box_pack_start( GTK_BOX(hbox) , combo , FALSE , FALSE , 0) ;
 #if  GTK_CHECK_VERSION(2,12,0)
@@ -1036,7 +1135,7 @@ mc_font_config_widget_new(void)
 	fontname_entry = gtk_entry_new() ;
 	gtk_entry_set_text( GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
-			mc_get_font_name( get_font_file() , new_fontsize ,
+			mc_get_font_name( get_font_file() ,
 					get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
 	gtk_widget_show( fontname_entry) ;
@@ -1052,6 +1151,32 @@ mc_font_config_widget_new(void)
 
 	gtk_box_pack_start( GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
 
+
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
+
+	label = gtk_label_new( _("Font width")) ;
+	gtk_widget_show( label) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , label , FALSE , FALSE , 5) ;
+
+	button = gtk_button_new_with_label( _("Narrow")) ;
+	gtk_widget_show( button) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , button , FALSE , FALSE , 5) ;
+	g_signal_connect( button , "clicked" , G_CALLBACK(narrow_width) , NULL) ;
+
+	button = gtk_button_new_with_label( _("Widen")) ;
+	gtk_widget_show( button) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , button , FALSE , FALSE , 5) ;
+	g_signal_connect( button , "clicked" , G_CALLBACK(widen_width) , NULL) ;
+
+	button = gtk_button_new_with_label( _("Default")) ;
+	gtk_widget_show( button) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , button , FALSE , FALSE , 5) ;
+	g_signal_connect( button , "clicked" , G_CALLBACK(default_width) , NULL) ;
+
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
+
+
 	radio = mc_radio_config_widget_new( MC_RADIO_FONT_POLICY) ;
 	gtk_widget_show( radio) ;
 	gtk_box_pack_start( GTK_BOX(vbox) , radio , FALSE , FALSE , 0) ;
@@ -1122,25 +1247,25 @@ mc_update_font_name(mc_io_t  io)
 	{
 		if( new_fontname_list[count])
 		{
-			mc_set_font_name( io , get_font_file() , new_fontsize ,
+			mc_set_font_name( io , get_font_file() ,
 				get_correct_cs( count) , new_fontname_list[count]) ;
 
-			if( count == 3)
+			if( count == 6)
 			{
 				/* Arabic Supplement */
-				mc_set_font_name( io , get_font_file() , new_fontsize ,
+				mc_set_font_name( io , get_font_file() ,
 					"U+0750-77F" , new_fontname_list[count]) ;
 				/* Arabic Extended-A */
-				mc_set_font_name( io , get_font_file() , new_fontsize ,
+				mc_set_font_name( io , get_font_file() ,
 					"U+08A0-8FF" , new_fontname_list[count]) ;
 				/* Arabic Presentation Forms-A */
-				mc_set_font_name( io , get_font_file() , new_fontsize ,
+				mc_set_font_name( io , get_font_file() ,
 					"U+FB50-FDFF" , new_fontname_list[count]) ;
 				/* Arabic Presentation Forms-B */
-				mc_set_font_name( io , get_font_file() , new_fontsize ,
+				mc_set_font_name( io , get_font_file() ,
 					"U+FE70-FEFF" , new_fontname_list[count]) ;
 				/* Arabic Mathematical Alphabetic Symbols */
-				mc_set_font_name( io , get_font_file() , new_fontsize ,
+				mc_set_font_name( io , get_font_file() ,
 					"U+1EE00-1EEFF" , new_fontname_list[count]) ;
 			}
 		}
