@@ -34,10 +34,10 @@ int  mc_get_flag_value(const char *  key) ;
 
 int  mc_gui_is_win32(void) ;
 
-int  mc_set_font_name(mc_io_t  io , const char *  file , const char *  font_size ,
-	const char *  cs , const char *  font_name) ;
+int  mc_set_font_name(mc_io_t  io , const char *  file , const char *  cs ,
+	const char *  font_name) ;
 
-char *  mc_get_font_name(const char *  file , const char *  font_size , const char *  cs) ;
+char *  mc_get_font_name(const char *  file , const char *  cs) ;
 
 int  mc_set_color_name(mc_io_t  io , const char *  color , const char *  value) ;
 
