@@ -21,7 +21,7 @@
 #include <kiklib/kik_types.h>	/* u_int32_t/u_int16_t */
 #include <kiklib/kik_def.h>	/* SSIZE_MAX, USE_WIN32API */
 #if  defined(__CYGWIN__) || defined(__MSYS__)
-#include <kiklib/kik_path.h>	/* cygwin_conv_to_win32_path */
+#include <kiklib/kik_path.h>	/* kik_conv_to_win32_path */
 #endif
 
 #ifdef  USE_WIN32API
@@ -44,7 +44,7 @@
 /* --- static functions --- */
 
 #ifndef  USE_WIN32GUI
-#define  USE_X11	/* Necessary to use closest_color_index(), lsb() and msb() */
+#define  USE_XLIB	/* Necessary to use closest_color_index(), lsb() and msb() */
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #endif
@@ -115,7 +115,7 @@ load_file(
 	return  pixbuf ;
 }
 
-#ifdef  USE_X11
+#ifdef  USE_XLIB
 
 /* returned cmap shuold be freed by the caller */
 static int
@@ -531,7 +531,7 @@ pixbuf_to_pixmap_and_mask(
 	return  1 ;
 }
 
-#endif	/* USE_X11 */
+#endif	/* USE_XLIB */
 
 
 /* --- global functions --- */
@@ -542,7 +542,7 @@ main(
 	char **  argv
 	)
 {
-#ifdef  USE_X11
+#ifdef  USE_XLIB
 	Display *  display ;
 	Visual *  visual ;
 	Colormap  colormap ;
@@ -561,7 +561,7 @@ main(
 	kik_set_msg_log_file_name( "mlterm/msg.log") ;
 #endif
 
-#ifdef  USE_X11
+#ifdef  USE_XLIB
 	if( argc == 5)
 	{
 		if( ! ( display = XOpenDisplay( NULL)))
@@ -614,9 +614,8 @@ main(
 	#if  defined(__CYGWIN__) || defined(__MSYS__)
 	#define  MAX_PATH  260	/* 3+255+1+1 */
 		char  winpath[MAX_PATH] ;
-		cygwin_conv_to_win32_path( argv[4] , winpath) ;
-
-		if( ! ( pixbuf = load_file( winpath , width , height , GDK_INTERP_BILINEAR)))
+		if( kik_conv_to_win32_path( argv[4] , winpath , sizeof(winpath)) < 0 ||
+		    ! ( pixbuf = load_file( winpath , width , height , GDK_INTERP_BILINEAR)))
 	#endif
 		{
 		#ifdef  DEBUG
@@ -681,7 +680,7 @@ main(
 			size -= n_wr ;
 		}
 	}
-#ifdef  USE_X11
+#ifdef  USE_XLIB
 	else
 	{
 		char  buf[10] ;
@@ -706,7 +705,7 @@ main(
 		/* Wait for parent process receiving pixmap. */
 		read( STDIN_FILENO , buf , sizeof(buf)) ;
 	}
-#endif	/* USE_X11 */
+#endif	/* USE_XLIB */
 
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " Exit image loader\n") ;
