@@ -210,7 +210,7 @@ typedef struct
 	unsigned char  height ;
 	unsigned char  ascent ;
 
-	int16_t *  glyph_indeces ;
+	u_int16_t *  glyph_indeces ;
 
 	/* for pcf */
 	int16_t  min_char_or_byte2 ;
@@ -395,9 +395,9 @@ typedef int XFontSet ;	/* dummy */
 #define IsKeypadKey(ksym)	(0xf200 <= (ksym) && (ksym) < 0xf300)
 #define IsModifierKey(ksym)      (KS_Shift_L <= (ksym) && (ksym) <= KS_Alt_R)
 
-#else	/* if __FreeBSD__ || __Linux__ */
+#else	/* if __FreeBSD__ || __linux__ */
 
-#if  defined(__FreeBSD__)
+#ifndef  __linux__	/* FreeBSD */
 
 #define KEY_CLEAR	0xff	/* dummy */
 #define KEY_LINEFEED	0xfe	/* dummy */
@@ -631,4 +631,20 @@ int  XParseGeometry( char *  str , int *  x , int *  y ,
 KeySym  XStringToKeysym( char *  str) ;
 
 
+/* === Platform dependent options === */
+
+#define  X_COLOR_HAS_RGB
+#undef  SUPPORT_TRUE_TRANSPARENT_BG
+#ifdef  USE_FREETYPE
+/* XXX pcf fonts isn't scalable, though... */
+#define  TYPE_XCORE_SCALABLE
+#else
+#undef  TYPE_XCORE_SCALABLE
+#endif
+#define  MANAGE_WINDOWS_BY_MYSELF
+/* See also fb/x_display.c where x_picture_display_closed() is never called. */
+#define  INLINE_PICTURE_MOVABLE_BETWEEN_DISPLAYS
+#undef  SUPPORT_POINT_SIZE_FONT
+
+
 #endif
