@@ -16,6 +16,7 @@
 #include  <kiklib/kik_privilege.h>	/* kik_priv_change_e(u|g)id */
 #include  <kiklib/kik_unistd.h>		/* kik_getuid */
 #include  <kiklib/kik_file.h>
+#include  <kiklib/kik_mem.h>		/* alloca */
 
 #include  <ml_color.h>
 
@@ -844,6 +845,9 @@ draw_mouse_cursor(void)
 	_mouse.cursor.is_drawn = 1 ;
 }
 
+/* XXX defined in fb/x_window.c */
+int  x_window_clear_margin_area( x_window_t *  win) ;
+
 static void
 expose_window(
 	x_window_t *  win ,
@@ -1528,7 +1532,10 @@ x_display_close_all(void)
 {
 	if( DISP_IS_INITED)
 	{
+		/* inline pictures are alive until ml_term_t is deleted. */
+	#if  0
 		x_picture_display_closed( _disp.display) ;
+	#endif
 
 		x_virtual_kbd_hide() ;
 
@@ -2515,9 +2522,10 @@ x_cmap_get_closest_color(
 	u_int  segment ;
 	u_int  offset ;
 	u_int  color ;
-	u_long  min = 0xffffff ;
-	u_long  diff ;
+	u_int  min = 0xffffff ;
+	u_int  diff ;
 	int  diff_r , diff_g , diff_b ;
+	u_int  linear_search_max ;
 
 	if( ! _display.cmap)
 	{
@@ -2560,7 +2568,17 @@ x_cmap_get_closest_color(
 		return  1 ;
 	}
 
-	for( color = 0 ; color < CMAP_SIZE(_display.cmap) ; color++)
+	if( ( linear_search_max = CMAP_SIZE(_display.cmap)) == 256)
+	{
+		if( ( linear_search_max = ml_get_closest_256_color( closest , &min ,
+						red , green , blue ,
+						COLOR_DISTANCE_THRESHOLD)) == 0)
+		{
+			goto  end ;
+		}
+	}
+
+	for( color = 0 ; color < linear_search_max ; color++)
 	{
 	#ifdef  USE_GRF
 		if( grf0_fd != -1 && ! use_tvram_cmap && color == TP_COLOR)
@@ -2573,7 +2591,7 @@ x_cmap_get_closest_color(
 		diff_r = red - WORD_COLOR_TO_BYTE(_display.cmap->red[color]) ;
 		diff_g = green - WORD_COLOR_TO_BYTE(_display.cmap->green[color]) ;
 		diff_b = blue - WORD_COLOR_TO_BYTE(_display.cmap->blue[color]) ;
-		diff = diff_r * diff_r * 9 + diff_g * diff_g * 30 + diff_b * diff_b ;
+		diff = COLOR_DISTANCE( diff_r , diff_g , diff_b) ;
 
 		if( diff < min)
 		{
@@ -2581,13 +2599,14 @@ x_cmap_get_closest_color(
 			*closest = color ;
 
 			/* no one may notice the difference (4[2^3/2]*4*9+4*4*30+4*4) */
-			if( diff < 640)
+			if( diff < COLOR_DISTANCE_THRESHOLD)
 			{
 				break ;
 			}
 		}
 	}
 
+end:
 #ifndef  COLOR_CACHE_MINIMUM
 	_display.color_cache->flags[offset / 32] |= (1 << (offset & 31)) ;
 #else
