@@ -24,6 +24,9 @@
 #include  <dirent.h>
 #endif
 
+#ifdef  USE_OT_LAYOUT
+#include  <otl.h>
+#endif
 
 #define  DIVIDE_ROUNDING(a,b)  ( ((int)((a)*10 + (b)*5)) / ((int)((b)*10)) )
 #define  DIVIDE_ROUNDINGUP(a,b) ( ((int)((a)*10 + (b)*10 - 1)) / ((int)((b)*10)) )
@@ -46,6 +49,10 @@
 #define  PCF_GLYPH_NAMES	(1<<7)
 #define  PCF_BDF_ACCELERATORS	(1<<8)
 
+#define  MAX_GLYPH_TABLES  512
+#define  GLYPH_TABLE_SIZE  128
+#define  INITIAL_GLYPH_INDEX_TABLE_SIZE  0x1000
+
 
 #if  0
 #define  __DEBUG
@@ -679,7 +686,50 @@ static FT_Library  library ;
 /* --- static functions --- */
 
 static int
-load_char(
+get_glyph_index(
+	FT_Face  face ,
+	u_int32_t  code
+	)
+{
+	int  idx ;
+
+	if( ( idx = FT_Get_Char_Index( face , code)) == 0)
+	{
+		/* XXX Some glyph indeces of ISCII fonts becomes 0 wrongly. */
+		if( 0x80 <= code && code <= 0xff)
+		{
+			u_int32_t  prev_idx ;
+			u_int32_t  next_idx ;
+			u_int32_t  c ;
+
+			for( c = code + 1 ; c <= 0xff ; c++)
+			{
+				if( ( next_idx = FT_Get_Char_Index( face , c)) > 0)
+				{
+					for( c = code - 1 ; c >= 80 ; c--)
+					{
+						if( ( prev_idx = FT_Get_Char_Index(
+									face , c)) > 0)
+						{
+							if( prev_idx + 1 < next_idx)
+							{
+								idx = prev_idx + 1 ;
+								break ;
+							}
+						}
+					}
+
+					break ;
+				}
+			}
+		}
+	}
+
+	return  idx ;
+}
+
+static int
+load_glyph(
 	FT_Face  face ,
 	int32_t  format ,
 	u_int32_t  code ,
@@ -688,41 +738,7 @@ load_char(
 {
 	if( is_aa)
 	{
-		u_int32_t  idx ;
-
-		if( ( idx = FT_Get_Char_Index( face , code)) == 0)
-		{
-			/* XXX Some glyph indeces of ISCII fonts becomes 0 wrongly. */
-			if( 0x80 <= code && code <= 0xff)
-			{
-				u_int32_t  prev_idx ;
-				u_int32_t  next_idx ;
-				u_int32_t  c ;
-
-				for( c = code + 1 ; c <= 0xff ; c++)
-				{
-					if( ( next_idx = FT_Get_Char_Index( face , c)) > 0)
-					{
-						for( c = code - 1 ; c >= 80 ; c--)
-						{
-							if( ( prev_idx = FT_Get_Char_Index(
-										face , c)) > 0)
-							{
-								if( prev_idx + 1 < next_idx)
-								{
-									idx = prev_idx + 1 ;
-									break ;
-								}
-							}
-						}
-
-						break ;
-					}
-				}
-			}
-		}
-
-		FT_Load_Glyph( face , idx , FT_LOAD_NO_BITMAP) ;
+		FT_Load_Glyph( face , code , FT_LOAD_NO_BITMAP) ;
 
 		if( face->glyph->format == FT_GLYPH_FORMAT_BITMAP)
 		{
@@ -731,7 +747,7 @@ load_char(
 	}
 	else
 	{
-		FT_Load_Char( face , code , 0) ;
+		FT_Load_Glyph( face , code , 0) ;
 	}
 
 	if( format & FONT_ITALIC)
@@ -785,9 +801,12 @@ load_ft(
 		}
 	}
 
+	fontsize = (format & ~(FONT_BOLD|FONT_ITALIC)) ;
+
 	for( count = 0 ; count < num_of_xfonts ; count++)
 	{
-		if( strcmp( xfonts[count]->file , file_path) == 0)
+		if( strcmp( xfonts[count]->file , file_path) == 0 &&
+		    (xfonts[count]->format & ~(FONT_BOLD|FONT_ITALIC)) == fontsize)
 		{
 			face = xfonts[count]->face ;
 
@@ -805,25 +824,24 @@ load_ft(
 	}
 
 face_found:
-	fontsize = (format & ~(FONT_BOLD|FONT_ITALIC)) ;
 	FT_Set_Pixel_Sizes( face , fontsize , fontsize) ;
 
 	xfont->format = format ;
 	xfont->face = face ;
 	xfont->is_aa = is_aa ;
 
-	if( ! load_char( face , format , 'W' , is_aa))
+	if( ! load_glyph( face , format , get_glyph_index( face , 'W') , is_aa))
 	{
 		kik_msg_printf( "%s doesn't have outline glyphs.\n" , file_path) ;
 
 		goto  error ;
 	}
 
-	xfont->num_of_indeces = 0x1000 ;
+	xfont->num_of_indeces = INITIAL_GLYPH_INDEX_TABLE_SIZE ;
 
 	if( ! ( xfont->file = strdup( file_path)) ||
 	    ! ( xfont->glyph_indeces = calloc( xfont->num_of_indeces , sizeof(u_int16_t))) ||
-	    ! ( xfont->glyphs = calloc( 512 , sizeof(u_char*))))
+	    ! ( xfont->glyphs = calloc( MAX_GLYPH_TABLES , sizeof(u_char*))))
 	{
 		goto  error ;
 	}
@@ -848,7 +866,7 @@ face_found:
 	xfont->ascent = (face->ascender * face->size->metrics.y_ppem
 			+ face->units_per_EM - 1) / face->units_per_EM ;
 
-	if( load_char( face , format , 'j' , is_aa))
+	if( load_glyph( face , format , get_glyph_index( face , 'j') , is_aa))
 	{
 		int  descent ;
 
@@ -910,6 +928,23 @@ init_iscii_ft(
 	}
 }
 
+static void
+clear_glyph_cache_ft(
+	XFontStruct *  xfont
+	)
+{
+	int  count ;
+
+	for( count = 0 ; ((u_char**)xfont->glyphs)[count] ; count ++)
+	{
+		free( ((u_char**)xfont->glyphs)[count]) ;
+	}
+
+	memset( xfont->glyphs , 0 , MAX_GLYPH_TABLES * sizeof(u_char*)) ;
+	xfont->num_of_glyphs = 0 ;
+	memset( xfont->glyph_indeces , 0 , xfont->num_of_indeces * sizeof(u_int16_t)) ;
+}
+
 static void
 unload_ft(
 	XFontStruct *  xfont
@@ -945,7 +980,8 @@ unload_ft(
 static u_char *
 get_ft_bitmap(
 	XFontStruct *  xfont ,
-	u_int32_t  code
+	u_int32_t  code ,
+	int  use_ot_layout
 	)
 {
 	u_int16_t *  indeces ;
@@ -953,9 +989,14 @@ get_ft_bitmap(
 	u_char **  glyphs ;
 	u_char *  glyph ;
 
-	if( code == 0x20)
+	if( ! use_ot_layout)
 	{
-		return  NULL ;
+		if( code == 0x20)
+		{
+			return  NULL ;
+		}
+
+		code = get_glyph_index( xfont->face , code) ;
 	}
 
 	if( code >= xfont->num_of_indeces)
@@ -987,7 +1028,7 @@ get_ft_bitmap(
 		int  pitch ;
 		int  rows ;
 
-		if( xfont->num_of_glyphs >= 128 * 512 - 1)
+		if( xfont->num_of_glyphs >= GLYPH_TABLE_SIZE * MAX_GLYPH_TABLES - 1)
 		{
 			kik_msg_printf( "Unable to show U+%x because glyph cache is full.\n" ,
 				code) ;
@@ -997,7 +1038,7 @@ get_ft_bitmap(
 
 		face = xfont->face ;
 
-		if( ! load_char( face , xfont->format , code , xfont->is_aa))
+		if( ! load_glyph( face , xfont->format , code , xfont->is_aa))
 		{
 			return  NULL ;
 		}
@@ -1005,7 +1046,7 @@ get_ft_bitmap(
 		if( OFF(xfont->num_of_glyphs) == 0)
 		{
 			if( ! ( glyphs[SEG(xfont->num_of_glyphs)] =
-					calloc( 128 , xfont->glyph_size)))
+					calloc( GLYPH_TABLE_SIZE , xfont->glyph_size)))
 			{
 				return  NULL ;
 			}
@@ -1113,7 +1154,7 @@ get_ft_bitmap(
 
 			if( IS_PROPORTIONAL(xfont))
 			{
-				/* Storing glyph position info. (ISCII) */
+				/* Storing glyph position info. (ISCII or ISO10646_UCS4_1_V) */
 
 				dst = glyph + xfont->glyph_size - 3 ;
 
@@ -1242,6 +1283,7 @@ x_font_t *
 x_font_new(
 	Display *  display ,
 	ml_font_t  id ,
+	int  size_attr ,
 	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present ,
 	const char *  fontname ,
@@ -1251,6 +1293,7 @@ x_font_new(
 	u_int  letter_space	/* Ignored for now. */
 	)
 {
+	mkf_charset_t  cs ;
 	char *  font_file ;
 	u_int  percent ;
 	x_font_t *  font ;
@@ -1260,13 +1303,15 @@ x_font_new(
 	u_int  format ;
 #endif
 
+	cs = FONT_CS(id) ;
+
 	if( ! fontname)
 	{
 	#ifdef  DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " Font file is not specified.\n") ;
 	#endif
 
-		if( FONT_CS(id) == ISO10646_UCS4_1 || FONT_CS(id) == ISO8859_1_R)
+		if( IS_ISO10646_UCS4(cs) || cs == ISO8859_1_R)
 		{
 			struct stat  st ;
 
@@ -1394,11 +1439,17 @@ x_font_new(
 		return  NULL ;
 	}
 
+	if( size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		fontsize *= 2 ;
+		col_width *= 2 ;
+	}
+
 #ifdef  USE_FREETYPE
 	if( percent > 0)
 	{
 		format = DIVIDE_ROUNDING( fontsize * percent , 100) |
-		         (id & (FONT_BOLD|FONT_ITALIC)) ;
+			 (id & (FONT_BOLD|FONT_ITALIC)) ;
 	}
 	else
 	{
@@ -1432,7 +1483,7 @@ x_font_new(
 	font->display = display ;
 
 	if( ! load_xfont( font->xfont , font_file , format ,
-		display->bytes_per_pixel , FONT_CS(id)))
+		display->bytes_per_pixel , cs))
 	{
 		kik_msg_printf( "Failed to load %s.\n" , font_file) ;
 
@@ -1441,7 +1492,13 @@ x_font_new(
 
 		if( fontname)
 		{
-			return  x_font_new( display , id , type_engine , font_present ,
+			if( size_attr >= DOUBLE_HEIGHT_TOP)
+			{
+				fontsize /= 2 ;
+				col_width /= 2 ;
+			}
+
+			return  x_font_new( display , id , size_attr , type_engine , font_present ,
 					NULL /* Fall back to the default font */ ,
 					fontsize , col_width , use_medium_for_bold ,
 					letter_space) ;
@@ -1483,14 +1540,12 @@ xfont_loaded:
 	 * font->is_var_col_width == false and font->is_proportional == true
 	 * is impossible on framebuffer.
 	 */
-#if  1
 #ifdef  USE_FREETYPE
-	if( IS_ISCII(FONT_CS(font->id)) && font->xfont->is_aa &&
+	if( IS_ISCII(cs) && font->xfont->is_aa &&
 	    ( font->xfont->ref_count == 1 || IS_PROPORTIONAL(font->xfont)))
 	{
 		/* Proportional glyph is available on ISCII alone for now. */
-		font->is_var_col_width = 1 ;
-		font->is_proportional = 1 ;
+		font->is_var_col_width = font->is_proportional = 1 ;
 
 		if( font->xfont->ref_count == 1)
 		{
@@ -1501,39 +1556,46 @@ xfont_loaded:
 	}
 	else
 #endif
-	if( ( font_present & FONT_VAR_WIDTH))
+	if( font_present & FONT_VAR_WIDTH)
 	{
 		/*
 		 * If you use fixed-width fonts whose width is differnet from
 		 * each other.
 		 */
 		font->is_var_col_width = 1 ;
-	}
-	else
-#endif
-	{
-		font->is_var_col_width = 0 ;
+
+	#ifdef  USE_FREETYPE
+		if( cs == ISO10646_UCS4_1_V)
+		{
+			font->is_proportional = 1 ;
+
+			if( font->xfont->ref_count == 1)
+			{
+				/* +3 is for storing glyph position info. */
+				font->xfont->glyph_size += 3 ;
+			}
+			else if( ! IS_PROPORTIONAL(font->xfont))
+			{
+				clear_glyph_cache_ft( font->xfont) ;
+
+				/* +3 is for storing glyph position info. */
+				font->xfont->glyph_size += 3 ;
+			}
+		}
+	#endif
 	}
 
 	if( font_present & FONT_VERTICAL)
 	{
 		font->is_vertical = 1 ;
 	}
-	else
-	{
-		font->is_vertical = 0 ;
-	}
 
 	if( use_medium_for_bold)
 	{
 		font->double_draw_gap = 1 ;
 	}
-	else
-	{
-		font->double_draw_gap = 0 ;
-	}
 
-	if( ( id & FONT_FULLWIDTH) && FONT_CS(id) == ISO10646_UCS4_1 &&
+	if( ( id & FONT_FULLWIDTH) && IS_ISO10646_UCS4(cs) &&
 	    font->xfont->width_full > 0)
 	{
 		font->width = font->xfont->width_full ;
@@ -1546,8 +1608,6 @@ xfont_loaded:
 	font->height = font->xfont->height ;
 	font->ascent = font->xfont->ascent ;
 
-	font->x_off = 0 ;
-
 	if( col_width == 0)
 	{
 		/* standard(usascii) font */
@@ -1671,6 +1731,15 @@ xfont_loaded:
 	}
 
 
+	if( size_attr == DOUBLE_WIDTH)
+	{
+		font->x_off += (font->width / 2) ;
+		font->width *= 2 ;
+	}
+
+	font->size_attr = size_attr ;
+
+
 	/*
 	 * checking if font width/height/ascent member is sane.
 	 */
@@ -1703,7 +1772,7 @@ xfont_loaded:
 		font_file , num_of_xfonts) ;
 #endif
 
-#ifdef  __DEBUG
+#ifdef  DEBUG
 	x_font_dump( font) ;
 #endif
 
@@ -1747,6 +1816,13 @@ x_font_delete(
 		free( font->xfont) ;
 	}
 
+#ifdef  USE_OT_LAYOUT
+	if( font->ot_font)
+	{
+		otl_close( font->ot_font) ;
+	}
+#endif
+
 	free( font) ;
 
 	return  1 ;
@@ -1777,6 +1853,52 @@ x_change_font_cols(
 	return  1 ;
 }
 
+#ifdef  USE_OT_LAYOUT
+
+int
+x_font_has_ot_layout_table(
+	x_font_t *  font
+	)
+{
+	if( font->xfont->face)
+	{
+		if( ! font->ot_font)
+		{
+			if( font->ot_font_not_found ||
+			    ! ( font->ot_font = otl_open( font->xfont->face , 0)))
+			{
+				font->ot_font_not_found = 1 ;
+
+				return  0 ;
+			}
+		}
+
+		return  1 ;
+	}
+
+	return  0 ;
+}
+
+u_int
+x_convert_text_to_glyphs(
+	x_font_t *  font ,
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+	return  otl_convert_text_to_glyphs( font->ot_font , shaped , shaped_len , offsets ,
+			widths , cmapped , src , src_len , script , features , 0) ;
+}
+
+#endif
+
 u_int
 x_calculate_char_width(
 	x_font_t *  font ,
@@ -1795,7 +1917,13 @@ x_calculate_char_width(
 	{
 		u_char *  glyph ;
 
-		if( ( glyph = get_ft_bitmap( font->xfont , ch)))
+		if( ( glyph = get_ft_bitmap( font->xfont , ch ,
+			#ifdef  USE_OT_LAYOUT
+				(font->use_ot_layout /* && font->ot_font */)
+			#else
+				0
+			#endif
+				)))
 		{
 			return  glyph[font->xfont->glyph_size - 3] ;
 		}
@@ -1888,7 +2016,8 @@ u_char *
 x_get_bitmap(
 	XFontStruct *  xfont ,
 	u_char *  ch ,
-	size_t  len
+	size_t  len ,
+	int  use_ot_layout
 	)
 {
 	size_t  ch_idx ;
@@ -1898,7 +2027,7 @@ x_get_bitmap(
 #ifdef  USE_FREETYPE
 	if( xfont->face)
 	{
-		return  get_ft_bitmap( xfont , 	mkf_bytes_to_int( ch , len)) ;
+		return  get_ft_bitmap( xfont , mkf_bytes_to_int( ch , len) , use_ot_layout) ;
 	}
 	else
 #endif
