@@ -28,6 +28,9 @@ static x_color_t  black = { TP_COLOR , 0 , 0 , 0 , 0 } ;
 #define  ParentRelative  (1L)
 #define  DummyPixmap  (2L)
 
+/* XXX Check if win is input method window or not. */
+#define  IS_IM_WINDOW(win)  ((win)->disp->num_of_roots >= 2 && (win) == (win)->disp->roots[1])
+
 
 /* --- static variables --- */
 
@@ -191,6 +194,9 @@ draw_string(
 	u_int  count ;
 	int  src_bg_is_set ;
 	int  orig_x ;
+	u_int  clip_bottom ;
+	int  use_ot_layout ;
+	int  orig_y_off ;
 
 	if( ! win->is_mapped)
 	{
@@ -226,11 +232,17 @@ draw_string(
 		return  0 ;
 	}
 
+#ifdef  USE_OT_LAYOUT
+	use_ot_layout = (font->use_ot_layout /* && font->ot_font */) ;
+#else
+	use_ot_layout = 0 ;
+#endif
+
 	if( ch_len == 1)
 	{
 		for( count = 0 ; count < len ; count++)
 		{
-			bitmaps[count] = x_get_bitmap( xfont , str + count , 1) ;
+			bitmaps[count] = x_get_bitmap( xfont , str + count , 1 , use_ot_layout) ;
 		}
 	}
 	else /* if( ch_len == 2) */
@@ -265,7 +277,7 @@ draw_string(
 				}
 			}
 
-			bitmaps[count] = x_get_bitmap( xfont , str , ch_len) ;
+			bitmaps[count] = x_get_bitmap( xfont , str , ch_len , use_ot_layout) ;
 
 			str += ch_len ;
 			ch_len = 2 ;
@@ -283,20 +295,38 @@ draw_string(
 	font_ascent = font->ascent ;
 	font_height = font->height ;
 
-	if( y >= win->height)
+	if( win->clip_height == 0)
 	{
-		font_ascent -= (y - win->height + 1) ;
-		font_height -= (y - win->height + 1) ;
+		clip_bottom = win->height ;
+	}
+	else
+	{
+		clip_bottom = win->clip_y + win->clip_height ;
 	}
 
-	if( y + font_height - font_ascent > win->height)
+	if( y >= clip_bottom + font_ascent)
 	{
-		font_height = win->height - y + font_ascent ;
+		return  0 ;
+	}
+	else if( y + font_height - font_ascent >= clip_bottom)
+	{
+		font_height -= (y + font_height - font_ascent - clip_bottom + 1) ;
+
+		if( font_height < font_ascent)
+		{
+			/* XXX I don't know why but without -1 causes line gap */
+			y -= (font_ascent - font_height - 1) ;
+			font_ascent = font_height ;
+		}
 	}
 
-	if( y < font_ascent)
+	if( y + font_height - font_ascent < win->clip_y)
 	{
-		y_off = font_ascent - y ;
+		return  0 ;
+	}
+	else if( y < win->clip_y + font_ascent)
+	{
+		y_off = win->clip_y + font_ascent - y ;
 	}
 	else
 	{
@@ -493,6 +523,7 @@ draw_string(
 #endif
 
 	orig_x = x ;
+	orig_y_off = y_off ;
 
 	for( ; y_off < font_height ; y_off++)
 	{
@@ -570,9 +601,25 @@ draw_string(
 
 						if( ( filled = (p - src) / bpp) < retreat)
 						{
-							bitmap_line += (retreat - filled) ;
-							x_off = -filled ;
-							p = src ;
+							if( x >= retreat)
+							{
+								x_off = -retreat ;
+								/* p is not changed. */
+								if( y_off == orig_y_off)
+								{
+									orig_x -= retreat ;
+								}
+								else
+								{
+									x += retreat ;
+								}
+							}
+							else
+							{
+								bitmap_line += (retreat - filled) ;
+								x_off = -filled ;
+								p = src ;
+							}
 						}
 						else
 						{
@@ -621,27 +668,32 @@ draw_string(
 						}
 					}
 
-					if( prev_crowded_out > advance)
+					if( count + 1 < len)
 					{
-						prev_crowded_out -= advance ;
-					}
-					else
-					{
-						prev_crowded_out = 0 ;
-					}
-
-					if( advance > 0 && x_off != advance)
-					{
-						p += ((advance - x_off) * bpp) ;
-
-						if( x_off > advance &&
-						    prev_crowded_out < x_off - advance)
+						if( prev_crowded_out > advance)
 						{
-							prev_crowded_out = x_off - advance ;
+							prev_crowded_out -= advance ;
+						}
+						else
+						{
+							prev_crowded_out = 0 ;
 						}
-					}
 
-					x = x + advance - font_width ;
+						if( advance > 0 && x_off != advance)
+						{
+							p += ((advance - x_off) * bpp) ;
+
+							if( x_off > advance &&
+							    prev_crowded_out < x_off - advance)
+							{
+								prev_crowded_out =
+									x_off - advance ;
+							}
+						}
+
+						/* -font_width is for +font_width in for() */
+						x = x + advance - font_width ;
+					}
 				}
 				else
 				{
@@ -968,6 +1020,7 @@ reset_input_focus(
 	}
 }
 
+#if  0
 static int
 check_child_window_area(
 	x_window_t *  win
@@ -992,6 +1045,7 @@ check_child_window_area(
 
 	return  1 ;
 }
+#endif
 
 
 /* --- global functions --- */
@@ -1069,21 +1123,14 @@ x_window_set_type_engine(
 	return  1 ;
 }
 
-int
-x_window_init_event_mask(
-	x_window_t *  win ,
-	long  event_mask
-	)
-{
-	return  1 ;
-}
-
 int
 x_window_add_event_mask(
 	x_window_t *  win ,
 	long  event_mask
 	)
 {
+	win->event_mask |= event_mask ;
+
 	return  1 ;
 }
 
@@ -1093,6 +1140,8 @@ x_window_remove_event_mask(
 	long  event_mask
 	)
 {
+	win->event_mask &= ~event_mask ;
+
 	return  1 ;
 }
 
@@ -1284,8 +1333,8 @@ x_window_add_child(
 	win->children = p ;
 
 	child->parent = win ;
-	child->x = x ;
-	child->y = y ;
+	child->x = x + win->hmargin ;
+	child->y = y + win->vmargin ;
 	child->is_focused = child->is_mapped = map ;
 
 	win->children[ win->num_of_children ++] = child ;
@@ -1368,7 +1417,7 @@ x_window_show(
 		win->gc = win->parent->gc ;
 	}
 
-	win->my_window = win ;	/* dummy */
+	win->my_window = win ;	/* Note that x_connect_dialog.c uses this. */
 
 	if( win->parent && ! win->parent->is_transparent &&
 	    win->parent->wall_picture)
@@ -1449,7 +1498,7 @@ x_window_resize(
 {
 	if( (flag & NOTIFY_TO_PARENT) &&
 	    /* XXX Check if win is input method window or not. */
-	    (win->disp->num_of_roots == 1 || win != win->disp->roots[1]))
+	    ! IS_IM_WINDOW(win))
 	{
 		if( win->parent)
 		{
@@ -1562,6 +1611,12 @@ x_window_move(
 	int  y
 	)
 {
+	if( win->parent)
+	{
+		x += win->parent->hmargin ;
+		y += win->parent->vmargin ;
+	}
+
 	if( win->x == x && win->y == y)
 	{
 		return  0 ;
@@ -1570,7 +1625,7 @@ x_window_move(
 	win->x = x ;
 	win->y = y ;
 
-	if( ! check_child_window_area( x_get_root_window( win)) ||
+	if( /* ! check_child_window_area( x_get_root_window( win)) || */
 	    win->x + ACTUAL_WIDTH(win) > win->disp->width ||
 	    win->y + ACTUAL_HEIGHT(win) > win->disp->height)
 	{
@@ -1595,7 +1650,7 @@ x_window_move(
 	 *    x_im_candidate_screen::window_exposed() ->
 	 *    x_im_candidate_screen::draw_screen()
 	 */
-	if( ( win->disp->num_of_roots == 1 || win != win->disp->roots[1]))
+	if( ! IS_IM_WINDOW(win))
 	{
 		clear_margin_area( win) ;
 
@@ -1885,7 +1940,7 @@ x_window_receive_event(
 			win->prev_button_press_event.y = event->xmotion.y ;
 			win->prev_button_press_event.time = event->xmotion.time ;
 		}
-		else if( win->pointer_motion)
+		else if( (win->event_mask & PointerMotionMask) && win->pointer_motion)
 		{
 			event->xmotion.x -= win->hmargin ;
 			event->xmotion.y -= win->vmargin ;
@@ -2094,8 +2149,7 @@ x_window_is_scrollable(
 	)
 {
 	/* XXX If input method module is activated, don't scroll window. */
-	if( win->is_scrollable &&
-	    (win->disp->num_of_roots == 1 || ! win->disp->roots[1]->is_mapped) )
+	if( win->is_scrollable && ! IS_IM_WINDOW(win))
 	{
 		return  1 ;
 	}
@@ -2256,6 +2310,27 @@ x_window_copy_area(
 	return  copy_area( win , src , mask , src_x , src_y , width , height , dst_x , dst_y , 0) ;
 }
 
+void
+x_window_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	win->clip_y = y ;
+	win->clip_height = height ;
+}
+
+void
+x_window_unset_clip(
+	x_window_t *  win
+	)
+{
+	win->clip_y = win->clip_height = 0 ;
+}
+
 int
 x_window_draw_decsp_string(
 	x_window_t *  win ,
