@@ -14,10 +14,14 @@
 #include  <cairo/cairo-xlib.h>
 #endif
 #include  <kiklib/kik_debug.h>
-#include  <kiklib/kik_mem.h>	/* alloca */
+#include  <kiklib/kik_mem.h>	/* realloc */
 #include  <kiklib/kik_str.h>	/* kik_str_sep/kik_str_to_int/memset/strncasecmp */
 #include  <ml_char.h>		/* UTF_MAX_SIZE */
 
+#ifdef  USE_OT_LAYOUT
+#include  <otl.h>
+#endif
+
 
 #define  DIVIDE_ROUNDING(a,b)  ( ((int)((a)*10 + (b)*5)) / ((int)((b)*10)) )
 #define  DIVIDE_ROUNDINGUP(a,b) ( ((int)((a)*10 + (b)*10 - 1)) / ((int)((b)*10)) )
@@ -60,10 +64,21 @@ parse_fc_font_name(
 {
 	char *  p ;
 	size_t  len ;
-	
+
+#if  1
+	/*
+	 * Compat with mlterm 3.6.3 or before: ... [SIZE]-[Encoding]:[Percentage]
+	 *                                               ^^^^^^^^^^^
+	 */
+	if( ( p = strstr( font_name , "-iso10646-1")))
+	{
+		memmove( p , p + 11 , strlen( p + 11) + 1) ;
+	}
+#endif
+
 	/*
 	 * XftFont format.
-	 * [Family]( [WEIGHT] [SLANT] [SIZE]-[Encoding]:[Percentage])
+	 * [Family]( [WEIGHT] [SLANT] [SIZE]:[Percentage])
 	 */
 
 	*font_family = font_name ;
@@ -73,10 +88,7 @@ parse_fc_font_name(
 	{
 		if( *p == '\\' && *(p + 1))
 		{
-			/*
-			 * It seems that XftFont format allows hyphens to be escaped.
-			 * (e.g. Foo\-Bold-iso10646-1)
-			 */
+			/* Compat with 3.6.3 or before. (e.g. Foo\-Bold-iso10646-1) */
 
 			/* skip backslash */
 			p ++ ;
@@ -89,28 +101,6 @@ parse_fc_font_name(
 			
 			break ;
 		}
-		else if( *p == '-')
-		{
-			/* Parsing "-[Encoding]:[Percentage]" */
-			
-			*font_name = '\0' ;
-
-			*font_encoding = ++p ;
-			
-			kik_str_sep( &p , ":") ;
-			if( p)
-			{
-				if( ! kik_str_to_uint( percent , p))
-				{
-				#ifdef  DEBUG
-					kik_warn_printf( KIK_DEBUG_TAG
-						" Percentage(%s) is illegal.\n" , p) ;
-				#endif
-				}
-			}
-			
-			break ;
-		}
 		else if( *p == ':')
 		{
 			/* Parsing ":[Percentage]" */
@@ -370,6 +360,8 @@ fc_pattern_create(
 	FcPatternAddBool( pattern , "embeddedbitmap" , True) ;
 #endif
 
+	FcConfigSubstitute( NULL , pattern , FcMatchPattern) ;
+
 	return  pattern ;
 }
 
@@ -460,6 +452,68 @@ xft_font_open(
 
 #ifdef  USE_TYPE_CAIRO
 
+static int
+is_same_family(
+	FcPattern *  pattern ,
+	const char *  family
+	)
+{
+	int  count ;
+	FcValue  val ;
+
+	for( count = 0 ;
+	     FcPatternGet( pattern , FC_FAMILY , count , &val) == FcResultMatch ;
+	     count ++)
+	{
+		if( strcmp( family , val.u.s) == 0)
+		{
+			return  1 ;
+		}
+	}
+
+	return  0 ;
+}
+
+static cairo_scaled_font_t *
+cairo_font_open_intern(
+	cairo_t *  cairo ,
+	FcPattern *  match ,
+	cairo_font_options_t *  options
+	)
+{
+	cairo_font_face_t *  font_face ;
+	double  pixel_size ;
+	int  pixel_size2 ;
+	cairo_matrix_t  font_matrix ;
+	cairo_matrix_t  ctm ;
+	cairo_scaled_font_t *  scaled_font ;
+
+	font_face = cairo_ft_font_face_create_for_pattern( match) ;
+
+	FcPatternGetDouble( match , FC_PIXEL_SIZE , 0 , &pixel_size) ;
+	/*
+	 * 10.5 / 2.0 = 5.25 ->(roundup) 6 -> 6 * 2 = 12
+	 * 11.5 / 2.0 = 5.75 ->(roundup) 6 -> 6 * 2 = 12
+	 *
+	 * If half width is 5.25 -> 6 and full width is 5.25 * 2 = 10.5 -> 11,
+	 * half width char -> x_bearing = 1 / width 5
+	 * full width char -> x_bearing = 1 / width 10.
+	 * This results in gap between chars.
+	 */
+	pixel_size2 = DIVIDE_ROUNDINGUP(pixel_size,2.0) * 2 ;
+
+	cairo_matrix_init_scale( &font_matrix , pixel_size2 , pixel_size2) ;
+	cairo_get_matrix( cairo , &ctm) ;
+
+	scaled_font = cairo_scaled_font_create( font_face , &font_matrix , &ctm , options) ;
+
+	cairo_destroy( cairo) ;
+	cairo_font_options_destroy( options) ;
+	cairo_font_face_destroy( font_face) ;
+
+	return  scaled_font ;
+}
+
 static cairo_scaled_font_t *
 cairo_font_open(
 	x_font_t *  font ,
@@ -477,12 +531,9 @@ cairo_font_open(
 	FcPattern *  pattern ;
 	FcPattern *  match ;
 	FcResult  result ;
-	cairo_font_face_t *  font_face ;
-	cairo_matrix_t  font_matrix ;
-	cairo_matrix_t  ctm ;
 	cairo_scaled_font_t *  xfont ;
-	double  pixel_size ;
-	int  pixel_size2 ;
+	FcCharSet *  charset ;
+	mkf_charset_t  cs ;
 
 	if( ! ( pattern = fc_pattern_create( family , size , encoding , weight , slant ,
 				ch_width , aa_opt)))
@@ -490,17 +541,13 @@ cairo_font_open(
 		return  NULL ;
 	}
 
-	FcConfigSubstitute( NULL , pattern , FcMatchPattern) ;
-
 	if( ! ( cairo = cairo_create( cairo_xlib_surface_create( font->display ,
 				DefaultRootWindow( font->display) ,
 				DefaultVisual( font->display , DefaultScreen( font->display)) ,
 				DisplayWidth( font->display , DefaultScreen( font->display)) ,
 				DisplayHeight( font->display , DefaultScreen( font->display))))))
 	{
-		FcPatternDestroy( pattern) ;
-
-		return  NULL ;
+		goto  error1 ;
 	}
 
 	options = cairo_font_options_create() ;
@@ -522,59 +569,43 @@ cairo_font_open(
 #endif
 	cairo_ft_font_options_substitute( options , pattern) ;
 
+	/* Supply default values for underspecified options */
 	FcDefaultSubstitute( pattern) ;
 
 	if( ! ( match = FcFontMatch( NULL , pattern , &result)))
 	{
 		cairo_destroy( cairo) ;
 		cairo_font_options_destroy( options) ;
-		FcPatternDestroy( pattern) ;
 
-		return  NULL ;
+		goto  error1 ;
 	}
 
 #if  0
 	FcPatternPrint( match) ;
 #endif
 
-	font_face = cairo_ft_font_face_create_for_pattern( match) ;
-
-	FcPatternGetDouble( match , FC_PIXEL_SIZE , 0 , &pixel_size) ;
-	/*
-	 * 10.5 / 2.0 = 5.25 ->(roundup) 6 -> 6 * 2 = 12
-	 * 11.5 / 2.0 = 5.75 ->(roundup) 6 -> 6 * 2 = 12
-	 *
-	 * If half width is 5.25 -> 6 and full width is 5.25 * 2 = 10.5 -> 11,
-	 * half width char -> x_bearing = 1 / width 5
-	 * full width char -> x_bearing = 1 / width 10.
-	 * This results in gap between chars.
-	 */
-	pixel_size2 = DIVIDE_ROUNDINGUP(pixel_size,2.0) * 2 ;
-
-	cairo_matrix_init_scale( &font_matrix , pixel_size2 , pixel_size2) ;
-	cairo_get_matrix( cairo , &ctm) ;
-
-	xfont = cairo_scaled_font_create( font_face , &font_matrix , &ctm , options) ;
-
-	cairo_destroy( cairo) ;
-	cairo_font_options_destroy( options) ;
-	cairo_font_face_destroy( font_face) ;
-	FcPatternDestroy( pattern) ;
-	FcPatternDestroy( match) ;
+	if( ! ( xfont = cairo_font_open_intern( cairo , match , options)))
+	{
+		goto  error2 ;
+	}
 
 	if( cairo_scaled_font_status( xfont))
 	{
 		cairo_scaled_font_destroy( xfont) ;
 
-		return  NULL ;
+		goto  error2 ;
 	}
 
+	cs = FONT_CS(font->id) ;
+
 #if  1
-	if( IS_ISCII(FONT_CS(font->id)))
+	if( IS_ISCII(cs))
 	{
 		FT_Face  face ;
 		int  count ;
 
+		FcPatternDestroy( pattern) ;
+
 		face = cairo_ft_scaled_font_lock_face( xfont) ;
 
 		for( count = 0 ; count < face->num_charmaps ; count++)
@@ -595,9 +626,203 @@ cairo_font_open(
 
 		cairo_ft_scaled_font_unlock_face( xfont) ;
 	}
+	else
 #endif
+	if( cs != US_ASCII && cs != ISO8859_1_R &&
+	    FcPatternGetCharSet( match , FC_CHARSET , 0 , &charset) == FcResultMatch &&
+	    ( font->compl_fonts = malloc( sizeof(*font->compl_fonts))))
+	{
+		FcValue  val ;
+		int  count ;
+
+		font->compl_fonts[0].charset = FcCharSetCopy( charset) ;
+		font->compl_fonts[0].next = NULL ;
+
+		count = 0 ;
+		while( FcPatternGet( pattern , FC_FAMILY , count , &val) == FcResultMatch)
+		{
+			if( is_same_family( match , val.u.s))
+			{
+				/* Remove not only matched name but also alias names */
+				FcPatternRemove( pattern , FC_FAMILY , count) ;
+			}
+			else
+			{
+				count ++ ;
+			}
+		}
+
+		font->pattern = pattern ;
+	}
+	else
+	{
+		FcPatternDestroy( pattern) ;
+	}
+
+	FcPatternDestroy( match) ;
 
 	return  xfont ;
+
+error2:
+	FcPatternDestroy( match) ;
+
+error1:
+	FcPatternDestroy( pattern) ;
+
+	return  NULL ;
+}
+
+#if  0
+static void
+print_family(
+	FcPattern *  pattern
+	)
+{
+	int  count ;
+	FcValue  val ;
+
+	for( count = 0 ;
+	     FcPatternGet( pattern , FC_FAMILY , count , &val) == FcResultMatch ;
+	     count ++)
+	{
+		kik_debug_printf( "Match(%d): %s\n" , count , val.u.s) ;
+	}
+}
+#endif
+
+static int
+cairo_compl_font_open(
+	x_font_t *  font ,
+	int  num_of_compl_fonts ,
+	FcPattern *  orig_pattern ,
+	int  ch
+	)
+{
+	FcValue  val ;
+	cairo_t *  cairo ;
+	cairo_font_options_t *  options ;
+	FcPattern *  pattern ;
+	FcPattern *  match = NULL ;
+	FcResult  result ;
+	cairo_scaled_font_t *  xfont ;
+	FcCharSet *  charset ;
+	int  count ;
+	int  ret = 0 ;
+
+	if( ! ( pattern = FcPatternDuplicate( orig_pattern)))
+	{
+		return  0 ;
+	}
+
+	/* Call FcConfigSubstitute before FcFontMatch */
+	FcConfigSubstitute( NULL , pattern , FcMatchPattern) ;
+#if  0
+	FcPatternPrint( orig_pattern) ;
+#endif
+
+	for( count = 0 ; FcPatternGet( pattern , FC_FAMILY , 0 , &val) == FcResultMatch ;
+	     count ++)
+	{
+		if( ! ( match = FcFontMatch( NULL , pattern , &result)))
+		{
+			break ;
+		}
+
+		FcPatternRemove( pattern , FC_FAMILY , 0) ;
+
+		if( FcPatternGetCharSet( match , FC_CHARSET , 0 , &charset) != FcResultMatch ||
+		    ! FcCharSetHasChar( charset , ch))
+		{
+			FcPatternDestroy( match) ;
+
+			continue ;
+		}
+		else
+		{
+			void *  p ;
+
+		#if  0
+			print_family( match) ;
+		#endif
+
+			FcPatternRemove( orig_pattern , FC_FAMILY , count++) ;
+
+			while( FcPatternGet( orig_pattern , FC_FAMILY , count , &val) ==
+					FcResultMatch)
+			{
+				if( is_same_family( match , val.u.s))
+				{
+					FcPatternRemove( orig_pattern , FC_FAMILY , count) ;
+				}
+				else
+				{
+					count ++ ;
+				}
+			}
+
+			if( ( p = realloc( font->compl_fonts ,
+					sizeof(*font->compl_fonts) * (num_of_compl_fonts + 1))))
+			{
+				font->compl_fonts = p ;
+			}
+			else
+			{
+				FcPatternDestroy( match) ;
+
+				break ;
+			}
+		}
+
+	#if  0
+		FcPatternPrint( match) ;
+	#endif
+
+		if( ! ( cairo = cairo_create( cairo_xlib_surface_create( font->display ,
+					DefaultRootWindow( font->display) ,
+					DefaultVisual( font->display ,
+						DefaultScreen( font->display)) ,
+					DisplayWidth( font->display ,
+						DefaultScreen( font->display)) ,
+					DisplayHeight( font->display ,
+						DefaultScreen( font->display))))))
+		{
+			break ;
+		}
+
+		options = cairo_font_options_create() ;
+		cairo_get_font_options( cairo , options) ;
+		/* For performance */
+		cairo_font_options_set_hint_style( options , CAIRO_HINT_STYLE_NONE) ;
+
+		if( ! ( xfont = cairo_font_open_intern( cairo , match , options)))
+		{
+			break ;
+		}
+
+		if( cairo_scaled_font_status( xfont))
+		{
+			cairo_scaled_font_destroy( xfont) ;
+
+			break ;
+		}
+
+		font->compl_fonts[num_of_compl_fonts - 1].next = xfont ;
+		font->compl_fonts[num_of_compl_fonts].charset = FcCharSetCopy( charset) ;
+		font->compl_fonts[num_of_compl_fonts].next = NULL ;
+
+		ret = 1 ;
+
+		break ;
+	}
+
+	FcPatternDestroy( pattern) ;
+
+	if( match)
+	{
+		FcPatternDestroy( match) ;
+	}
+
+	return  ret ;
 }
 
 #endif
@@ -699,7 +924,6 @@ fc_set_font(
 	 * x_off related to percent is set before ft_font_open while
 	 * x_off related to is_vertical and letter_space is set after.
 	 */
-	font->x_off = 0 ;
 
 	if( fontname)
 	{
@@ -744,7 +968,7 @@ fc_set_font(
 				ch_width = get_fc_col_width( font , fontsize_d , percent ,
 								letter_space) ;
 
-				if( percent > 100 && use_xft)  /* cairo ignores percent. */
+				if( percent > 100)
 				{
 					/*
 					 * Centering
@@ -859,14 +1083,10 @@ fc_set_font(
 
 font_found:
 
-	font->is_proportional = 0 ;
-
 	if( use_xft)
 	{
 	#ifdef  USE_TYPE_XFT
-	#if  defined(FC_EMBOLDEN) /* Synthetic emboldening (fontconfig >= 2.3.0) */
-		font->double_draw_gap = 0 ;
-	#else	/* FC_EMBOLDEN */
+	#ifndef  FC_EMBOLDEN /* Synthetic emboldening (fontconfig >= 2.3.0) */
 		if( weight == FC_WEIGHT_BOLD &&
 		    XftPatternGetInteger( xfont->pattern , FC_WEIGHT , 0 , &weight) ==
 			XftResultMatch &&
@@ -874,10 +1094,6 @@ font_found:
 		{
 			font->double_draw_gap = 1 ;
 		}
-		else
-		{
-			font->double_draw_gap = 0 ;
-		}
 	#endif	/* FC_EMBOLDEN */
 
 		font->xft_font = xfont ;
@@ -888,13 +1104,17 @@ font_found:
 		if( ch_width == 0)
 		{
 			/*
-			 * font->is_var_col_width is true or letter_space == 0.
-			 * (see get_fc_col_width())
+			 * (US_ASCII)
+			 *  font->is_var_col_width is true or letter_space == 0.
+			 *  (see get_fc_col_width())
+			 * (Other CS)
+			 *  font->is_var_col_width is true.
 			 */
 
 			font->width = xft_calculate_char_width( font , 'W') ;
 
-			if( font->width != xft_calculate_char_width( font , 'l'))
+			if( font->width != xft_calculate_char_width( font , 'l') ||
+			    (col_width > 0 && font->width != col_width * font->cols))
 			{
 				/* Regard it as proportional. */
 
@@ -945,11 +1165,7 @@ font_found:
 		{
 			font->double_draw_gap = 1 ;
 		}
-		else
 	#endif
-		{
-			font->double_draw_gap = 0 ;
-		}
 
 		font->cairo_font = xfont ;
 
@@ -1040,6 +1256,13 @@ font_found:
 			}
 		#endif
 		}
+		else
+		{
+			if( col_width > 0 && font->width != col_width * font->cols)
+			{
+				font->is_proportional = 1 ;
+			}
+		}
 	#endif	/* USE_TYPE_CAIRO */
 	}
 
@@ -1099,12 +1322,34 @@ xft_unset_font(
 	x_font_t *  font
 	)
 {
+#ifdef  USE_OT_LAYOUT
+	if( font->ot_font)
+	{
+		otl_close( font->ot_font) ;
+	}
+#endif
+
 	XftFontClose( font->display , font->xft_font) ;
 	font->xft_font = NULL ;
 
 	return  1 ;
 }
 
+int
+xft_set_ot_font(
+	x_font_t *  font
+	)
+{
+#ifdef  USE_OT_LAYOUT
+	font->ot_font = otl_open( XftLockFace( font->xft_font) , 0) ;
+	XftUnlockFace( font->xft_font) ;
+
+	return  (font->ot_font != NULL) ;
+#else
+	return  0 ;
+#endif
+}
+
 u_int
 xft_calculate_char_width(
 	x_font_t *  font ,
@@ -1113,6 +1358,24 @@ xft_calculate_char_width(
 {
 	XGlyphInfo  extents ;
 
+#ifdef  USE_OT_LAYOUT
+	if( font->use_ot_layout /* && font->ot_font */)
+	{
+		if( sizeof(FT_UInt) != sizeof(u_int32_t))
+		{
+			FT_UInt  idx ;
+
+			idx = ch ;
+
+			XftGlyphExtents( font->display , font->xft_font , &idx , 1 , &extents) ;
+		}
+		else
+		{
+			XftGlyphExtents( font->display , font->xft_font , &ch , 1 , &extents) ;
+		}
+	}
+	else
+#endif
 	if( ch < 0x100)
 	{
 		u_char  c ;
@@ -1140,6 +1403,40 @@ xft_calculate_char_width(
 
 #ifdef  USE_TYPE_CAIRO
 
+int
+x_search_next_cairo_font(
+	x_font_t *  font ,
+	int  ch
+	)
+{
+	int  count ;
+
+	if( ! font->compl_fonts)
+	{
+		return  -1 ;
+	}
+
+	for( count = 0 ; font->compl_fonts[count].next ; count++)
+	{
+		if( FcCharSetHasChar( font->compl_fonts[count + 1].charset , ch))
+		{
+			return  count ;
+		}
+	}
+
+	if( cairo_compl_font_open( font , count + 1 , font->pattern , ch))
+	{
+		return  count ;
+	}
+	else
+	{
+		/* To avoid to research it. */
+		FcCharSetAddChar( font->compl_fonts[0].charset , ch) ;
+
+		return  -1 ;
+	}
+}
+
 int
 cairo_set_font(
 	x_font_t *  font ,
@@ -1170,12 +1467,57 @@ cairo_unset_font(
 	x_font_t *  font
 	)
 {
+#ifdef  USE_OT_LAYOUT
+	if( font->ot_font)
+	{
+		otl_close( font->ot_font) ;
+	}
+#endif
+
 	cairo_scaled_font_destroy( font->cairo_font) ;
 	font->cairo_font = NULL ;
 
+	if( font->compl_fonts)
+	{
+		int  count ;
+		cairo_scaled_font_t *  xfont ;
+
+		for( count = 0 ; ; count++)
+		{
+			FcCharSetDestroy( font->compl_fonts[count].charset) ;
+			if( ! ( xfont = font->compl_fonts[count].next))
+			{
+				break ;
+			}
+			cairo_scaled_font_destroy( xfont) ;
+		}
+
+		free( font->compl_fonts) ;
+	}
+
+	if( font->pattern)
+	{
+		FcPatternDestroy( font->pattern) ;
+	}
+
 	return  1 ;
 }
 
+int
+cairo_set_ot_font(
+	x_font_t *  font
+	)
+{
+#ifdef  USE_OT_LAYOUT
+	font->ot_font = otl_open( cairo_ft_scaled_font_lock_face( font->cairo_font) , 0) ;
+	cairo_ft_scaled_font_unlock_face( font->cairo_font) ;
+
+	return  (font->ot_font != NULL) ;
+#else
+	return  0 ;
+#endif
+}
+
 size_t
 x_convert_ucs4_to_utf8(
 	u_char *  utf8 ,	/* size of utf8 should be greater than 5. */
@@ -1250,12 +1592,37 @@ cairo_calculate_char_width(
 	cairo_text_extents_t  extents ;
 	int  width ;
 
-	utf8[ x_convert_ucs4_to_utf8( utf8 , ch)] = '\0' ;
-	cairo_scaled_font_text_extents( font->cairo_font , utf8 , &extents) ;
+#ifdef  USE_OT_LAYOUT
+	if( font->use_ot_layout /* && font->ot_font */)
+	{
+		cairo_glyph_t  glyph ;
+
+		glyph.index = ch ;
+		glyph.x = glyph.y = 0 ;
+		cairo_scaled_font_glyph_extents( font->cairo_font , &glyph , 1 , &extents) ;
+	}
+	else
+#endif
+	{
+		int  idx ;
+
+		utf8[ x_convert_ucs4_to_utf8( utf8 , ch)] = '\0' ;
+
+		if( ! FcCharSetHasChar( font->compl_fonts[0].charset , ch) &&
+		    ( idx = x_search_next_cairo_font( font , ch)) >= 0)
+		{
+			cairo_scaled_font_text_extents( font->compl_fonts[idx].next ,
+				utf8 , &extents) ;
+		}
+		else
+		{
+			cairo_scaled_font_text_extents( font->cairo_font , utf8 , &extents) ;
+		}
+	}
 
 #if  0
 	kik_debug_printf( KIK_DEBUG_TAG " CHAR(%x) x_bearing %f width %f x_advance %f\n" ,
-		ch[0] , extents.x_bearing , extents.width , extents.x_advance) ;
+		ch , extents.x_bearing , extents.width , extents.x_advance) ;
 #endif
 
 	if( ( width = DOUBLE_ROUNDUP_TO_INT(extents.x_advance)) < 0)
@@ -1270,3 +1637,25 @@ cairo_calculate_char_width(
 }
 
 #endif
+
+u_int
+ft_convert_text_to_glyphs(
+	x_font_t *  font ,
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+#ifdef  USE_OT_LAYOUT
+	return  otl_convert_text_to_glyphs( font->ot_font , shaped , shaped_len , offsets ,
+			widths , cmapped , src , src_len , script , features , 0) ;
+#else
+	return  0 ;
+#endif
+}
