@@ -2,7 +2,7 @@
  *	$Id$
  */
 
-#include  "../x_type_loader.h"
+#include  "../xlib/x_type_loader.h"
 
 
 /* Dummy declaration */
@@ -13,6 +13,10 @@ void  cairo_resize(void) ;
 void  cairo_set_font(void) ;
 void  cairo_unset_font(void) ;
 void  cairo_calculate_char_width(void) ;
+void  cairo_set_clip(void) ;
+void  cairo_unset_clip(void) ;
+void  cairo_set_ot_font(void) ;
+void  ft_convert_text_to_glyphs(void) ;
 
 
 /* --- global variables --- */
@@ -27,5 +31,9 @@ void *  x_type_cairo_func_table[MAX_TYPE_FUNCS] =
 	cairo_set_font ,
 	cairo_unset_font ,
 	cairo_calculate_char_width ,
+	cairo_set_clip ,
+	cairo_unset_clip ,
+	cairo_set_ot_font ,
+	ft_convert_text_to_glyphs ,
 
 } ;
