@@ -2,7 +2,7 @@
  *	$Id$
  */
 
-#include  "../x_type_loader.h"
+#include  "../xlib/x_type_loader.h"
 
 #include  <stdio.h>		/* NULL */
 
@@ -14,6 +14,10 @@ void  x_window_xft_draw_string32(void) ;
 void  xft_set_font(void) ;
 void  xft_unset_font(void) ;
 void  xft_calculate_char_width(void) ;
+void  xft_set_clip(void) ;
+void  xft_unset_clip(void) ;
+void  xft_set_ot_font(void) ;
+void  ft_convert_text_to_glyphs(void) ;
 
 
 /* --- global variables --- */
@@ -28,5 +32,9 @@ void *  x_type_xft_func_table[MAX_TYPE_FUNCS] =
 	xft_set_font ,
 	xft_unset_font ,
 	xft_calculate_char_width ,
+	xft_set_clip ,
+	xft_unset_clip ,
+	xft_set_ot_font ,
+	ft_convert_text_to_glyphs ,
 
 } ;
