@@ -5,6 +5,7 @@
 #include  "../x_window.h"
 
 #include  <X11/Xft/Xft.h>
+#include  <kiklib/kik_mem.h>	/* alloca */
 
 
 #define  x_color_to_xft(xcolor)  _x_color_to_xft( alloca(sizeof(XftColor)) , (xcolor))
@@ -117,16 +118,72 @@ x_window_xft_draw_string32(
 
 	xftcolor = x_color_to_xft( fg_color) ;
 
-	XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
-		x + font->x_off + win->hmargin ,
-		y + win->vmargin , str , len) ;
+#ifdef  USE_OT_LAYOUT
+	if( font->use_ot_layout /* && font->ot_font */)
+	{
+		XftDrawGlyphs( win->xft_draw , xftcolor , font->xft_font ,
+			x + font->x_off + win->hmargin ,
+			y + win->vmargin , str , len) ;
+	}
+	else
+#endif
+	{
+		XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
+			x + font->x_off + win->hmargin ,
+			y + win->vmargin , str , len) ;
+	}
 
 	if( font->double_draw_gap)
 	{
-		XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
-			x + font->x_off + win->hmargin + font->double_draw_gap ,
-			y + win->vmargin , str , len) ;
+	#ifdef  USE_OT_LAYOUT
+		if( font->use_ot_layout /* && font->ot_font */)
+		{
+			XftDrawGlyphs( win->xft_draw , xftcolor , font->xft_font ,
+				x + font->x_off + win->hmargin ,
+				y + win->vmargin , str , len) ;
+		}
+		else
+	#endif
+		{
+			XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
+				x + font->x_off + win->hmargin + font->double_draw_gap ,
+				y + win->vmargin , str , len) ;
+		}
 	}
 
 	return  1 ;
 }
+
+void
+xft_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	XRectangle  rect ;
+
+	rect.x = 0 ;
+	rect.y = 0 ;
+	rect.width = width ;
+	rect.height = height ;
+
+	XftDrawSetClipRectangles( win->xft_draw , x , y , &rect , 1) ;
+}
+
+void
+xft_unset_clip(
+	x_window_t *  win
+	)
+{
+	XRectangle  rect ;
+
+	rect.x = 0 ;
+	rect.y = 0 ;
+	rect.width = ACTUAL_WIDTH(win) ;
+	rect.height = ACTUAL_HEIGHT(win) ;
+
+	XftDrawSetClipRectangles( win->xft_draw , 0 , 0 , &rect , 1) ;
+}
