@@ -15,6 +15,10 @@
 #include  <mkf/mkf_ucs_property.h>
 #include  <ml_char_encoding.h>	/* x_convert_to_xft_ucs4 */
 
+#ifdef  USE_OT_LAYOUT
+#include  "otl.h"
+#endif
+
 
 #define  FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p) \
 	for( (font_encoding_p) = &csinfo->encoding_names[0] ; *(font_encoding_p) ; (font_encoding_p) ++)
@@ -66,6 +70,7 @@ static wincs_info_t  wincs_info_table[] =
 static cs_info_t  cs_info_table[] =
 {
 	{ ISO10646_UCS4_1 , DEFAULT_CHARSET , } ,
+	{ ISO10646_UCS4_1_V , DEFAULT_CHARSET , } ,
 	
 	{ DEC_SPECIAL , SYMBOL_CHARSET , } ,
 	{ ISO8859_1_R , ANSI_CHARSET , } ,
@@ -92,7 +97,6 @@ static cs_info_t  cs_info_table[] =
 	{ ISCII_MALAYALAM , DEFAULT_CHARSET , } ,
 	{ ISCII_ORIYA , DEFAULT_CHARSET , } ,
 	{ ISCII_PUNJABI , DEFAULT_CHARSET , } ,
-	{ ISCII_ROMAN , DEFAULT_CHARSET , } ,
 	{ ISCII_TAMIL , DEFAULT_CHARSET , } ,
 	{ ISCII_TELUGU , DEFAULT_CHARSET , } ,
 	{ VISCII , VIETNAMESE_CHARSET , } ,
@@ -375,8 +379,10 @@ calculate_char_width(
 	{
 		u_int32_t  ucs4_code ;
 		u_char  utf16[4] ;
+		int  len ;
+		BOOL  ret ;
 
-		if( cs == ISO10646_UCS4_1)
+		if( IS_ISO10646_UCS4(cs))
 		{
 			ucs4_code = ch ;
 		}
@@ -410,8 +416,33 @@ calculate_char_width(
 			}
 		}
 
-		if( ! GetTextExtentPointW( display_gc , utf16 ,
-			x_convert_ucs4_to_utf16( utf16 , ucs4_code) / 2 , &sz))
+		len = x_convert_ucs4_to_utf16( utf16 , ucs4_code) / 2 ;
+
+	#ifdef  USE_OT_LAYOUT
+		if( font->use_ot_layout /* && font->ot_font */)
+		{
+		#ifdef  USE_WIN32API
+			/* GetTextExtentPointI doesn't exist on mingw-i686-pc-mingw */
+			static WINGDIAPI BOOL WINAPI (*func)( HDC , LPWORD , int , LPSIZE) ;
+
+			if( ! func)
+			{
+				func = GetProcAddress( GetModuleHandleA("GDI32.DLL") ,
+					"GetTextExtentPointI") ;
+			}
+
+			ret = (*func)( display_gc , utf16 , len , &sz) ;
+		#else
+			ret = GetTextExtentPointI( display_gc , utf16 , len , &sz) ;
+		#endif
+		}
+		else
+	#endif
+		{
+			ret = GetTextExtentPoint32W( display_gc , utf16 , len , &sz) ;
+		}
+
+		if( ! ret)
 		{
 			return  0 ;
 		}
@@ -421,13 +452,20 @@ calculate_char_width(
 		u_char  c ;
 
 		c = ch ;
-		if( ! GetTextExtentPointA( display_gc , &c , 1 , &sz))
+		if( ! GetTextExtentPoint32A( display_gc , &c , 1 , &sz))
 		{
 			return  0 ;
 		}
 	}
 
-	return  sz.cx ;
+	if( sz.cx < 0)
+	{
+		return  0 ;
+	}
+	else
+	{
+		return  sz.cx ;
+	}
 }
 
 
@@ -445,6 +483,7 @@ x_font_t *
 x_font_new(
 	Display *  display ,
 	ml_font_t  id ,
+	int  size_attr ,
 	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present ,
 	const char *  fontname ,
@@ -462,7 +501,7 @@ x_font_new(
 	double  fontsize_d ;
 	u_int  percent ;
 	
-	if( type_engine != TYPE_XCORE || ( font = malloc( sizeof( x_font_t))) == NULL)
+	if( type_engine != TYPE_XCORE || ( font = calloc( 1 , sizeof( x_font_t))) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
@@ -483,23 +522,23 @@ x_font_new(
 		font->cols = 1 ;
 	}
 
-	if( font_present & FONT_VAR_WIDTH || IS_ISCII(FONT_CS(font->id)))
+	if( IS_ISCII(FONT_CS(font->id)) || FONT_CS(font->id) == ISO10646_UCS4_1_V)
+	{
+		/*
+		 * For exampe, 'W' width and 'l' width of OR-TTSarala font for ISCII_ORIYA
+		 * are the same by chance, though it is actually a proportional font.
+		 */
+		font->is_var_col_width = font->is_proportional = 1 ;
+	}
+	else if( font_present & FONT_VAR_WIDTH)
 	{
 		font->is_var_col_width = 1 ;
 	}
-	else
-	{
-		font->is_var_col_width = 0 ;
-	}
 
 	if( font_present & FONT_VERTICAL)
 	{
 		font->is_vertical = 1 ;
 	}
-	else
-	{
-		font->is_vertical = 0 ;
-	}
 
 	if( ( wincsinfo = get_wincs_info( FONT_CS(font->id))) == NULL)
 	{
@@ -542,6 +581,16 @@ x_font_new(
 		is_italic = FALSE ;
 	}
 
+	if( size_attr)
+	{
+		if( size_attr >= DOUBLE_HEIGHT_TOP)
+		{
+			fontsize *= 2 ;
+		}
+
+		col_width *= 2 ;
+	}
+
 	font_family = NULL ;
 	percent = 0 ;
 	fontsize_d = (double)fontsize ;
@@ -586,7 +635,12 @@ x_font_new(
 				col_width ? (font->is_vertical ? col_width / 2 : col_width) :
 					(int)fontsize_d / 2 ,
 			#else
-				0 ,			/* Width (0=auto) */
+				size_attr == DOUBLE_WIDTH ?
+					(use_point_size ?
+						-MulDiv( (int)fontsize_d ,
+						GetDeviceCaps( display_gc , LOGPIXELSX) , 72) :
+						(int)fontsize_d) :
+					0 ,		/* Width (0=auto) */
 			#endif
 				0 ,			/* text angle */
 				0 ,			/* char angle */
@@ -625,32 +679,41 @@ x_font_new(
 		 * MS Gothic is regarded as VARIABLE_PITCH. (tm.tmPitchAndFamily)
 		 * So "w" and "l" width is compared to check if font is proportional or not.
 		 */
-		if( GetTextExtentPointA( display_gc , "w" , 1 , &w_sz) &&
-		    GetTextExtentPointA( display_gc , "l" , 1 , &l_sz) &&
-		    w_sz.cx != l_sz.cx)
-		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG
-				" w-width %d l-width %d\n" , w_sz.cx , l_sz.cx) ;
-		#endif
+		GetTextExtentPoint32A( display_gc , "w" , 1 , &w_sz) ;
+		GetTextExtentPoint32A( display_gc , "l" , 1 , &l_sz) ;
 
-			font->is_proportional = 1 ;
-		}
-		else
-		{
-			font->is_proportional = 0 ;
-		}
-		
 	#if  0
 		kik_debug_printf(
-		"Family %s Size %d CS %x => AveCharWidth %d MaxCharWidth %d Height %d Ascent %d ExLeading %d InLeading %d Pitch&Family %d Weight %d\n" ,
+		"Family %s Size %d CS %x => AveCharWidth %d MaxCharWidth %d 'w' width %d 'l' width %d Height %d Ascent %d ExLeading %d InLeading %d Pitch&Family %d Weight %d\n" ,
 		font_family , fontsize , wincsinfo->cs ,
-		tm.tmAveCharWidth , tm.tmMaxCharWidth , tm.tmHeight , tm.tmAscent ,
-		tm.tmExternalLeading , tm.tmInternalLeading ,
+		tm.tmAveCharWidth , tm.tmMaxCharWidth , w_sz.cx , l_sz.cx ,
+		tm.tmHeight , tm.tmAscent , tm.tmExternalLeading , tm.tmInternalLeading ,
 		tm.tmPitchAndFamily , tm.tmWeight) ;
 	#endif
 
-		font->width = tm.tmAveCharWidth * font->cols ;
+		if( w_sz.cx != l_sz.cx)
+		{
+			font->is_proportional = 1 ;
+			font->width = tm.tmAveCharWidth * font->cols ;
+		}
+		else
+		{
+			SIZE  sp_sz ;
+			WORD  sp = 0x3000 ;	/* wide space */
+
+			if( font->cols == 2 &&
+			    GetTextExtentPoint32W( display_gc , &sp , 1 , &sp_sz) &&
+			    sp_sz.cx != l_sz.cx * 2)
+			{
+				font->is_proportional = 1 ;
+				font->width = sp_sz.cx ;
+			}
+			else
+			{
+				font->width = w_sz.cx * font->cols ;
+			}
+		}
+
 		font->height = tm.tmHeight ;
 		font->ascent = tm.tmAscent ;
 
@@ -658,17 +721,13 @@ x_font_new(
 		{
 			font->double_draw_gap = 1 ;
 		}
-		else
-		{
-			font->double_draw_gap = 0 ;
-		}
 	}
 
+	font->size = fontsize ;
+
 	/*
 	 * Following processing is same as x_font.c:set_xfont()
 	 */
-	 
-	font->x_off = 0 ;
 
 	if( col_width == 0)
 	{
@@ -773,13 +832,11 @@ x_font_new(
 		}
 	}
 
+	font->size_attr = size_attr ;
 
-	if( wincsinfo->cs == ANSI_CHARSET || wincsinfo->cs == SYMBOL_CHARSET ||
-	    FONT_CS(font->id) == ISO10646_UCS4_1)
-	{
-		font->conv = NULL ;
-	}
-	else
+
+	if( wincsinfo->cs != ANSI_CHARSET && wincsinfo->cs != SYMBOL_CHARSET &&
+	    ! IS_ISO10646_UCS4(FONT_CS(font->id)))
 	{
 		if( ! ( font->conv = ml_conv_new( wincsinfo->encoding)))
 		{
@@ -789,13 +846,11 @@ x_font_new(
 		#endif
 		}
 	}
-	
-	font->decsp_font = NULL ;
 
 	if( font->is_proportional && ! font->is_var_col_width)
 	{
 		kik_msg_printf(
-			"Characters (cs %d) are drawn *one by one* to arrange column width.\n" ,
+			"Characters (cs %x) are drawn *one by one* to arrange column width.\n" ,
 			FONT_CS(font->id)) ;
 
 	}
@@ -808,9 +863,16 @@ x_font_delete(
 	x_font_t *  font
 	)
 {
+#ifdef  USE_OT_LAYOUT
+	if( font->ot_font)
+	{
+		otl_close( font->ot_font) ;
+	}
+#endif
+
 	if( font->fid)
 	{
-		DeleteObject(font->fid) ;
+		DeleteObject( font->fid) ;
 	}
 	
 	if( font->conv)
@@ -854,6 +916,114 @@ x_change_font_cols(
 	return  1 ;
 }
 
+#ifdef  USE_OT_LAYOUT
+int
+x_font_has_ot_layout_table(
+	x_font_t *  font
+	)
+{
+	if( ! font->ot_font)
+	{
+		u_char  buf[4] ;
+		void *  font_data ;
+		u_int  size ;
+		int  is_ttc ;
+
+		if( font->ot_font_not_found)
+		{
+			return  0 ;
+		}
+
+		if( ! display_gc)
+		{
+			/*
+			 * Cached as far as x_caculate_char_width is called.
+			 * display_gc is deleted in x_font_new or x_font_delete.
+			 */
+			display_gc = CreateIC( "Display" , NULL , NULL , NULL) ;
+		}
+
+		SelectObject( display_gc , font->fid) ;
+
+#define  TTC_TAG  ('t' << 0) + ('t' << 8) + ('c' << 16) + ('f' << 24)
+
+		if( GetFontData( display_gc , TTC_TAG , 0 , &buf , 1) == 1)
+		{
+			is_ttc = 1 ;
+			size = GetFontData( display_gc , TTC_TAG , 0 , NULL , 0) ;
+		}
+		else
+		{
+			is_ttc = 0 ;
+			size = GetFontData( display_gc , 0 , 0 , NULL , 0) ;
+		}
+
+		if( ! ( font_data = malloc( size)))
+		{
+			font->ot_font = NULL ;
+		}
+		else
+		{
+			if( is_ttc)
+			{
+				GetFontData( display_gc , TTC_TAG , 0 , font_data , size) ;
+			}
+			else
+			{
+				GetFontData( display_gc , 0 , 0 , font_data , size) ;
+			}
+
+			font->ot_font = otl_open( font_data , size) ;
+		}
+
+		if( ! font->ot_font)
+		{
+			font->ot_font_not_found = 1 ;
+
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+u_int
+x_convert_text_to_glyphs(
+	x_font_t *  font ,
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+	u_int  size ;
+
+	if( use_point_size)
+	{
+		if( ! display_gc)
+		{
+			display_gc = CreateIC( "Display" , NULL , NULL , NULL) ;
+		}
+
+		size = MulDiv( (int)font->size ,
+				GetDeviceCaps( display_gc , LOGPIXELSY) , 72) ;
+	}
+	else
+	{
+		size = font->size ;
+	}
+
+	return  otl_convert_text_to_glyphs( font->ot_font , shaped , shaped_len , offsets ,
+			widths , cmapped , src , src_len , script , features ,
+			size * (font->size_attr >= DOUBLE_WIDTH ? 2 : 1)) ;
+}
+#endif	/* USE_OT_LAYOUT */
+
 u_int
 x_calculate_char_width(
 	x_font_t *  font ,
@@ -873,21 +1043,21 @@ x_calculate_char_width(
 		{
 			u_int  width ;
 
-			if( ( width = calculate_char_width( font , ch , cs)) == 0)
-			{
-				goto  fixed_col_width ;
-			}
-
-			return  width ;
+			return  width = calculate_char_width( font , ch , cs) ;
 		}
 
-	fixed_col_width:
 		if( draw_alone)
 		{
 			*draw_alone = 1 ;
 		}
 	}
-	else if( draw_alone && cs == ISO10646_UCS4_1)
+	else if( draw_alone &&
+	         /* ISO10646_UCS4_1_V is always proportional */
+	         cs == ISO10646_UCS4_1
+	#ifdef  USE_OT_LAYOUT
+		 && (! font->use_ot_layout /* || ! font->ot_font */)
+	#endif
+		 )
 	{
 		if( mkf_get_ucs_property( ch) & MKF_AWIDTH)
 		{
@@ -903,10 +1073,10 @@ x_calculate_char_width(
 
 void
 x_font_use_point_size(
-	int  bool
+	int  use
 	)
 {
-	use_point_size = bool ;
+	use_point_size = use ;
 }
 
 /* Return written size */
