@@ -590,13 +590,23 @@ text_out(
 	int  y ,
 	u_char *  str ,
 	u_int  len ,
-	mkf_charset_t  cs	/* FONT_CS(font->id) */
+	mkf_charset_t  cs ,	/* FONT_CS(font->id) */
+	int  is_glyph ,
+	int  is_ucs
 	)
 {
-	if( cs == ISO10646_UCS4_1)
+	if( is_ucs)
 	{
-		/* TextOutW is supported in windows 9x. */
-		return  TextOutW( gc , x , y , (WCHAR*)str , len / 2) ;
+		if( is_glyph)
+		{
+			return  ExtTextOutW( gc , x , y , ETO_GLYPH_INDEX , NULL ,
+					(WCHAR*)str , len / 2 , NULL) ;
+		}
+		else
+		{
+			/* TextOutW is supported in windows 9x. */
+			return  TextOutW( gc , x , y , (WCHAR*)str , len / 2) ;
+		}
 	}
 	else if( cs == ISCII_BENGALI || cs == ISCII_ASSAMESE)
 	{
@@ -635,11 +645,13 @@ draw_string(
 	int  y ,
 	u_char *  str ,
 	u_int  len ,
-	int  is_tp
+	int  is_tp ,
+	int  is_ucs
 	)
 {
 	u_char *  str2 ;
-	
+	int  is_glyph ;
+
 	str2 = NULL ;
 
 	if( font->conv)
@@ -665,9 +677,15 @@ draw_string(
 		SetBkMode( win->gc->gc , TRANSPARENT) ;
 	}
 
+#ifdef  USE_OT_LAYOUT
+	is_glyph = (font->use_ot_layout /* && font->ot_font */) ;
+#else
+	is_glyph = 0 ;
+#endif
+
 	text_out( win->gc->gc, x + (font->is_var_col_width ? 0 : font->x_off) + win->hmargin ,
-		y + win->vmargin , str , len , FONT_CS(font->id)) ;
-	
+		y + win->vmargin , str , len , FONT_CS(font->id) , is_glyph , is_ucs) ;
+
 	if( font->double_draw_gap)
 	{
 		SetBkMode( win->gc->gc , TRANSPARENT) ;
@@ -675,7 +693,7 @@ draw_string(
 		text_out( win->gc->gc ,
 			x + (font->is_var_col_width ? 0 : font->x_off) + win->hmargin +
 				font->double_draw_gap ,
-			y + win->vmargin , str , len , FONT_CS(font->id)) ;
+			y + win->vmargin , str , len , FONT_CS(font->id) , is_glyph , is_ucs) ;
 
 		SetBkMode( win->gc->gc , OPAQUE) ;
 	}
@@ -770,11 +788,10 @@ clear_margin_area(
 		if( win->hmargin > 0 || right_margin > 0)
 		{
 			BitBlt( win->gc->gc , 0 , 0 ,
-				win->hmargin , ACTUAL_HEIGHT(win) + right_margin ,
+				win->hmargin , ACTUAL_HEIGHT(win) ,
 				pic , src_x , src_y , SRCCOPY) ;
 			BitBlt( win->gc->gc , win_width + win->hmargin , 0 ,
-				win->hmargin + right_margin ,
-				ACTUAL_HEIGHT(win) + bottom_margin ,
+				win->hmargin + right_margin , ACTUAL_HEIGHT(win) ,
 				pic ,
 				src_x + win_width + win->hmargin , src_y , SRCCOPY) ;
 		}
@@ -796,19 +813,15 @@ clear_margin_area(
 	{
 		HBRUSH  brush ;
 		RECT  r ;
-		int  right ;
-		int  bottom ;
 
 		brush = x_acquire_brush( win->bg_color.pixel) ;
-		right = ACTUAL_WIDTH(win) + right_margin ;
-		bottom = ACTUAL_HEIGHT(win) + bottom_margin ;
 
 		if( win->hmargin > 0 || right_margin > 0)
 		{
-			SetRect( &r , 0 , 0 , win->hmargin , bottom) ;
+			SetRect( &r , 0 , 0 , win->hmargin , ACTUAL_HEIGHT(win)) ;
 			FillRect( win->gc->gc , &r , brush) ;
 			SetRect( &r , win_width + win->hmargin , 0 ,
-				right , bottom) ;
+				ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 			FillRect( win->gc->gc , &r , brush) ;
 		}
 
@@ -818,7 +831,7 @@ clear_margin_area(
 				win_width + win->hmargin , win->vmargin) ;
 			FillRect( win->gc->gc , &r , brush) ;
 			SetRect( &r , win->hmargin , win_height + win->vmargin ,
-				win_width + win->hmargin , bottom) ;
+				win_width + win->hmargin , ACTUAL_HEIGHT(win)) ;
 			FillRect( win->gc->gc , &r , brush) ;
 		}
 
@@ -1121,21 +1134,14 @@ x_window_set_type_engine(
 	return  0 ;
 }
 
-int
-x_window_init_event_mask(
-	x_window_t *  win ,
-	long  event_mask
-	)
-{
-	return  1 ;
-}
-
 int
 x_window_add_event_mask(
 	x_window_t *  win ,
 	long  event_mask
 	)
 {
+	win->event_mask |= event_mask ;
+
 	return  1 ;
 }
 
@@ -1145,6 +1151,8 @@ x_window_remove_event_mask(
 	long  event_mask
 	)
 {
+	win->event_mask &= ~event_mask ;
+
 	return  1 ;
 }
 
@@ -1310,8 +1318,8 @@ x_window_add_child(
 	win->children = p ;
 
 	child->parent = win ;
-	child->x = x ;
-	child->y = y ;
+	child->x = x + win->hmargin ;
+	child->y = y + win->vmargin ;
 
 	if( ! ( child->is_mapped = map) && child->inputtable > 0)
 	{
@@ -1498,11 +1506,24 @@ x_window_show(
 	/* Don't place this before x_window_show(children). */
 	if( update_decorate_size( win))
 	{
-		SetWindowPos( win->my_window , 0 , 0 , 0 ,
-			ACTUAL_WINDOW_WIDTH(win) , ACTUAL_WINDOW_HEIGHT(win) ,
+		SetWindowPos( win->my_window , win->cmd_show == SW_SHOWNA ? HWND_TOPMOST : 0 ,
+			0 , 0 , ACTUAL_WINDOW_WIDTH(win) , ACTUAL_WINDOW_HEIGHT(win) ,
+			(win->cmd_show == SW_SHOWNA ? SWP_NOACTIVATE : 0) |
 			SWP_NOMOVE | SWP_NOZORDER) ;
 	}
 
+	/*
+	 * XXX
+	 * Some area in a main screen is not correctly drawn without this
+	 * in showing a status screen.
+	 * stat_screen is shown in im->stat_screen->show() in x_im_redraw_preedit()
+	 */
+	if( win->app_name &&
+	    strstr( win->app_name , "mlterm-"))	/* mlterm-{candidate|status}-window" */
+	{
+		win->is_mapped = 0 ;
+	}
+
 	if( win->is_mapped)
 	{
 		ShowWindow( win->my_window , win->cmd_show) ;
@@ -1601,10 +1622,10 @@ x_window_resize(
 	}
 
 	update_decorate_size( win) ;
-	SetWindowPos( win->my_window , 0 , 0 , 0 ,
+	SetWindowPos( win->my_window , win->cmd_show == SW_SHOWNA ? HWND_TOPMOST : 0 , 0 , 0 ,
 		win->parent ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win) ,
 		win->parent ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win) ,
-		SWP_NOMOVE | SWP_NOZORDER) ;
+		(win->cmd_show == SW_SHOWNA ? SWP_NOACTIVATE : 0) | SWP_NOMOVE | SWP_NOZORDER) ;
 		
 	if( (flag & NOTIFY_TO_MYSELF) && win->window_resized)
 	{
@@ -1658,12 +1679,20 @@ x_window_set_override_redirect(
 
 	if( ( root = x_get_root_window( win))->my_window)
 	{
-		ShowWindow( root->my_window , SW_HIDE) ;
+	#if  0
+		if( root->is_mapped)
+		{
+			ShowWindow( root->my_window , SW_HIDE) ;
+		}
+	#endif
 
 		if( flag)
 		{
 			SetWindowLong( root->my_window , GWL_STYLE , 0) ;
 			win->cmd_show = SW_SHOWNA ;
+
+			SetWindowPos( root->my_window , HWND_TOPMOST , 0 , 0 , 0 , 0 ,
+				SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER) ;
 		}
 		else
 		{
@@ -1672,7 +1701,12 @@ x_window_set_override_redirect(
 			win->cmd_show = SW_SHOWNORMAL ;
 		}
 
-		ShowWindow( root->my_window , win->cmd_show) ;
+	#if  0
+		if( root->is_mapped)
+		{
+			ShowWindow( root->my_window , win->cmd_show) ;
+		}
+	#endif
 
 		return  1 ;
 	}
@@ -1698,6 +1732,12 @@ x_window_move(
 	int  y
 	)
 {
+	if( win->parent)
+	{
+		x += win->parent->hmargin ;
+		y += win->parent->vmargin ;
+	}
+
 	if( win->x == x && win->y == y)
 	{
 		return  0 ;
@@ -1708,10 +1748,10 @@ x_window_move(
 
 	update_decorate_size( win) ;
 
-	SetWindowPos( win->my_window , 0 , x , y ,
+	SetWindowPos( win->my_window , 0 , win->x , win->y ,
 		win->parent ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win) ,
 		win->parent ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win) ,
-		SWP_NOSIZE | SWP_NOZORDER) ;
+		(win->cmd_show == SW_SHOWNA ? SWP_NOACTIVATE : 0) | SWP_NOSIZE | SWP_NOZORDER) ;
 
 	return  1 ;
 }
@@ -1898,7 +1938,8 @@ x_window_blank(
 	}
 
 	brush = x_acquire_brush( win->fg_color.pixel) ;
-	SetRect( &r , win->hmargin , win->vmargin , win->width , win->height) ;
+	SetRect( &r , win->hmargin , win->vmargin ,
+		win->width - RIGHT_MARGIN(win) , win->height - BOTTOM_MARGIN(win)) ;
 	FillRect( win->gc->gc , &r , brush) ;
 	x_release_brush( brush) ;
 
@@ -2579,7 +2620,8 @@ x_window_receive_event(
 		return  1 ;
 
 	case  WM_MOUSEMOVE:
-		if( win->button_is_pressing || win->pointer_motion)
+		if( win->button_is_pressing ||
+		    ((win->event_mask & PointerMotionMask) && win->pointer_motion))
 		{
 			XMotionEvent  mev ;
 
@@ -3073,6 +3115,30 @@ x_window_copy_area(
 	return  1 ;
 }
 
+void
+x_window_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	HRGN  r = CreateRectRgn( x + win->hmargin , y + win->vmargin ,
+			x + width + win->hmargin , y + height + win->vmargin) ;
+
+	SelectClipRgn( win->gc->gc , r) ;
+	DeleteObject( r) ;
+}
+
+void
+x_window_unset_clip(
+	x_window_t *  win
+	)
+{
+	SelectClipRgn( win->gc->gc , NULL) ;
+}
+
 int
 x_window_draw_decsp_string(
 	x_window_t *  win ,
@@ -3145,7 +3211,7 @@ x_window_draw_string(
 	 * In case US_ASCII characters is drawn by Unicode font.
 	 * 8 bit charcter => 16 bit character.
 	 */
-	if( FONT_CS(font->id) == ISO10646_UCS4_1)
+	if( IS_ISO10646_UCS4(FONT_CS(font->id)))
 	{
 		u_char *  dbl_str ;
 
@@ -3160,12 +3226,12 @@ x_window_draw_string(
 				dbl_str[count * 2 + 1] = 0x0 ;
 			}
 
-			draw_string( win , font , x , y , dbl_str , len * 2 , 1) ;
+			draw_string( win , font , x , y , dbl_str , len * 2 , 1 , 1) ;
 		}
 	}
 	else
 	{
-		draw_string( win , font , x , y , str , len , 1) ;
+		draw_string( win , font , x , y , str , len , 1 , 0) ;
 	}
 	
 	return  1 ;
@@ -3190,7 +3256,8 @@ x_window_draw_string16(
 	x_gc_set_fid( win->gc, font->fid) ;
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 	
-	draw_string( win , font , x , y , (u_char*)str , len * 2 , 1) ;
+	draw_string( win , font , x , y , (u_char*)str , len * 2 , 1 ,
+		IS_ISO10646_UCS4(FONT_CS(font->id))) ;
 
 	return  1 ;
 }
@@ -3221,7 +3288,7 @@ x_window_draw_image_string(
 	 * In case US_ASCII characters is drawn by Unicode font.
 	 * 8 bit charcter => 16 bit character.
 	 */
-	if( FONT_CS(font->id) == ISO10646_UCS4_1)
+	if( IS_ISO10646_UCS4(FONT_CS(font->id)))
 	{
 		u_char *  dbl_str ;
 
@@ -3236,12 +3303,12 @@ x_window_draw_image_string(
 				dbl_str[count * 2 + 1] = 0x0 ;
 			}
 
-			draw_string( win , font , x , y , dbl_str , len * 2 , 0) ;
+			draw_string( win , font , x , y , dbl_str , len * 2 , 0 , 1) ;
 		}
 	}
 	else
 	{
-		draw_string( win , font , x , y , str , len , 0) ;
+		draw_string( win , font , x , y , str , len , 0 , 0) ;
 	}
 
 	return  1 ;
@@ -3268,7 +3335,8 @@ x_window_draw_image_string16(
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 	x_gc_set_bg_color( win->gc, bg_color->pixel) ;
 
-	draw_string( win, font, x, y, (u_char*)str, len * 2 , 0) ;
+	draw_string( win, font, x, y, (u_char*)str, len * 2 , 0 ,
+		IS_ISO10646_UCS4(FONT_CS(font->id))) ;
 
 	return  1 ;
 }
