@@ -10,28 +10,24 @@
 
 #include  "win32/x.h"
 
+#elif  defined(USE_CONSOLE)
+
+#include  "console/x.h"
+
 #elif  defined(USE_FRAMEBUFFER)
 
 #include  "fb/x.h"
 
-#else	/* USE_WIN32GUI/USE_FRAMEBUFFER */
+#elif  defined(USE_QUARTZ)
 
-/* This must be included ahead of Xft.h on XFree86-4.0.x or before. */
-#include  <X11/Xlib.h>
-#include  <X11/keysym.h>	/* XK_xxx */
-#include  <X11/Xatom.h>		/* XA_xxx */
-#include  <X11/Xutil.h>		/* IsKeypadKey */
-#include  <X11/cursorfont.h>	/* for cursor shape */
+#include  "quartz/x.h"
 
-typedef Pixmap PixmapMask ;
-
-#ifdef  XK_F21
-#define  XK_FMAX  XK_F35
 #else
-#define  XK_FMAX  XK_F20
-#endif
 
-#endif	/* USE_WIN32GUI/USE_FRAMEBUFFER */
+#include  "xlib/x.h"
+#define  USE_XLIB
+
+#endif
 
 /*
  * Xlib utility definitions.
