@@ -4,6 +4,7 @@
 
 #include  "x_draw_str.h"
 
+#include  <kiklib/kik_mem.h>	/* alloca */
 #include  <ml_drcs.h>
 
 #ifndef  NO_IMAGE
@@ -20,6 +21,47 @@
 
 /* --- static functions --- */
 
+static u_int
+calculate_char_width(
+	x_font_t *  font ,
+	u_int  code ,
+	mkf_charset_t  cs ,
+	ml_char_t *  comb ,
+	u_int  comb_size ,
+	int *  draw_alone
+	)
+{
+	u_int  width ;
+
+	width = x_calculate_char_width( font , code , cs , draw_alone) ;
+
+	if( cs == ISO10646_UCS4_1_V)
+	{
+		int  w ;
+
+		for( ; comb_size > 0 ; comb_size-- , comb++)
+		{
+		#ifdef  DEBUG
+			if( ml_char_cs( comb) != ISO10646_UCS4_1_V)
+			{
+				kik_debug_printf( KIK_DEBUG_TAG " %x cs is unexpectedly"
+					" combined to ISO10646_UCS4_1_V.\n") ;
+
+				continue ;
+			}
+		#endif
+
+			if( ( w = ml_char_get_offset( comb) +
+			          (int)ml_char_get_width( comb)) > (int)width)
+			{
+				width = w ;
+			}
+		}
+	}
+
+	return  width ;
+}
+
 static void
 draw_line(
 	x_window_t *  window ,
@@ -191,8 +233,11 @@ draw_picture(
 			x_window_clear( window , dst_x , dst_y , dst_width , line_height) ;
 		}
 
-		if( src_width > 0 && src_height > 0 &&
-		    pic->disp == window->disp)
+		if( src_width > 0 && src_height > 0
+		#ifndef  INLINE_PICTURE_MOVABLE_BETWEEN_DISPLAYS
+		    && cur_pic->disp == window->disp
+		#endif
+		    )
 		{
 		#ifdef  __DEBUG
 			kik_debug_printf(
@@ -258,8 +303,15 @@ draw_picture(
 		src_x , src_y , src_width , src_height) ;
 #endif
 
-	x_window_copy_area( window , cur_pic->pixmap , cur_pic->mask ,
-		src_x , src_y , src_width , src_height , dst_x , dst_y) ;
+	if( src_width > 0 && src_height > 0
+	#ifndef  INLINE_PICTURE_MOVABLE_BETWEEN_DISPLAYS
+	    && cur_pic->disp == window->disp
+	#endif
+	    )
+	{
+		x_window_copy_area( window , cur_pic->pixmap , cur_pic->mask ,
+			src_x , src_y , src_width , src_height , dst_x , dst_y) ;
+	}
 
 	return  1 ;
 }
@@ -285,12 +337,23 @@ draw_drcs(
 	int  y ,
 	u_int  ch_width ,
 	u_int  line_height ,
-	x_color_t *  fg_xcolor
+	x_color_t *  fg_xcolor ,
+	int  size_attr
 	)
 {
-	int  y_off ;
+	int  y_off = 0 ;
 
-	for( y_off = 0 ; y_off < line_height ; y_off++)
+	if( size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		if( size_attr == DOUBLE_HEIGHT_BOTTOM)
+		{
+			y_off = line_height ;
+		}
+
+		line_height *= 2 ;
+	}
+
+	for( ; y_off < line_height ; y_off++)
 	{
 		u_int  w ;
 		int  x_off_sum ;	/* x_off for all glyphs */
@@ -331,10 +394,21 @@ draw_drcs(
 				}
 			}
 
-			left_x = ((int)((double)(x_off * glyph_width) /
-					(double)ch_width * 10 + 5)) / 10 - smpl_width / 2 ;
-			top_y = ((int)((double)(y_off * glyph_height) /
-					(double)line_height * 10 + 5)) / 10 - smpl_height / 2 ;
+			left_x = (x_off * glyph_width * 10 / ch_width + 5) / 10 - smpl_width / 2 ;
+			top_y = (y_off * glyph_height * 10 / line_height + 5) / 10
+					- smpl_height / 2 ;
+			/*
+			 * If top_y < 0 or top_y >= glyph_height, w is always 0
+			 * regardless of content of glyph.
+			 */
+			if( top_y < 0)
+			{
+				top_y = 0 ;
+			}
+			else if( top_y >= glyph_height)
+			{
+				top_y = glyph_height - 1 ;
+			}
 
 		#if  0
 			kik_debug_printf( KIK_DEBUG_TAG
@@ -354,9 +428,9 @@ draw_drcs(
 				for( smpl_x = 0 ; smpl_x < smpl_width ; smpl_x++)
 				{
 					if( 0 <= left_x + smpl_x &&
-					    left_x + smpl_x <= glyph_width &&
+					    left_x + smpl_x < glyph_width &&
 					    0 <= top_y + smpl_y &&
-					    top_y + smpl_y <= glyph_height)
+					    top_y + smpl_y < glyph_height)
 					{
 						if( get_drcs_bitmap( glyph , glyph_width ,
 							left_x + smpl_x , top_y + smpl_y))
@@ -387,8 +461,33 @@ draw_drcs(
 				continue ;
 			}
 
-			x_window_fill_with( window , fg_xcolor ,
-				x + x_off_sum - w , y + y_off , w , 1) ;
+			if( size_attr >= DOUBLE_HEIGHT_TOP)
+			{
+				int  exp_y ;
+
+				if( size_attr == DOUBLE_HEIGHT_TOP)
+				{
+					if( y_off >= line_height / 2)
+					{
+						return  1 ;
+					}
+
+					exp_y = y + y_off ;
+				}
+				else
+				{
+					exp_y = y + y_off - line_height / 2 ;
+				}
+
+				x_window_fill_with( window , fg_xcolor ,
+					x + x_off_sum - w , exp_y , w , 1) ;
+			}
+			else
+			{
+				x_window_fill_with( window , fg_xcolor ,
+					x + x_off_sum - w , y + y_off , w , 1) ;
+			}
+
 			w = 0 ;
 		}
 	}
@@ -450,11 +549,11 @@ get_state(
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 
-static int
+static void
 fc_draw_combining_chars(
 	x_window_t *  window ,
 	x_font_manager_t *  font_man ,
-	x_color_manager_t *  color_man ,
+	x_color_t *  xcolor ,		/* fg color */
 	ml_char_t *  chars ,
 	u_int  size ,
 	int  x ,
@@ -465,7 +564,6 @@ fc_draw_combining_chars(
 	u_int32_t  ch_code ;
 	mkf_charset_t  ch_cs ;
 	x_font_t *  xfont ;
-	x_color_t *  xcolor ;
 
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -477,7 +575,6 @@ fc_draw_combining_chars(
 		ch_code = ml_char_code( &chars[count]) ;
 		ch_cs = ml_char_cs( &chars[count]) ;
 		xfont = x_get_font( font_man , ml_char_font( &chars[count])) ;
-		xcolor = x_get_xcolor( color_man , ml_char_fg_color( &chars[count])) ;
 
 		if( ch_cs == DEC_SPECIAL)
 		{
@@ -498,15 +595,19 @@ fc_draw_combining_chars(
 		{
 			/* FcChar32 */ u_int32_t  ucs4 ;
 
-			if( ( ucs4 = x_convert_to_xft_ucs4( ch_code , ch_cs)))
+			if( ch_cs == ISO10646_UCS4_1_V)
+			{
+				x_window_ft_draw_string32( window , xfont , xcolor ,
+					x + ml_char_get_offset( &chars[count]) , y ,
+					&ch_code , 1) ;
+			}
+			else if( ( ucs4 = x_convert_to_xft_ucs4( ch_code , ch_cs)))
 			{
 				x_window_ft_draw_string32( window , xfont , xcolor ,
 					x , y , &ucs4 , 1) ;
 			}
 		}
 	}
-
-	return  1 ;
 }
 
 static int
@@ -593,8 +694,10 @@ fc_draw_str(
 	ch_code = ml_char_code( &chars[count]) ;
 	xfont = x_get_font( font_man , (font = ml_char_font( &chars[count]))) ;
 	ch_cs = FONT_CS(font) ;
+	comb_chars = ml_get_combining_chars( &chars[count] , &comb_size) ;
 
-	ch_width = x_calculate_char_width( xfont , ch_code , ch_cs , &draw_alone) ;
+	ch_width = calculate_char_width( xfont , ch_code , ch_cs ,
+			comb_chars , comb_size , &draw_alone) ;
 
 	if( ( current_width = x + ch_width) > window->width ||
 	    y + height > window->height)
@@ -608,8 +711,6 @@ fc_draw_str(
 		return  0 ;
 	}
 
-	comb_chars = ml_get_combining_chars( &chars[count] , &comb_size) ;
-
 	if( ( state = get_state( ch_cs , ch_code , comb_chars ,
 			&pic_glyph , &drcs_glyph , &draw_alone)) == 0 &&
 	    ch_cs != US_ASCII && ch_cs != ISO8859_1_R && ! IS_ISCII(ch_cs))
@@ -656,12 +757,14 @@ fc_draw_str(
 			if( ! ( ucs4 = x_convert_to_xft_ucs4( ch_code , ch_cs)))
 			{
 			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG " strange character , ignored.\n") ;
+				kik_warn_printf( KIK_DEBUG_TAG
+					" strange character 0x%x, ignored.\n" , ch_code) ;
 			#endif
-				ucs4 = 0x20 ;
 			}
-
-			str32[str_len++] = ucs4 ;
+			else
+			{
+				str32[str_len++] = ucs4 ;
+			}
 		}
 
 		/*
@@ -690,10 +793,11 @@ fc_draw_str(
 			next_bg_color = ml_char_bg_color( &chars[count]) ;
 			next_underline_style = ml_char_underline_style( &chars[count]) ;
 			next_is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
-			next_ch_width = x_calculate_char_width( next_xfont ,
-						ch_code , ch_cs , &next_draw_alone) ;
 			next_comb_chars = ml_get_combining_chars( &chars[count] ,
 							&next_comb_size) ;
+			next_ch_width = calculate_char_width( next_xfont , ch_code , ch_cs ,
+						next_comb_chars , next_comb_size ,
+						&next_draw_alone) ;
 
 			if( ( next_state = get_state( ch_cs , ch_code , next_comb_chars ,
 						&pic_glyph , &drcs_glyph ,
@@ -809,29 +913,29 @@ fc_draw_str(
 			else /* if( state == 3) */
 			{
 				draw_drcs( window , drcs_glyphs , str_len ,
-					x , y + top_margin ,
-					ch_width , height - top_margin - bottom_margin ,
-					fg_xcolor) ;
+					x , y , ch_width , height ,
+					fg_xcolor , font_man->size_attr) ;
 			}
 
 			if( comb_chars)
 			{
-				fc_draw_combining_chars( window , font_man , color_man ,
-					comb_chars , comb_size ,
-				/*
-				 * 'current_width' is for some thai fonts which automatically
-				 * draw combining chars.
-				 * e.g.)
-				 *  -thai-fixed-medium-r-normal--14-100-100-100-m-70-tis620.2529-1
-				 *  (distributed by ZzzThai http://zzzthai.fedu.uec.ac.jp/ZzzThai/)
-				 *  win32 unicode font.
-				 */
-				#if  0
-					current_width
-				#else
-					current_width - ch_width
-				#endif
-					, y + ascent) ;
+				fc_draw_combining_chars( window , font_man ,
+						fg_xcolor , comb_chars , comb_size ,
+					/*
+					 * 'current_width' is for some thai fonts which
+					 * automatically draw combining chars.
+					 * e.g.)
+					 *  -thai-fixed-medium-r-normal--14-100-100-100-m-70-tis620.2529-1
+					 *  (distributed by ZzzThai
+					 *   http://zzzthai.fedu.uec.ac.jp/ZzzThai/)
+					 *  win32 unicode font.
+					 */
+					#if  0
+						current_width
+					#else
+						current_width - ch_width
+					#endif
+						, y + ascent) ;
 			}
 
 			if( ! hide_underline && underline_style)
@@ -894,7 +998,7 @@ static int
 xcore_draw_combining_chars(
 	x_window_t *  window ,
 	x_font_manager_t *  font_man ,
-	x_color_manager_t *  color_man ,
+	x_color_t *  xcolor ,	/* fg color */
 	ml_char_t *  chars ,
 	u_int  size ,
 	int  x ,
@@ -905,7 +1009,7 @@ xcore_draw_combining_chars(
 	u_int32_t  ch_code ;
 	mkf_charset_t  ch_cs ;
 	x_font_t *  xfont ;
-	x_color_t *  xcolor ;
+	int  x_off ;
 
 	for( count = 0 ; count < size ; count ++)
 	{
@@ -917,7 +1021,6 @@ xcore_draw_combining_chars(
 		ch_code = ml_char_code( &chars[count]) ;
 		ch_cs = ml_char_cs( &chars[count]) ;
 		xfont = x_get_font( font_man , ml_char_font( &chars[count])) ;
-		xcolor = x_get_xcolor( color_man , ml_char_fg_color( &chars[count])) ;
 
 		if( ch_cs == DEC_SPECIAL)
 		{
@@ -926,35 +1029,50 @@ xcore_draw_combining_chars(
 			c = ch_code ;
 			x_window_draw_decsp_string( window , xfont , xcolor , x , y , &c , 1) ;
 		}
-		else if( ch_code < 0x100)
-		{
-			u_char  c ;
-
-			c = ch_code ;
-			x_window_draw_string( window , xfont , xcolor , x , y , &c , 1) ;
-		}
-		else if( ch_cs != ISO10646_UCS4_1)
-		{
-			XChar2b  xch ;
-
-			xch.byte1 = (ch_code >> 8) & 0xff ;
-			xch.byte2 = ch_code & 0xff ;
-
-			x_window_draw_string16( window , xfont , xcolor ,
-				x , y , &xch , 1) ;
-		}
 		else
 		{
-			/* UCS4 */
-
-			/* [2] is for surroage pair. */
-			XChar2b  xch[2] ;
-			u_int  len ;
-
-			if( ( len = (x_convert_ucs4_to_utf16( xch , ch_code) / 2)) > 0)
+			if( ch_cs == ISO10646_UCS4_1_V)
 			{
+				x_off = ml_char_get_offset( &chars[count]) ;
+			}
+			else
+			{
+				x_off = 0 ;
+			}
+
+			if( ch_code < 0x100)
+			{
+				u_char  c ;
+
+				c = ch_code ;
+				x_window_draw_string( window , xfont , xcolor ,
+					x + x_off , y , &c , 1) ;
+			}
+			else
+			{
+				/* UCS4 */
+
+				/* [2] is for surroage pair. */
+				XChar2b  xch[2] ;
+				u_int  len ;
+
+				if( IS_ISO10646_UCS4(ch_cs))
+				{
+					if( ( len = x_convert_ucs4_to_utf16( xch , ch_code) / 2)
+					    == 0)
+					{
+						continue ;
+					}
+				}
+				else
+				{
+					xch[0].byte1 = (ch_code >> 8) & 0xff ;
+					xch[0].byte2 = ch_code & 0xff ;
+					len = 1 ;
+				}
+
 				x_window_draw_string16( window , xfont , xcolor ,
-					x , y , xch , len) ;
+						x , y , xch , len) ;
 			}
 		}
 	}
@@ -1044,8 +1162,10 @@ xcore_draw_str(
 	ch_code = ml_char_code( &chars[count]) ;
 	xfont = x_get_font( font_man , (font = ml_char_font( &chars[count]))) ;
 	ch_cs = FONT_CS(font) ;
+	comb_chars = ml_get_combining_chars( &chars[count] , &comb_size) ;
 
-	ch_width = x_calculate_char_width( xfont , ch_code , ch_cs , &draw_alone) ;
+	ch_width = calculate_char_width( xfont , ch_code , ch_cs ,
+			comb_chars , comb_size , &draw_alone) ;
 
 	if( ( current_width = x + ch_width) > window->width ||
 	    y + height > window->height)
@@ -1059,8 +1179,6 @@ xcore_draw_str(
 		return  0 ;
 	}
 
-	comb_chars = ml_get_combining_chars( &chars[count] , &comb_size) ;
-
 	if( ( state = get_state( ch_cs , ch_code , comb_chars ,
 			&pic_glyph , &drcs_glyph , &draw_alone)) == 0 && ch_code >= 0x100)
 	{
@@ -1099,7 +1217,7 @@ xcore_draw_str(
 				drcs_glyphs[str_len++] = drcs_glyph ;
 			}
 		}
-		else if( ch_cs != ISO10646_UCS4_1)
+		else if( ! IS_ISO10646_UCS4(ch_cs))
 		{
 			str2b[str_len].byte1 = (ch_code >> 8) & 0xff ;
 			str2b[str_len].byte2 = ch_code & 0xff ;
@@ -1138,10 +1256,11 @@ xcore_draw_str(
 			next_bg_color = ml_char_bg_color( &chars[count]) ;
 			next_underline_style = ml_char_underline_style( &chars[count]) ;
 			next_is_crossed_out = ml_char_is_crossed_out( &chars[count]) ;
-			next_ch_width = x_calculate_char_width( next_xfont ,
-						ch_code , ch_cs , &next_draw_alone) ;
 			next_comb_chars = ml_get_combining_chars( &chars[count] ,
 							&next_comb_size) ;
+			next_ch_width = calculate_char_width( next_xfont , ch_code , ch_cs ,
+						next_comb_chars , next_comb_size ,
+						&next_draw_alone) ;
 
 			if( ( next_state = get_state( ch_cs , ch_code , next_comb_chars ,
 						&pic_glyph , &drcs_glyph ,
@@ -1214,8 +1333,7 @@ xcore_draw_str(
 			fg_xcolor = x_get_xcolor( color_man , fg_color) ;
 
 		#ifdef  USE_FRAMEBUFFER
-			if( x_window_has_wall_picture( window) &&
-			           bg_color == ML_BG_COLOR)
+			if( x_window_has_wall_picture( window) && bg_color == ML_BG_COLOR)
 			{
 				bg_xcolor = NULL ;
 			}
@@ -1225,19 +1343,78 @@ xcore_draw_str(
 				bg_xcolor = x_get_xcolor( color_man , bg_color) ;
 			}
 
+		#ifdef  USE_CONSOLE
+			/* XXX DRCS (state == 3) is ignored */
+			if( state < 3)
+			{
+				u_int  comb_count ;
+
+				for( comb_count = 0 ; comb_count < comb_size ; comb_count++)
+				{
+					u_int  comb_code ;
+
+					comb_code = ml_char_code( &comb_chars[comb_count]) ;
+
+					if( state <= 1)
+					{
+						str[str_len++] = comb_code ;
+					}
+					else if( ! IS_ISO10646_UCS4(ch_cs))
+					{
+						str2b[str_len].byte1 = (comb_code >> 8) & 0xff ;
+						str2b[str_len].byte2 = comb_code & 0xff ;
+						str_len ++ ;
+					}
+					else
+					{
+						/* UCS4 */
+						str_len += (x_convert_ucs4_to_utf16(
+								str2b + str_len ,
+								comb_code) / 2) ;
+					}
+				}
+
+				if( state == 2)
+				{
+					x_window_console_draw_string16( window , xfont ,
+						fg_xcolor , bg_xcolor ,
+						x , y + ascent , str2b , str_len ,
+						underline_style) ;
+				}
+				else /* if( state == 0) */
+				{
+					x_window_console_draw_string( window , xfont ,
+						fg_xcolor , bg_xcolor ,
+						x , y + ascent , str , str_len ,
+						underline_style) ;
+				}
+			}
+		#else	/* USE_CONSOLE */
+		#ifdef  USE_QUARTZ
+			if( 1)
+		#else	/* USE_QUARTZ */
 			if(
 		#ifdef  USE_FRAMEBUFFER
 			#ifdef  USE_FREETYPE
-			    xfont->is_proportional ||	/* ISCII */
+			    xfont->is_proportional ||	/* ISCII or ISO10646_UCS4_1_V */
 			#endif
 			    /* draw_alone || */ /* draw_alone is always false on framebuffer. */
-		#else
+		#else	/* USE_FRAMEBUFFER */
+			#if  defined(USE_WIN32GUI) && defined(USE_OT_LAYOUT)
+			    /*
+			     * U+2022 is ambiguous and should be drawn one by one, but
+			     * x_calculate_char_width() can't tell it as ambigous if
+			     * use_ot_layout is true because ch_code is glyph index.
+			     */
+			    (xfont->use_ot_layout /* && xfont->ot_font */) ||
+			#endif
 			    ( x_window_has_wall_picture( window) &&
 			      bg_color == ML_BG_COLOR) ||
 			    draw_alone ||
-		#endif
-			    bottom_margin + top_margin > 0 /* == line space XXX */ ||
+		#endif	/* USE_FRAMEBUFFER */
+			    xfont->height != height ||
 			    state == 3)
+		#endif	/* USE_QUARTZ */
 			{
 				if( bg_color == ML_BG_COLOR)
 				{
@@ -1272,9 +1449,8 @@ xcore_draw_str(
 				else /* if( state == 3) */
 				{
 					draw_drcs( window , drcs_glyphs , str_len ,
-						x , y + top_margin ,
-						ch_width , height - top_margin - bottom_margin ,
-						fg_xcolor) ;
+						x , y , ch_width , height ,
+						fg_xcolor , font_man->size_attr) ;
 				}
 			}
 			else
@@ -1301,7 +1477,7 @@ xcore_draw_str(
 
 			if( comb_chars)
 			{
-				xcore_draw_combining_chars( window , font_man , color_man ,
+				xcore_draw_combining_chars( window , font_man , fg_xcolor ,
 					comb_chars , comb_size ,
 				/*
 				 * 'current_width' is for some thai fonts which automatically
@@ -1333,6 +1509,7 @@ xcore_draw_str(
 					x , y , current_width - x , height , ascent ,
 					top_margin) ;
 			}
+		#endif	/* USE_CONSOLE */
 
 		end_draw:
 			start_draw = 0 ;
@@ -1392,39 +1569,51 @@ x_draw_str(
 	)
 {
 	u_int  updated_width ;
+	int  ret ;
+
+	if( font_man->size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		x_window_set_clip( window , x , y , window->width - x , height) ;
+		ascent = height - (height - ascent) * 2 ;
+
+		if( font_man->size_attr == DOUBLE_HEIGHT_TOP)
+		{
+			ascent += height ;
+		}
+	}
 
 	switch( x_get_type_engine( font_man))
 	{
 	default:
-		return  0 ;
+		ret = 0 ;
+		break ;
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	case  TYPE_XFT:
 	case  TYPE_CAIRO:
-		if( ! fc_draw_str( window , font_man , color_man , &updated_width ,
+		ret = fc_draw_str( window , font_man , color_man , &updated_width ,
 			chars , num_of_chars , x , y , height , ascent ,
-			top_margin , bottom_margin , hide_underline))
-		{
-			return  0 ;
-		}
+			top_margin , bottom_margin , hide_underline) ;
 
 		break ;
 #endif
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	case  TYPE_XCORE:
-		if( ! xcore_draw_str( window , font_man , color_man , &updated_width ,
+		ret = xcore_draw_str( window , font_man , color_man , &updated_width ,
 			chars , num_of_chars , x , y , height , ascent ,
-			top_margin , bottom_margin , hide_underline))
-		{
-			return  0 ;
-		}
+			top_margin , bottom_margin , hide_underline) ;
 
 		break ;
 #endif
 	}
-	
-	return  1 ;
+
+	if( font_man->size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		x_window_unset_clip( window) ;
+	}
+
+	return  ret ;
 }
 
 int
@@ -1444,36 +1633,43 @@ x_draw_str_to_eol(
 	)
 {
 	u_int  updated_width ;
+	int  ret ;
+
+	if( font_man->size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		x_window_set_clip( window , x , y , window->width - x , height) ;
+		ascent = height - (height - ascent) * 2 ;
+
+		if( font_man->size_attr == DOUBLE_HEIGHT_TOP)
+		{
+			ascent += height ;
+		}
+	}
 
 	switch( x_get_type_engine( font_man))
 	{
 	default:
-		return  0 ;
+		ret = 0 ;
+		break ;
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	case  TYPE_XFT:
 	case  TYPE_CAIRO:
 		x_window_clear( window , x , y , window->width - x , height) ;
 
-		if( ! fc_draw_str( window , font_man , color_man ,
+		ret = fc_draw_str( window , font_man , color_man ,
 			NULL /* NULL disables x_window_clear() in fc_draw_str() */ ,
 			chars , num_of_chars , x , y , height , ascent ,
-			top_margin , bottom_margin , hide_underline))
-		{
-			return  0 ;
-		}
+			top_margin , bottom_margin , hide_underline) ;
 
 		break ;
 #endif
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	case  TYPE_XCORE:
-		if( ! xcore_draw_str( window , font_man , color_man , &updated_width ,
+		ret = xcore_draw_str( window , font_man , color_man , &updated_width ,
 			chars , num_of_chars , x , y , height , ascent ,
-			top_margin , bottom_margin , hide_underline))
-		{
-			return	0 ;
-		}
+			top_margin , bottom_margin , hide_underline) ;
 
 		if( updated_width < window->width)
 		{
@@ -1485,5 +1681,35 @@ x_draw_str_to_eol(
 #endif
 	}
 
-	return	1 ;
+	if( font_man->size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		x_window_unset_clip( window) ;
+	}
+
+	return	ret ;
+}
+
+u_int
+x_calculate_mlchar_width(
+	x_font_t *  font ,
+	ml_char_t *  ch ,
+	int *  draw_alone
+	)
+{
+	mkf_charset_t  cs ;
+	ml_char_t *  comb ;
+	u_int  comb_size ;
+
+	if( ( cs = FONT_CS(font->id)) == ISO10646_UCS4_1_V)
+	{
+		comb = ml_get_combining_chars( ch , &comb_size) ;
+	}
+	else
+	{
+		comb = NULL ;
+		comb_size = 0 ;
+	}
+
+	return  calculate_char_width(  font , ml_char_code(ch) , cs , comb , comb_size ,
+			draw_alone) ;
 }
