@@ -14,7 +14,7 @@
 #endif
 
 #include  <kiklib/kik_debug.h>
-#include  <kiklib/kik_mem.h>	/* alloca/kik_alloca_garbage_collect/malloc/free */
+#include  <kiklib/kik_mem.h>	/* realloc/free */
 #include  <kiklib/kik_types.h>	/* u_int */
 #include  <ml_term_manager.h>
 
@@ -74,21 +74,19 @@ receive_next_event(void)
 	u_int  num_of_terms ;
 	int  xfd ;
 	int  ptyfd ;
-#ifdef  USE_LIBSSH2
-	int *  xssh_fds ;
-	u_int  num_of_xssh_fds ;
-#endif
 	int  maxfd ;
 	int  ret ;
 	fd_set  read_fds ;
 	struct timeval  tval ;
 	x_display_t **  displays ;
 	u_int  num_of_displays ;
-
-	num_of_terms = ml_get_all_terms( &terms) ;
 #ifdef  USE_LIBSSH2
+	int *  xssh_fds ;
+	u_int  num_of_xssh_fds ;
+
 	num_of_xssh_fds = ml_pty_ssh_get_x11_fds( &xssh_fds) ;
 #endif
+	num_of_terms = ml_get_all_terms( &terms) ;
 
 	while( 1)
 	{
@@ -157,7 +155,7 @@ receive_next_event(void)
 
 		for( count = 0 ; count < num_of_displays ; count ++)
 		{
-		#ifdef  X_PROTOCOL
+		#if  defined(X_PROTOCOL) && ! defined(USE_QUARTZ)
 			/*
 			 * Need to read pending events and to flush events in
 			 * output buffer on X11 before waiting in select().
