@@ -31,6 +31,8 @@ typedef enum x_font_present
 
 typedef struct _XftFont *  xft_font_ptr_t ;
 typedef struct _cairo_scaled_font *  cairo_scaled_font_ptr_t ;
+typedef struct _FcCharSet *  fc_charset_ptr_t ;
+typedef struct _FcPattern *  fc_pattern_ptr_t ;
 
 /* defined in xlib/x_decsp_font.h */
 typedef struct x_decsp_font *  x_decsp_font_ptr_t ;
@@ -47,15 +49,25 @@ typedef struct x_font
 	 */
 	ml_font_t  id ;
 
-#ifdef  USE_WIN32GUI
+#if  defined(USE_WIN32GUI)
 	Font  fid ;
 	mkf_conv_t *  conv ;
+#elif  defined(USE_QUARTZ)
+	void *  cg_font ;
+	u_int  size ;
 #else
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	xft_font_ptr_t  xft_font ;
 #endif
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	cairo_scaled_font_ptr_t  cairo_font ;
+	struct
+	{
+		fc_charset_ptr_t  charset ;
+		void *  next ;
+
+	} *  compl_fonts ;
+	fc_pattern_ptr_t  pattern ;
 #endif
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	XFontStruct *  xfont ;
@@ -64,6 +76,16 @@ typedef struct x_font
 
 	x_decsp_font_ptr_t  decsp_font ;
 
+#ifdef  USE_OT_LAYOUT
+	/* ot_font == NULL and use_ot_layout == true is possible in ISO10646_UCS4_1_V font. */
+	void *  ot_font ;
+#ifdef  USE_WIN32GUI
+	u_int16_t  size ;	/* font size */
+#endif
+	int8_t  ot_font_not_found ;
+	int8_t  use_ot_layout ;
+#endif
+
 	/*
 	 * These members are never zero.
 	 */
@@ -83,13 +105,15 @@ typedef struct x_font
 	int8_t  is_proportional ;
 	int8_t  is_vertical ;
 	int8_t  double_draw_gap ;
+	int8_t  size_attr ;
 
 } x_font_t ;
 
 
 int  x_compose_dec_special_font(void) ;
 
-x_font_t *  x_font_new( Display *  display , ml_font_t  id , x_type_engine_t  type_engine ,
+x_font_t *  x_font_new( Display *  display , ml_font_t  id , int  size_attr ,
+	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present , const char *  fontname , u_int  fontsize ,
 	u_int  col_width , int  use_medium_for_bold , u_int  letter_space) ;
 
@@ -108,7 +132,13 @@ int  x_change_font_cols( x_font_t *  font , u_int  cols) ;
 u_int  x_calculate_char_width( x_font_t *  font , u_int32_t  ch ,
 	mkf_charset_t  cs , int *  draw_alone) ;
 
-#if  ! defined(USE_FRAMEBUFFER) && ! defined(USE_WIN32GUI)
+int  x_font_has_ot_layout_table( x_font_t *  font) ;
+
+u_int  x_convert_text_to_glyphs( x_font_t *  font , u_int32_t *  shaped , u_int  shaped_len ,
+	int8_t *  offsets , u_int8_t *  widths , u_int32_t *  cmapped ,
+	u_int32_t *  src , u_int  src_len , const char *  script , const char *  features) ;
+
+#ifdef  USE_XLIB
 char **  x_font_get_encoding_names( mkf_charset_t  cs) ;
 
 /* For mlterm-libvte */
@@ -117,8 +147,8 @@ void  x_font_set_dpi_for_fc( double  dpi) ;
 #define  x_font_get_encoding_names(cs)  (0)
 #endif
 
-#ifndef  USE_FRAMEBUFFER
-void  x_font_use_point_size( int  bool) ;
+#ifdef  SUPPORT_POINT_SIZE_FONT
+void  x_font_use_point_size( int  use) ;
 #else
 #define  x_font_use_point_size(bool)  (0)
 #endif
