@@ -23,10 +23,16 @@ typedef struct  x_font_manager
 	u_int8_t  step_in_changing_font_size ;
 	int8_t  use_bold_font ;
 	int8_t  use_italic_font ;
+	int8_t  size_attr ;
+#ifdef  USE_OT_LAYOUT
+	int8_t  use_ot_layout ;
+#endif
 
 }  x_font_manager_t ;
 
 
+int  x_set_font_size_range( u_int  min_font_size , u_int  max_font_size) ;
+
 x_font_manager_t *  x_font_manager_new( Display *  display , x_type_engine_t  type_engine ,
 	x_font_present_t  font_present , u_int  font_size , mkf_charset_t  usascii_font_cs ,
 	int  use_multi_col_char , u_int  step_in_changing_font_size , u_int  letter_space ,
@@ -34,7 +40,9 @@ x_font_manager_t *  x_font_manager_new( Display *  display , x_type_engine_t  ty
 	
 int  x_font_manager_delete( x_font_manager_t *  font_man) ;
 
-x_font_t *  x_get_font( x_font_manager_t *  font_man , ml_font_t  fontattr) ;
+void  x_font_manager_set_attr( x_font_manager_t *  font_man , int  size_attr , int  use_ot_layout) ;
+
+x_font_t *  x_get_font( x_font_manager_t *  font_man , ml_font_t  font) ;
 
 #define  x_get_usascii_font( font_man)  ((font_man)->font_cache->usascii_font)
 
@@ -73,7 +81,7 @@ int  x_set_use_italic_font( x_font_manager_t *  font_man , int  use_italic_font)
 
 #define  x_is_using_italic_font( font_man)  ((font_man)->use_italic_font)
 
-XFontSet  x_get_fontset( x_font_manager_t *  font_man) ;
+#define  x_get_fontset( font_man)  x_font_cache_get_fontset( (font_man)->font_cache)
 
 #define  x_get_current_usascii_font_cs( font_man)  ((font_man)->font_cache->usascii_font_cs)
 
