@@ -6,12 +6,18 @@
 
 #ifdef  USE_IM_PLUGIN
 
+#include  <kiklib/kik_mem.h>
 #include  "ml_str.h"
 #include  "ml_vt100_parser.h"
 #include  "x_draw_str.h"
 
+#ifdef  USE_CONSOLE
+#define  MARGIN		0
+#define  LINE_SPACE	0
+#else
 #define  MARGIN		3
 #define  LINE_SPACE	2
+#endif
 
 
 /* --- static functions --- */
@@ -46,7 +52,7 @@ adjust_window_position_by_size(
 	}
 }
 
-#ifdef  USE_FRAMEBUFFER
+#ifdef  MANAGE_WINDOWS_BY_MYSELF
 static void
 reset_screen(
 	x_window_t *  win
@@ -71,7 +77,7 @@ static void
 draw_screen(
 	x_im_status_screen_t *  stat_screen ,
 	int  do_resize ,
-	int  modified_beg
+	int  modified_beg	/* for canna */
 	)
 {
 #define  MAX_ROWS ((sizeof(stat_screen->head_indexes) / sizeof(stat_screen->head_indexes[0])) - 1)
@@ -80,6 +86,7 @@ draw_screen(
 	int *  heads ;
 	u_int  i ;
 
+	x_font_manager_set_attr( stat_screen->font_man , 0 , 0) ;
 	xfont = x_get_usascii_font( stat_screen->font_man) ;
 	line_height = xfont->height + LINE_SPACE ;
 	heads = stat_screen->head_indexes ;
@@ -182,9 +189,11 @@ draw_screen(
 				x_window_move( &stat_screen->window , x , y) ;
 			}
 
-		#ifdef  USE_FRAMEBUFFER
+		#ifdef  MANAGE_WINDOWS_BY_MYSELF
 			reset_screen( &stat_screen->window) ;
 		#endif
+
+			modified_beg = 0 ;
 		}
 	}
 
@@ -204,8 +213,7 @@ draw_screen(
 			x_draw_str_to_eol( &stat_screen->window ,
 				   stat_screen->font_man ,
 				   stat_screen->color_man ,
-				   stat_screen->chars + heads[i] ,
-				   len ,
+				   stat_screen->chars + heads[i] , len ,
 				   0 , line_height * i ,
 				   line_height ,
 				   xfont->ascent + LINE_SPACE / 2 ,
@@ -243,9 +251,7 @@ show(
 	x_im_status_screen_t *  stat_screen
 	)
 {
-	x_window_map( &stat_screen->window) ;
-
-	return  1 ;
+	return  x_window_map( &stat_screen->window) ;
 }
 
 static int
@@ -253,9 +259,7 @@ hide(
 	x_im_status_screen_t *  stat_screen
 	)
 {
-	x_window_unmap( &stat_screen->window) ;
-
-	return  1 ;
+	return  x_window_unmap( &stat_screen->window) ;
 }
 
 static int
@@ -273,7 +277,7 @@ set_spot(
 	if( stat_screen->window.x != x || stat_screen->window.y != y)
 	{
 		x_window_move( &stat_screen->window , x , y) ;
-	#ifdef  USE_FRAMEBUFFER
+	#ifdef  MANAGE_WINDOWS_BY_MYSELF
 		reset_screen( &stat_screen->window) ;
 		draw_screen( stat_screen , 0 , 0) ;
 	#endif
@@ -293,7 +297,7 @@ set(
 	u_char *  str
 	)
 {
-	int  count = 0 ;
+	int  count ;
 	mkf_char_t  ch ;
 	ml_char_t *  p ;
 	ml_char_t *  old_chars ;
@@ -307,11 +311,7 @@ set(
 
 	(*parser->init)( parser) ;
 	(*parser->set_str)( parser , str , strlen( str)) ;
-
-	while( (*parser->next_char)( parser , &ch))
-	{
-		count++ ;
-	}
+	for( count = 0 ; (*parser->next_char)( parser , &ch) ; count++) ;
 
 	old_chars = stat_screen->chars ;
 	old_num_of_chars = stat_screen->num_of_chars ;
@@ -398,7 +398,10 @@ set(
 		ml_str_delete( old_chars , old_num_of_chars) ;
 	}
 
-	draw_screen( stat_screen , 1 , modified_beg) ;
+	if( modified_beg < old_filled_len || old_filled_len != stat_screen->filled_len)
+	{
+		draw_screen( stat_screen , 1 , modified_beg) ;
+	}
 
 	return  1 ;
 }
@@ -488,9 +491,6 @@ x_im_status_screen_new(
 		goto  error ;
 	}
 
-	x_window_init_event_mask( &stat_screen->window , 0) ;
-
-
 	/*
 	 * +------------+
 	 * | x_window.c | --- window events ---> +----------------------+
