@@ -8,7 +8,14 @@
 #include  <kiklib/kik_str.h>
 
 
+#ifdef  USE_CONSOLE
+static u_int  dummy_arg ;
+#define  SEPARATOR_WIDTH  x_get_opened_displays(&dummy_arg)[0]->display->col_width
+#define  SEPARATOR_HEIGHT  x_get_opened_displays(&dummy_arg)[0]->display->line_height
+#else
 #define  SEPARATOR_WIDTH  1
+#define  SEPARATOR_HEIGHT  1
+#endif
 #define  SCROLLBAR_WIDTH(scrollbar)  (ACTUAL_WIDTH(&(scrollbar).window) + SEPARATOR_WIDTH)
 
 
@@ -20,19 +27,24 @@ modify_separator(
 	u_int  total ,
 	u_int  next_min ,
 	u_int  unit ,
-	u_int  fixed
+	u_int  fixed ,
+	u_int  margin
 	)
 {
 	u_int  surplus ;
 
-	if( total < separator + SEPARATOR_WIDTH + next_min)
+	if( total < separator + margin + next_min)
 	{
-		separator = total - next_min - SEPARATOR_WIDTH ;
+		separator = total - next_min - margin ;
 	}
 
 	if( ( surplus = (separator - fixed) % unit) > 0)
 	{
-		if( separator - fixed > surplus)
+		if( surplus > unit / 2)
+		{
+			separator += (unit - surplus) ;
+		}
+		else if( separator - fixed > surplus)
 		{
 			separator -= surplus ;
 		}
@@ -68,7 +80,8 @@ reset_layout(
 				x_col_width( term->screen) ,
 				term->screen->window.hmargin * 2 +
 				(term->sb_mode != SBM_NONE ?
-					SCROLLBAR_WIDTH(term->scrollbar) : 0)) ;
+					SCROLLBAR_WIDTH(term->scrollbar) : 0) ,
+				SEPARATOR_WIDTH) ;
 	}
 	else
 	{
@@ -82,7 +95,8 @@ reset_layout(
 				term->next[1]->screen->window.vmargin * 2 +
 				x_line_height( term->next[1]->screen) ,
 				x_line_height( term->screen) ,
-				term->screen->window.vmargin * 2) ;
+				term->screen->window.vmargin * 2 ,
+				SEPARATOR_HEIGHT) ;
 	}
 	else
 	{
@@ -117,7 +131,7 @@ reset_layout(
 			ACTUAL_WIDTH(&term->scrollbar.window) ,
 			child_height , NOTIFY_TO_MYSELF) ;
 
-	#if  defined(USE_FRAMEBUFFER)
+	#ifdef  MANAGE_WINDOWS_BY_MYSELF
 		x_window_fill_with( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
 				&X_SCREEN_TO_LAYOUT(term->screen)->window.fg_color ,
 				sep_x , y , SEPARATOR_WIDTH , child_height) ;
@@ -144,25 +158,35 @@ reset_layout(
 			child_width , child_height , NOTIFY_TO_MYSELF) ;
 	}
 
+#ifdef  USE_QUARTZ
+	/* x_window_move() clears screen in true transparency on MacOSX/Cocoa. */
+	if( term->screen->color_man->alpha < 255)
+	{
+		x_window_update_all( &term->screen->window) ;
+	}
+#endif
+
 	if( term->yfirst)
 	{
 		if( term->next[1] && term->separator_y > 0)
 		{
-			reset_layout( term->next[1] , x , y + term->separator_y + SEPARATOR_WIDTH ,
-				width , height - term->separator_y - SEPARATOR_WIDTH) ;
-		#ifdef  USE_FRAMEBUFFER
+			reset_layout( term->next[1] ,
+				x , y + term->separator_y + SEPARATOR_HEIGHT ,
+				width , height - term->separator_y - SEPARATOR_HEIGHT) ;
+		#ifdef  MANAGE_WINDOWS_BY_MYSELF
 			x_window_fill_with( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
 				&X_SCREEN_TO_LAYOUT(term->screen)->window.fg_color ,
 				x , y + term->separator_y ,
-				width , SEPARATOR_WIDTH) ;
+				width , SEPARATOR_HEIGHT) ;
 		#endif
 		}
 
 		if( term->next[0] && term->separator_x > 0)
 		{
-			reset_layout( term->next[0] , x + term->separator_x + SEPARATOR_WIDTH , y ,
+			reset_layout( term->next[0] ,
+				x + term->separator_x + SEPARATOR_WIDTH , y ,
 				width - term->separator_x - SEPARATOR_WIDTH , child_height) ;
-		#ifdef  USE_FRAMEBUFFER
+		#ifdef  MANAGE_WINDOWS_BY_MYSELF
 			x_window_fill_with( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
 				&X_SCREEN_TO_LAYOUT(term->screen)->window.fg_color ,
 				x + term->separator_x , y ,
@@ -174,9 +198,10 @@ reset_layout(
 	{
 		if( term->next[0] && term->separator_x > 0)
 		{
-			reset_layout( term->next[0] , x + term->separator_x + SEPARATOR_WIDTH , y ,
+			reset_layout( term->next[0] ,
+				x + term->separator_x + SEPARATOR_WIDTH , y ,
 				width - term->separator_x - SEPARATOR_WIDTH , height) ;
-		#ifdef  USE_FRAMEBUFFER
+		#ifdef  MANAGE_WINDOWS_BY_MYSELF
 			x_window_fill_with( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
 				&X_SCREEN_TO_LAYOUT(term->screen)->window.fg_color ,
 				x + term->separator_x , y ,
@@ -186,13 +211,14 @@ reset_layout(
 
 		if( term->next[1] && term->separator_y > 0)
 		{
-			reset_layout( term->next[1] , x , y + term->separator_y + SEPARATOR_WIDTH ,
-				child_width , height - term->separator_y - SEPARATOR_WIDTH) ;
-		#ifdef  USE_FRAMEBUFFER
+			reset_layout( term->next[1] ,
+				x , y + term->separator_y + SEPARATOR_HEIGHT ,
+				child_width , height - term->separator_y - SEPARATOR_HEIGHT) ;
+		#ifdef  MANAGE_WINDOWS_BY_MYSELF
 			x_window_fill_with( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
 				&X_SCREEN_TO_LAYOUT(term->screen)->window.fg_color ,
 				x , y + term->separator_y ,
-				child_width , SEPARATOR_WIDTH) ;
+				child_width , SEPARATOR_HEIGHT) ;
 		#endif
 		}
 	}
@@ -487,7 +513,7 @@ window_resized(
 	#endif
 	}
 
-	reset_layout( &layout->term , 0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+	reset_layout( &layout->term , 0 , 0 , win->width , win->height) ;
 }
 
 static void
@@ -504,8 +530,8 @@ child_window_resized(
 	if( layout->term.next[0] || layout->term.next[1])
 	{
 		reset_layout( &layout->term , 0 , 0 ,
-			ACTUAL_WIDTH(&layout->window) ,
-			ACTUAL_HEIGHT(&layout->window)) ;
+			layout->window.width ,
+			layout->window.height) ;
 
 		return ;
 	}
@@ -679,6 +705,52 @@ window_deleted(
 	window_deleted_intern( &layout->term) ;
 }
 
+#ifndef  USE_QUARTZ
+static void  change_sb_mode_intern( struct terminal *  term ,
+		x_sb_mode_t  new_mode , int  dynamic_change) ;
+
+static void
+check_scrollbar_visible(
+	struct terminal *  term
+	)
+{
+	if( term->autohide_scrollbar && term->sb_mode == SBM_RIGHT)
+	{
+		if( term->scrollbar.window.button_is_pressing)
+		{
+			term->idling_count = 0 ;
+		}
+		else if( ++term->idling_count == 30)
+		{
+			term->idling_count = 0 ;
+			change_sb_mode_intern( term , SBM_NONE , 1) ;
+		}
+	}
+
+	if( term->next[0])
+	{
+		check_scrollbar_visible( term->next[0]) ;
+	}
+
+	if( term->next[1])
+	{
+		check_scrollbar_visible( term->next[1]) ;
+	}
+}
+
+static void
+window_idling(
+	x_window_t *  win
+	)
+{
+	x_layout_t *  layout ;
+
+	layout = (x_layout_t*) win ;
+
+	check_scrollbar_visible( &layout->term) ;
+}
+#endif
+
 
 #if  0
 /*
@@ -985,8 +1057,33 @@ sb_mode(
 	struct terminal *  term ;
 
 	term = p ;
-	
-	return  term->sb_mode ;
+
+	if( term->autohide_scrollbar)
+	{
+		return  SBM_AUTOHIDE ;
+	}
+	else
+	{
+		return  term->sb_mode ;
+	}
+}
+
+static void
+screen_color_changed(
+	void *  p
+	)
+{
+	x_layout_t *  layout ;
+
+	layout = X_SCREEN_TO_LAYOUT(((struct terminal*)p)->screen) ;
+
+	if( &layout->term == p)
+	{
+		x_window_set_fg_color( &layout->window ,
+			&layout->term.screen->window.fg_color) ;
+		x_window_set_bg_color( &layout->window ,
+			&layout->term.screen->window.bg_color) ;
+	}
 }
 
 static u_int
@@ -1022,7 +1119,7 @@ total_height(
 
 	if( term->separator_y > 0 && term->next[1])
 	{
-		height += (total_height( term->next[1]) + SEPARATOR_WIDTH) ;
+		height += (total_height( term->next[1]) + SEPARATOR_HEIGHT) ;
 	}
 
 	return  height ;
@@ -1064,7 +1161,7 @@ total_hint_size(
 
 	if( term->next[1])
 	{
-		*height += SEPARATOR_WIDTH ;
+		*height += SEPARATOR_HEIGHT ;
 
 		total_hint_size( term->next[1] , width , height , width_inc , height_inc) ;
 	}
@@ -1086,16 +1183,65 @@ update_normal_hints(
 		width_inc , height_inc) ;
 }
 
-static void
-change_sb_mode(
-	void *  p ,
-	x_sb_mode_t  new_mode
+#ifndef  USE_QUARTZ
+static int
+need_idling_event(
+	struct terminal *  term
 	)
 {
-	struct terminal *  term ;
+	if( ! term->autohide_scrollbar &&
+	    ( ! term->next[0] || ! need_idling_event( term->next[0])) &&
+	    ( ! term->next[1] || ! need_idling_event( term->next[1])))
+	{
+		return  0 ;
+	}
+	else
+	{
+		return  1 ;
+	}
+}
+#endif
+
+static void
+change_sb_mode_intern(
+	struct terminal *  term ,
+	x_sb_mode_t  new_mode ,
+	int  dynamic_change
+	)
+{
+	x_layout_t *  layout ;
 	x_sb_mode_t  old_mode ;
 
-	term = p ;
+	layout = X_SCREEN_TO_LAYOUT(term->screen) ;
+
+	if( new_mode == SBM_AUTOHIDE)
+	{
+		if( term->autohide_scrollbar)
+		{
+			return ;
+		}
+
+		new_mode = SBM_NONE ;
+		term->autohide_scrollbar = 1 ;
+		x_window_add_event_mask( &term->screen->window , PointerMotionMask) ;
+	#ifndef  USE_QUARTZ
+		layout->window.idling = window_idling ;
+	#endif
+	}
+	else if( ! dynamic_change)
+	{
+		term->autohide_scrollbar = 0 ;
+		x_window_remove_event_mask( &term->screen->window , PointerMotionMask) ;
+		(*term->screen->xterm_listener.set_mouse_report)(
+			term->screen->xterm_listener.self) ;
+
+	#ifndef  USE_QUARTZ
+		if( ! need_idling_event( &layout->term))
+		{
+			layout->window.idling = NULL ;
+		}
+	#endif
+	}
 
 	if( (old_mode = term->sb_mode) == new_mode)
 	{
@@ -1112,7 +1258,7 @@ change_sb_mode(
 	{
 		x_window_unmap( &term->scrollbar.window) ;
 	}
-#ifdef  USE_FRAMEBUFFER
+#ifdef  MANAGE_WINDOWS_BY_MYSELF
 	else if( old_mode == SBM_NONE)
 	{
 		/* scrollbar's height may have been changed. */
@@ -1134,9 +1280,7 @@ change_sb_mode(
 		}
 	}
 
-	reset_layout( &X_SCREEN_TO_LAYOUT( term->screen)->term , 0 , 0 ,
-		ACTUAL_WIDTH( &X_SCREEN_TO_LAYOUT( term->screen)->window) ,
-		ACTUAL_HEIGHT( &X_SCREEN_TO_LAYOUT( term->screen)->window)) ;
+	reset_layout( &layout->term , 0 , 0 , layout->window.width , layout->window.height) ;
 
 	if( new_mode == SBM_NONE)
 	{
@@ -1158,19 +1302,28 @@ change_sb_mode(
 		goto  noresize ;
 	}
 
-	if( ! x_window_resize_with_margin( &X_SCREEN_TO_LAYOUT(term->screen)->window ,
-		total_width( &X_SCREEN_TO_LAYOUT(term->screen)->term) ,
-		total_height( &X_SCREEN_TO_LAYOUT(term->screen)->term) ,
-		NOTIFY_TO_MYSELF))
+	if( ( term->screen->window.x + ACTUAL_WIDTH(&term->screen->window) +
+	      (old_mode != SBM_NONE ? SCROLLBAR_WIDTH(term->scrollbar) : 0) <
+		ACTUAL_WIDTH(&layout->window)) ||
+	    ! x_window_resize( &layout->window , total_width( &layout->term) ,
+		total_height( &layout->term) , NOTIFY_TO_MYSELF))
 	{
 	noresize:
-		reset_layout( &X_SCREEN_TO_LAYOUT( term->screen)->term , 0 , 0 ,
-			ACTUAL_WIDTH( &X_SCREEN_TO_LAYOUT( term->screen)->window) ,
-			ACTUAL_HEIGHT( &X_SCREEN_TO_LAYOUT( term->screen)->window)) ;
+		reset_layout( &layout->term , 0 , 0 , layout->window.width ,
+			layout->window.height) ;
 	}
 #endif
 
-	update_normal_hints( X_SCREEN_TO_LAYOUT( term->screen)) ;
+	update_normal_hints( layout) ;
+}
+
+static void
+change_sb_mode(
+	void *  p ,
+	x_sb_mode_t  new_mode
+	)
+{
+	change_sb_mode_intern( p , new_mode , 0) ;
 }
 
 static void
@@ -1188,6 +1341,50 @@ term_changed(
 	x_scrollbar_set_num_of_filled_log_lines( &term->scrollbar , logged_lines) ;
 }
 
+
+static void
+screen_pointer_motion(
+	x_window_t *  win ,
+	XMotionEvent *  event
+	)
+{
+	struct terminal *  term ;
+	x_layout_t *  layout ;
+
+	layout = X_SCREEN_TO_LAYOUT((x_screen_t*)win) ;
+	(*layout->pointer_motion)( win , event) ;
+
+	term = search_term( &layout->term , (x_screen_t*)win) ;
+
+	if( term->autohide_scrollbar)
+	{
+		x_sb_mode_t  mode ;
+
+		if( ((int)win->width) - 2 <= event->x &&
+		    0 <= event->y && event->y < win->height)
+		{
+			if( term->scrollbar.window.is_mapped)
+			{
+				return ;
+			}
+
+			mode = SBM_RIGHT ;
+		}
+		else
+		{
+			if( ! term->scrollbar.window.is_mapped)
+			{
+				return ;
+			}
+
+			mode = SBM_NONE ;
+		}
+
+		change_sb_mode_intern( term , mode , 1) ;
+	}
+}
+
+
 static void
 delete_term(
 	struct terminal *  term
@@ -1266,7 +1463,9 @@ x_layout_new(
 	char *  view_name ,
 	char *  fg_color ,
 	char *  bg_color ,
-	x_sb_mode_t  mode
+	x_sb_mode_t  mode ,
+	u_int  hmargin ,
+	u_int  vmargin
 	)
 {
 	x_layout_t *  layout ;
@@ -1329,6 +1528,7 @@ x_layout_new(
 	layout->term.screen_scroll_listener.sb_mode = sb_mode ;
 	layout->term.screen_scroll_listener.change_sb_mode = change_sb_mode ;
 	layout->term.screen_scroll_listener.term_changed = term_changed ;
+	layout->term.screen_scroll_listener.screen_color_changed = screen_color_changed ;
 
 	x_set_screen_scroll_listener( screen ,
 		&layout->term.screen_scroll_listener) ;
@@ -1336,7 +1536,17 @@ x_layout_new(
 	layout->line_scrolled_out = screen->screen_listener.line_scrolled_out ;
 	screen->screen_listener.line_scrolled_out = line_scrolled_out ;
 
-	layout->term.sb_mode = mode ;
+	if( mode == SBM_AUTOHIDE)
+	{
+		layout->term.sb_mode = SBM_NONE ;
+		layout->term.autohide_scrollbar = 1 ;
+	}
+	else
+	{
+		layout->term.sb_mode = mode ;
+	}
+	layout->pointer_motion = screen->window.pointer_motion ;
+	screen->window.pointer_motion = screen_pointer_motion ;
 
 	if( layout->term.sb_mode == SBM_NONE)
 	{
@@ -1354,7 +1564,7 @@ x_layout_new(
 
 	if( x_window_init( &layout->window ,
 		actual_width , ACTUAL_HEIGHT( &screen->window) ,
-		min_width , 0 , 0 , 0 , 0 , 0 , 0 , 0) == 0)
+		min_width , 0 , 0 , 0 , hmargin , vmargin , 0 , 0) == 0)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_window_init() failed.\n") ;
@@ -1391,7 +1601,7 @@ x_layout_new(
 	/*
 	 * event call backs.
 	 */
-	x_window_init_event_mask( &layout->window , KeyPressMask) ;
+	x_window_add_event_mask( &layout->window , KeyPressMask) ;
 	layout->window.window_finalized = window_finalized ;
 	layout->window.window_resized = window_resized ;
 	layout->window.child_window_resized = child_window_resized ;
@@ -1403,6 +1613,15 @@ x_layout_new(
 #ifndef  DISABLE_XDND
 	layout->window.set_xdnd_config = set_xdnd_config ;
 #endif
+
+	if( layout->term.autohide_scrollbar)
+	{
+	#ifndef  USE_QUARTZ
+		layout->window.idling = window_idling ;
+	#endif
+		x_window_add_event_mask( &screen->window , PointerMotionMask) ;
+	}
+
 	return  layout ;
 
 error:
@@ -1598,9 +1817,15 @@ x_layout_add_child(
 	}
 
 	next->sb_mode = term->sb_mode ;
+	if( ( next->autohide_scrollbar = term->autohide_scrollbar))
+	{
+		x_window_add_event_mask( &screen->window , PointerMotionMask) ;
+	}
+	screen->window.pointer_motion = screen_pointer_motion ;
 
-	reset_layout( &layout->term , 0 , 0 , ACTUAL_WIDTH(&layout->window) ,
-		ACTUAL_HEIGHT(&layout->window)) ;
+
+	reset_layout( &layout->term , 0 , 0 , layout->window.width ,
+		layout->window.height) ;
 
 	if( term->sb_mode != SBM_NONE)
 	{
@@ -1608,6 +1833,8 @@ x_layout_add_child(
 	}
 	x_window_map( &screen->window) ;
 
+	x_window_set_input_focus( &term->screen->window) ;
+
 	update_normal_hints( layout) ;
 
 	return  1 ;
@@ -1621,7 +1848,7 @@ x_layout_remove_child(
 {
 	struct terminal *  term ;
 	int  idx2 ;
-#ifndef  USE_FRAMEBUFFER
+#ifndef  MANAGE_WINDOWS_BY_MYSELF
 	u_int  w_surplus ;
 	u_int  h_surplus ;
 #endif
@@ -1694,7 +1921,9 @@ x_layout_remove_child(
 		x_set_screen_scroll_listener( term->screen , &term->screen_scroll_listener) ;
 		term->sb_listener.self = term ;
 		term->scrollbar.sb_listener = &term->sb_listener ;
+	#ifndef  USE_QUARTZ
 		term->scrollbar.view->win = &term->scrollbar.window ;
+	#endif
 
 		term = next ;
 	}
@@ -1774,7 +2003,7 @@ x_layout_remove_child(
 		x_window_unset_wall_picture( &layout->window , 0) ;
 	}
 
-#ifndef  USE_FRAMEBUFFER
+#ifndef  MANAGE_WINDOWS_BY_MYSELF
 	if( ! layout->term.next[0] && ! layout->term.next[1])
 	{
 		w_surplus = (layout->window.width -
@@ -1800,13 +2029,13 @@ x_layout_remove_child(
 	else
 #endif
 	{
-		reset_layout( &layout->term , 0 , 0 , ACTUAL_WIDTH(&layout->window) ,
-			ACTUAL_HEIGHT(&layout->window)) ;
+		reset_layout( &layout->term , 0 , 0 , layout->window.width ,
+			layout->window.height) ;
 	}
 
 	update_normal_hints( layout) ;
 
-#ifndef  USE_FRAMEBUFFER
+#ifndef  MANAGE_WINDOWS_BY_MYSELF
 	if( x_screen_attached( screen))
 	{
 		/* Revert to the original size. */
@@ -1822,6 +2051,13 @@ x_layout_remove_child(
 
 	x_window_set_input_focus( &layout->term.screen->window) ;
 
+#ifndef  USE_QUARTZ
+	if( layout->window.idling && ! need_idling_event( &layout->term))
+	{
+		layout->window.idling = NULL ;
+	}
+#endif
+
 	return  1 ;
 }
 
@@ -1931,8 +2167,8 @@ x_layout_resize(
 		term->separator_y += step ;
 	}
 
-	reset_layout( &layout->term , 0 , 0 , ACTUAL_WIDTH(&layout->window) ,
-		ACTUAL_HEIGHT(&layout->window)) ;
+	reset_layout( &layout->term , 0 , 0 , layout->window.width ,
+		layout->window.height) ;
 
 	return  1 ;
 }
