@@ -13,6 +13,44 @@
 #include  "ml_term_manager.h"
 
 
+/* --- static functions --- */
+
+static ml_char_encoding_t
+get_encoding(
+	const char *  value ,
+	int *  is_auto_encoding		/* overwritten only if auto encoding */
+	)
+{
+	ml_char_encoding_t  encoding ;
+
+	if( ! value)
+	{
+		/* goto "auto" */
+	}
+	else if( ( encoding = ml_get_char_encoding( value)) == ML_UNKNOWN_ENCODING)
+	{
+		kik_msg_printf( "Use auto detected encoding instead of %s(unsupported).\n" ,
+			value) ;
+	}
+	else
+	{
+		return  encoding ;
+	}
+
+	if( ( encoding = ml_get_char_encoding( "auto")) == ML_UNKNOWN_ENCODING)
+	{
+		encoding = ML_ISO8859_1 ;
+	}
+
+	if( is_auto_encoding)
+	{
+		*is_auto_encoding = 1 ;
+	}
+
+	return  encoding ;
+}
+
+
 /* --- global functions --- */
 
 int
@@ -60,20 +98,20 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , '%' , "logseq" , 1 , "logging_vt_seq" ,
 		"enable logging vt100 sequence [false]") ;
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	kik_conf_add_opt( conf , '&' , "borderless" , 1 , "borderless" ,
 		"override redirect [false]") ;
 	kik_conf_add_opt( conf , '*' , "type" , 0 , "type_engine" ,
 		"type engine "
-	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
-		"[xft]"
-	#elif  ! defined(USE_TYPE_XCORE) && defined(USE_TYPE_CAIRO)
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 		"[cairo]"
+	#elif  ! defined(USE_TYPE_XCORE) && defined(USE_TYPE_XFT)
+		"[xft]"
 	#else
 		"[xcore]"
 	#endif
 		) ;
-#endif	/* USE_WIN32GUI/USE_FRAMEBUFFER */
+#endif
 
 	kik_conf_add_opt( conf , '1' , "wscr" , 0 , "screen_width_ratio" ,
 		"screen width in percent against font width [100]") ;
@@ -103,7 +141,7 @@ x_prepare_for_main_config(
 #endif
 	kik_conf_add_opt( conf , 'B' , "sbbg" , 0 , "sb_bg_color" , 
 		"scrollbar background color") ;
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND) || defined(USE_OT_LAYOUT)
 	kik_conf_add_opt( conf , 'C' , "ctl" , 1 , "use_ctl" ,
 		"use complex text layouting [true]") ;
 #endif
@@ -117,7 +155,7 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , 'H' , "bright" , 0 , "brightness" ,
 		"brightness of background image in percent [100]") ;
 #endif
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	kik_conf_add_opt( conf , 'I' , "icon" , 0 , "icon_name" , 
 		"icon name") ;
 #endif
@@ -155,7 +193,7 @@ x_prepare_for_main_config(
 		"columns for Unicode \"EastAsianAmbiguous\" character [1]") ;
 	kik_conf_add_opt( conf , 'b' , "bg" , 0 , "bg_color" , 
 		"background color") ;
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#if  defined(USE_XLIB) || defined(USE_CONSOLE)
 	kik_conf_add_opt( conf , 'd' , "display" , 0 , "display" , 
 		"X server to connect") ;
 #endif
@@ -225,6 +263,14 @@ x_prepare_for_main_config(
 		"Color to use to display crossed-out characters") ;
 	kik_conf_add_opt( conf , '\0' , "noul" , 1 , "hide_underline" ,
 		"Don't draw underline [false]") ;
+#ifdef  USE_OT_LAYOUT
+	kik_conf_add_opt( conf , '\0' , "otl" , 1 , "use_ot_layout" ,
+		"OpenType shape [false]") ;
+	kik_conf_add_opt( conf , '\0' , "ost" , 0 , "ot_script" ,
+		"Script of glyph subsutitution [latn]") ;
+	kik_conf_add_opt( conf , '\0' , "oft" , 0 , "ot_features" ,
+		"Features of glyph subsutitution [liga,clig,dlig,hlig,rlig]") ;
+#endif
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	kik_conf_add_opt( conf , '\0' , "servlist" , 0 , "server_list" ,
 		"list of servers to connect") ;
@@ -264,6 +310,8 @@ x_prepare_for_main_config(
 		"blink cursor [false]") ;
 	kik_conf_add_opt( conf , '\0' , "border" , 0 , "inner_border" ,
 		"inner border [2]") ;
+	kik_conf_add_opt( conf , '\0' , "lborder" , 0 , "layout_inner_border" ,
+		"inner border of layout manager [0]") ;
 	kik_conf_add_opt( conf , '\0' , "restart" , 1 , "auto_restart" ,
 		"restart mlterm automatically if an error like segv happens. [true]") ;
 	kik_conf_add_opt( conf , '\0' , "logmsg" , 1 , "logging_msg" ,
@@ -297,6 +345,22 @@ x_prepare_for_main_config(
 		"working directory") ;
 	kik_conf_add_opt( conf , '\0' , "seqfmt" , 0 , "vt_seq_format" ,
 		"format of logging vt100 sequence. [raw]") ;
+	kik_conf_add_opt( conf , '\0' , "uriword" , 1 , "regard_uri_as_word" ,
+		"select uri by double-clicking it [false]") ;
+	kik_conf_add_opt( conf , '\0' , "vtcolor" , 0 , "vt_color_mode" ,
+		"vt color mode [high]") ;
+	kik_conf_add_opt( conf , '\0' , "da1" , 0 , "primary_da" ,
+		"primary device atttributes string "
+	#ifndef  NO_IMAGE
+		"[63;1;2;3;4;7;29]"
+	#else
+		"[63;1;2;7;29]"
+	#endif
+		) ;
+	kik_conf_add_opt( conf , '\0' , "da2" , 0 , "secondary_da" ,
+		"secondary device atttributes string [24;279;0]") ;
+	kik_conf_add_opt( conf , '\0' , "metaprefix" , 0 , "mod_meta_prefix" ,
+		"prefix characters in pressing meta key if mod_meta_mode = esc") ;
 #ifdef  USE_GRF
 	kik_conf_add_opt( conf , '\0' , "multivram" , 1 , "separate_wall_picture" ,
 		"draw wall picture on another vram. (available on 4bpp) [true]") ;
@@ -305,6 +369,14 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , '\0' , "rotate" , 0 , "rotate_display" ,
 		"rotate display. [none]") ;
 #endif
+#ifdef  USE_CONSOLE
+	kik_conf_add_opt( conf , '\0' , "ckm" , 0 , "console_encoding" ,
+		"character encoding of console [none]") ;
+	kik_conf_add_opt( conf , '\0' , "csc" , 0 , "console_sixel_colors" ,
+		"the number of sixel colors of console [16]") ;
+	kik_conf_add_opt( conf , '\0' , "csz" , 0 , "default_cell_size" ,
+		"default cell size [8,16]") ;
+#endif
 #ifdef  USE_IM_CURSOR_COLOR
 	kik_conf_add_opt( conf , '\0' , "imcolor" , 0 , "im_cursor_color" ,
 		"cursor color when input method is activated. [false]") ;
@@ -333,7 +405,11 @@ x_main_config_init(
 		kik_locale_init( value) ;
 	}
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+	/*
+	 * xlib: "-d :0.0" etc.
+	 * console: "-d client:fd"
+	 */
+#if  defined(USE_XLIB) || defined(USE_CONSOLE)
 	if( ( value = kik_conf_get_value( conf , "display")) == NULL)
 #endif
 	{
@@ -357,21 +433,6 @@ x_main_config_init(
 		main_config->font_size = 16 ;
 	}
 
-	if( main_config->font_size > x_get_max_font_size())
-	{
-		kik_msg_printf( "font size %d is too large. %d is used.\n" ,
-			main_config->font_size , x_get_max_font_size()) ;
-		
-		main_config->font_size = x_get_max_font_size() ;
-	}
-	else if( main_config->font_size < x_get_min_font_size())
-	{
-		kik_msg_printf( "font size %d is too small. %d is used.\n" ,
-			main_config->font_size , x_get_min_font_size()) ;
-			
-		main_config->font_size = x_get_min_font_size() ;
-	}
-
 	if( ( value = kik_conf_get_value( conf , "app_name")))
 	{
 		main_config->app_name = strdup( value) ;
@@ -382,7 +443,7 @@ x_main_config_init(
 		main_config->title = strdup( value) ;
 	}
 
-#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
+#ifdef  USE_XLIB
 	if( ( value = kik_conf_get_value( conf , "icon_name")))
 	{
 		main_config->icon_name = strdup( value) ;
@@ -481,18 +542,20 @@ x_main_config_init(
 		}
 	}
 
-#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
-	main_config->type_engine = TYPE_XFT ;
-#elif   defined(USE_TYPE_XCORE)
-	main_config->type_engine = TYPE_XCORE ;
-#else
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	main_config->type_engine = TYPE_CAIRO ;
+#elif  ! defined(USE_TYPE_XCORE) && defined(USE_TYPE_XFT)
+	main_config->type_engine = TYPE_XFT ;
+#else
+	main_config->type_engine = TYPE_XCORE ;
 #endif
 
+#ifdef  USE_XLIB
 	if( ( value = kik_conf_get_value( conf , "type_engine")))
 	{
 		main_config->type_engine = x_get_type_engine_by_name( value) ;
 	}
+#endif
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( ( value = kik_conf_get_value( conf , "use_anti_alias")))
@@ -503,10 +566,10 @@ x_main_config_init(
 			if( main_config->type_engine == TYPE_XCORE)
 			{
 				/* forcibly use xft or cairo */
-			#if  ! defined(USE_TYPE_XFT) && defined(USE_TYPE_CAIRO)
-				main_config->type_engine = TYPE_CAIRO ;
-			#else
+			#if  defined(USE_TYPE_XFT) && ! defined(USE_TYPE_CAIRO)
 				main_config->type_engine = TYPE_XFT ;
+			#else
+				main_config->type_engine = TYPE_CAIRO ;
 			#endif
 			}
 		}
@@ -604,7 +667,7 @@ x_main_config_init(
 		main_config->term_type = strdup( "xterm") ;
 	}
 
-#ifdef  USE_FRAMEBUFFER
+#if  defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
 	/*
 	 * The pty is always resized to fit the display size.
 	 * Don't use 80x24 for the default value because the screen is not drawn
@@ -744,9 +807,13 @@ x_main_config_init(
 			kik_msg_printf( invalid_msg , "tab size" , value) ;
 		}
 	}
-	
+
+#ifdef  USE_QUARTZ
+	main_config->use_login_shell = 1 ;
+#else
 	main_config->use_login_shell = 0 ;
-	
+#endif
+
 	if( ( value = kik_conf_get_value( conf , "use_login_shell")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -765,11 +832,19 @@ x_main_config_init(
 
 	main_config->use_mdi = 1 ;
 
+#ifndef  USE_QUARTZ
 	if( ( value = kik_conf_get_value( conf , "use_mdi")))
 	{
-		main_config->use_mdi = (strcmp( value , "true") == 0) ;
+		if( strcmp( value , "false") == 0)
+		{
+			main_config->use_mdi = 0 ;
+		}
 	}
+#endif
 
+#ifdef  USE_CONSOLE
+	main_config->sb_mode = SBM_NONE ;
+#else
 	if( ( value = kik_conf_get_value( conf , "scrollbar_mode")))
 	{
 		main_config->sb_mode = x_get_sb_mode_by_name( value) ;
@@ -789,6 +864,7 @@ x_main_config_init(
 			main_config->sb_mode = SBM_LEFT ;
 		}
 	}
+#endif
 
 	if( ( value = kik_conf_get_value( conf , "iso88591_font_for_usascii")))
 	{
@@ -835,7 +911,7 @@ x_main_config_init(
 			}
 		}
 	}
-#ifdef  __ANDROID__
+#if  defined(__ANDROID__) || defined(USE_QUARTZ)
 	else
 	{
 		main_config->unicode_policy = ONLY_USE_UNICODE_FONT ;
@@ -853,13 +929,16 @@ x_main_config_init(
 			main_config->unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT ;
 		}
 	}
-#ifdef  __ANDROID__
+#if  defined(__ANDROID__) || defined(USE_QUARTZ)
 	else
 	{
 		main_config->unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT ;
 	}
 #endif
 
+#ifdef  USE_QUARTZ
+	main_config->receive_string_via_ucs = 1 ;
+#else
 	if( ( value = kik_conf_get_value( conf , "receive_string_via_ucs")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -867,6 +946,7 @@ x_main_config_init(
 			main_config->receive_string_via_ucs = 1 ;
 		}
 	}
+#endif
 
 	/* "cn" and "ko" ? */
 	if( strcmp( kik_get_lang() , "ja") == 0)
@@ -997,41 +1077,41 @@ x_main_config_init(
 		}
 	}
 
-	if( ( value = kik_conf_get_value( conf , "encoding")))
+	main_config->encoding = get_encoding( kik_conf_get_value( conf , "encoding") ,
+					&main_config->is_auto_encoding) ;
+
+#ifdef  USE_CONSOLE
+	x_display_set_char_encoding( NULL ,
+		get_encoding( kik_conf_get_value( conf , "console_encoding") , NULL)) ;
+
+	if( ( value = kik_conf_get_value( conf , "console_sixel_colors")))
 	{
-		while( ( main_config->encoding = ml_get_char_encoding( value))
-			== ML_UNKNOWN_ENCODING)
-		{
-			kik_msg_printf(
-				"%s encoding is not supported. Auto detected encoding is used.\n" ,
-				value) ;
+		x_display_set_sixel_colors( NULL , value) ;
+	}
 
-			value = "auto" ;
-		}
+	if( ( value = kik_conf_get_value( conf , "cell_size")))
+	{
+		u_int  width ;
+		u_int  height ;
 
-		if( strcmp( value , "auto") == 0)
+		if( sscanf( value , "%d,%d" , &width , &height) == 2)
 		{
-			main_config->is_auto_encoding = 1 ;
+			x_display_set_default_cell_size( width , height) ;
 		}
 	}
-	else
-	{
-		main_config->encoding = ml_get_char_encoding( "auto") ;
-		main_config->is_auto_encoding = 1 ;
-	}
+#endif
 
-	if( main_config->encoding == ML_UNKNOWN_ENCODING)
-	{
-		main_config->encoding = ML_ISO8859_1 ;
-	}
-
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND) || defined(USE_OT_LAYOUT)
 	main_config->use_ctl = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_ctl")))
 	{
 		if( strcmp( value , "false") == 0)
 		{
+			/*
+			 * If use_ot_layout is true, use_ctl = true forcibly.
+			 * See processing "use_ot_layout" option.
+			 */
 			main_config->use_ctl = 0 ;
 		}
 	}
@@ -1113,7 +1193,8 @@ x_main_config_init(
 			main_config->use_extended_scroll_shortcut = 1 ;
 		}
 	}
-	
+
+#ifdef  USE_XLIB
 	if( ( value = kik_conf_get_value( conf , "borderless")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1121,6 +1202,7 @@ x_main_config_init(
 			main_config->borderless = 1 ;
 		}
 	}
+#endif
 
 	main_config->bs_mode = BSM_DEFAULT ;
 
@@ -1337,6 +1419,28 @@ x_main_config_init(
 		}
 	}
 
+	main_config->layout_hmargin = main_config->layout_vmargin = 0 ;
+
+	if( ( value = kik_conf_get_value( conf , "layout_inner_border")))
+	{
+		u_int  hmargin ;
+		u_int  vmargin ;
+
+		/* 640x480 => (640-224*2)x(480-224*2) => 192x32 on framebuffer. */
+		if( sscanf( value , "%d,%d" , &hmargin , &vmargin) == 2)
+		{
+			if( hmargin <= 224 && vmargin <= 224)
+			{
+				main_config->layout_hmargin = hmargin ;
+				main_config->layout_vmargin = vmargin ;
+			}
+		}
+		else if( kik_str_to_uint( &hmargin , value) && hmargin <= 224)
+		{
+			main_config->layout_hmargin = main_config->layout_vmargin = hmargin ;
+		}
+	}
+
 	main_config->use_bold_font = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_bold_font")))
@@ -1370,12 +1474,14 @@ x_main_config_init(
 		ml_set_word_separators( value) ;
 	}
 
+	if( ( value = kik_conf_get_value( conf , "regard_uri_as_word")))
+	{
+		ml_set_regard_uri_as_word( strcmp( value , "true") == 0) ;
+	}
+
 	if( ( value = kik_conf_get_value( conf , "use_clipboard")))
 	{
-		if( strcmp( value , "false") == 0)
-		{
-			x_set_use_clipboard_selection( 0) ;
-		}
+		x_set_use_clipboard_selection( strcmp( value , "true") == 0) ;
 	}
 
 	if( ! ( value = kik_conf_get_value( conf , "auto_restart")) ||
@@ -1444,6 +1550,49 @@ x_main_config_init(
 		main_config->work_dir = strdup( value) ;
 	}
 
+	if( ( value = kik_conf_get_value( conf , "vt_color_mode")))
+	{
+		ml_set_color_mode( value) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "primary_da")))
+	{
+		ml_set_primary_da( value) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "secondary_da")))
+	{
+		ml_set_secondary_da( value) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "mod_meta_prefix")))
+	{
+		x_set_mod_meta_prefix( kik_str_unescape( value)) ;
+	}
+
+#ifdef  USE_OT_LAYOUT
+	main_config->use_ot_layout = 0 ;
+
+	if( ( value = kik_conf_get_value( conf , "use_ot_layout")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			main_config->use_ot_layout = 1 ;
+			main_config->use_ctl = 1 ;
+		}
+	}
+
+	if( ( value = kik_conf_get_value( conf , "ot_script")))
+	{
+		ml_set_ot_layout_attr( value , OT_SCRIPT) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "ot_features")))
+	{
+		ml_set_ot_layout_attr( value , OT_FEATURES) ;
+	}
+#endif
+
 #ifdef  USE_GRF
 	if( ( value = kik_conf_get_value( conf , "separate_wall_picture")))
 	{
