@@ -71,6 +71,7 @@ typedef struct  x_screen_scroll_event_listener
 	x_sb_mode_t  (*sb_mode)( void *) ;
 	void  (*change_sb_mode)( void * , x_sb_mode_t) ;
 	void  (*term_changed)( void * , u_int , u_int) ;
+	void  (*screen_color_changed)( void *) ;
 
 } x_screen_scroll_event_listener_t ;
 
@@ -160,6 +161,8 @@ void  x_allow_change_shortcut( int  flag) ;
 
 void  x_set_mod_meta_prefix( char *  prefix) ;
 
+#define  x_free_mod_meta_prefix()  x_set_mod_meta_prefix( NULL)
+
 #ifdef  USE_IM_CURSOR_COLOR
 void  x_set_im_cursor_color( char *  color) ;
 #endif
