@@ -59,6 +59,9 @@ static u_int  num_of_startup_screens ;
 static x_system_event_listener_t  system_listener ;
 
 static x_main_config_t  main_config ;
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+static const char *  cur_default_server ;
+#endif
 
 static x_shortcut_t  shortcut ;
 
@@ -137,7 +140,7 @@ create_term_intern(void)
 			main_config.use_dynamic_comb , main_config.bs_mode ,
 			main_config.vertical_mode , main_config.use_local_echo ,
 			main_config.title , main_config.icon_name ,
-			main_config.alt_color_mode)) == NULL)
+			main_config.alt_color_mode , main_config.use_ot_layout)) == NULL)
 	{
 		return  NULL ;
 	}
@@ -233,7 +236,10 @@ open_pty_intern(
 			pass = strdup( "") ;
 			exec_cmd = NULL ;
 
-			ml_pty_ssh_set_use_x11_forwarding( session , x11_fwd) ;
+			if( x11_fwd)
+			{
+				ml_pty_ssh_set_use_x11_forwarding( session , x11_fwd) ;
+			}
 		}
 		else
 	#endif
@@ -325,19 +331,33 @@ open_pty_intern(
 	 */
 	if( ! uri && ! cmd_path)
 	{
-		/*
-		 * SHELL env var -> /etc/passwd -> /bin/sh
-		 */
-		if( ( cmd_path = getenv( "SHELL")) == NULL || *cmd_path == '\0')
-		{
-		#ifndef  USE_WIN32API
-			struct passwd *  pw ;
+	#ifdef  USE_QUARTZ
+		char *  user ;
 
-			if( ( pw = getpwuid(getuid())) == NULL ||
-				*( cmd_path = pw->pw_shell) == '\0')
-		#endif
+		if( ( user = getenv("USER")) && ( cmd_argv = alloca( sizeof(char*) * 4)))
+		{
+			cmd_argv[0] = cmd_path = "login" ;
+			cmd_argv[1] = "-fp" ;
+			cmd_argv[2] = user ;
+			cmd_argv[3] = NULL ;
+		}
+		else
+	#endif
+		{
+			/*
+			 * SHELL env var -> /etc/passwd -> /bin/sh
+			 */
+			if( ( cmd_path = getenv( "SHELL")) == NULL || *cmd_path == '\0')
 			{
-				cmd_path = "/bin/sh" ;
+			#ifndef  USE_WIN32API
+				struct passwd *  pw ;
+
+				if( ( pw = getpwuid(getuid())) == NULL ||
+					*( cmd_path = pw->pw_shell) == '\0')
+			#endif
+				{
+					cmd_path = "/bin/sh" ;
+				}
 			}
 		}
 	}
@@ -389,6 +409,10 @@ open_pty_intern(
 		if( ret && kik_compare_str( uri , main_config.default_server) != 0)
 		{
 			x_main_config_add_to_server_list( &main_config , uri) ;
+			if( cur_default_server == main_config.default_server)
+			{
+				cur_default_server = uri ;
+			}
 			free( main_config.default_server) ;
 			main_config.default_server = uri ;
 		}
@@ -578,7 +602,11 @@ open_screen_intern(
 		}
 	}
 
-	if( ( disp = x_display_open( main_config.disp_name , depth)) == NULL)
+	if( layout)
+	{
+		disp = layout->window.disp ;
+	}
+	else if( ( disp = x_display_open( main_config.disp_name , depth)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_display_open failed.\n") ;
@@ -676,7 +704,8 @@ open_screen_intern(
 		    ( layout = x_layout_new( screen ,
 					main_config.scrollbar_view_name ,
 					main_config.sb_fg_color , main_config.sb_bg_color ,
-					main_config.sb_mode)))
+					main_config.sb_mode ,
+					main_config.layout_hmargin , main_config.layout_vmargin)))
 		{
 			root = &layout->window ;
 		}
@@ -734,7 +763,7 @@ open_screen_intern(
 	else
 	{
 		if( ! open_pty_intern( term , main_config.cmd_path , main_config.cmd_argv ,
-			DisplayString( disp->display) , root->my_window ,
+			DisplayString( disp->display) , screen->window.my_window ,
 			screen->window.width , screen->window.height))
 		{
 			x_screen_detach( screen) ;
@@ -884,13 +913,18 @@ open_pty(
 		main_config.encoding = ml_term_get_encoding( screen->term) ;
 	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 		default_server = main_config.default_server ;
-		main_config.default_server = ml_term_get_uri( screen->term) ;
-		/*
-		 * If show_dialog == 1, main_config.default_server can be
-		 * free'ed in open_pty_intern.
-		 */
 		show_dialog = main_config.show_dialog ;
-		main_config.show_dialog = 0 ;
+
+		if( ! main_config.default_server ||
+		    kik_compare_str( main_config.default_server , cur_default_server) == 0)
+		{
+			main_config.default_server = ml_term_get_uri( screen->term) ;
+			/*
+			 * If show_dialog == 1, main_config.default_server can be
+			 * free'ed in open_pty_intern.
+			 */
+			main_config.show_dialog = 0 ;
+		}
 
 		if( ( new_cmd_line = ml_term_get_cmd_line( screen->term)) &&
 		    ( new_cmd_line = kik_str_alloca_dup( new_cmd_line)))
@@ -908,7 +942,7 @@ open_pty(
 		ret = open_pty_intern( new , main_config.cmd_path ,
 			main_config.cmd_argv ,
 			DisplayString( screen->window.disp->display) ,
-			x_get_root_window( &screen->window)->my_window ,
+			screen->window.my_window ,
 			screen->window.width , screen->window.height) ;
 
 		main_config.encoding = encoding ;
@@ -1260,7 +1294,7 @@ mlclient(
 #endif
 
 	if( argc == 0
-	#ifdef  USE_FRAMEBUFFER
+	#if  defined(USE_FRAMEBUFFER)
 	    || screen == NULL
 	#endif
 	    )
@@ -1306,7 +1340,7 @@ mlclient(
 		char **  server_list ;
 	#endif
 		
-		if( argc >= 2 && *argv[1] != '-')
+		if( argc >= 2 && *(argv[1]) != '-')
 		{
 			/*
 			 * mlclient [dev] [options...]
@@ -1395,7 +1429,10 @@ x_screen_manager_init(
 	depth = _depth ;
 
 	main_config = *_main_config ;
-	
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+	cur_default_server = main_config.default_server ;
+#endif
+
 	max_screens_multiple = _max_screens_multiple ;
 
 	if( ( dead_mask = calloc( sizeof( *dead_mask) , max_screens_multiple)) == NULL)
@@ -1472,7 +1509,7 @@ x_screen_manager_init(
 #else
 	system_listener.exit = NULL ;
 #endif
-#ifdef  USE_FRAMEBUFFER
+#if  defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
 	system_listener.open_screen = NULL ;
 #else
 	system_listener.open_screen = open_screen ;
