@@ -948,7 +948,7 @@ view_created:
 	/*
 	 * event callbacks.
 	 */
-	x_window_init_event_mask( &sb->window , ButtonPressMask | ButtonReleaseMask | ButtonMotionMask) ;
+	x_window_add_event_mask( &sb->window , ButtonPressMask | ButtonReleaseMask | ButtonMotionMask) ;
 	sb->window.window_realized = window_realized ;
 	sb->window.button_pressed = button_pressed ;
 	sb->window.button_released = button_released ;
@@ -986,7 +986,7 @@ x_scrollbar_final(
 	free( sb->fg_color) ;
 	free( sb->bg_color) ;
 	free( sb->view_name) ;
-	
+
 	return  1 ;
 }
 
@@ -1161,6 +1161,11 @@ x_scrollbar_move(
 		row = -(sb->num_of_filled_log_lines) ;
 	}
 
+	if( sb->current_row == row)
+	{
+		return  0 ;
+	}
+
 	sb->current_row = row ;
 
 	set_redraw_area( sb, sb->bar_top_y, sb->bar_height) ;
