@@ -109,14 +109,25 @@ typedef struct  x_window
 	u_int16_t  hmargin ;
 	u_int16_t  vmargin ;
 
+	/*
+	 * mlterm-con doesn't use these members, but they are necessary to keep
+	 * the size of x_window_t as the same as mlterm-fb for input method plugins.
+	 */
+#if  defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
+	u_int  clip_y ;
+	u_int  clip_height ;
+#endif
+
 	/* used by x_xim */
 	x_xim_ptr_t  xim ;
 	x_xim_event_listener_t *  xim_listener ;
 	x_xic_ptr_t  xic ;	/* Only root window manages xic in win32 */
 
-#ifdef  USE_WIN32GUI
+#if  defined(USE_WIN32GUI)
 	WORD  update_window_flag ;
 	int  cmd_show ;
+#elif  defined(USE_QUARTZ)
+	int  update_window_flag ;
 #endif
 
 	/* button */
@@ -139,18 +150,17 @@ typedef struct  x_window
 	 * XClassHint
 	 */
 	char *  app_name ;
-
 	
 	/*
 	 * flags etc.
 	 */
 
-#if  defined(USE_WIN32GUI) || defined(USE_FRAMEBUFFER)
-	Pixmap  wall_picture ;
-#else
+#ifdef  USE_XLIB
 	int8_t  wall_picture_is_set ;	/* Actually set picture (including transparency) or not. */
 	int8_t  wait_copy_area_response ;	/* Used for XCopyArea() */
 	int8_t  configure_root ;
+#else
+	Pixmap  wall_picture ;
 #endif
 	int8_t  is_sel_owner ;
 	int8_t  is_transparent ;
@@ -206,8 +216,6 @@ int  x_window_final( x_window_t *  win) ;
 
 int  x_window_set_type_engine( x_window_t *  win , x_type_engine_t  type_engine) ;
 
-int  x_window_init_event_mask( x_window_t *  win , long  event_mask) ;
-
 int  x_window_add_event_mask( x_window_t *  win , long  event_mask) ;
 
 int  x_window_remove_event_mask( x_window_t *  win , long  event_mask) ;
@@ -220,10 +228,10 @@ int  x_window_set_wall_picture( x_window_t *  win , Pixmap  pic , int  do_expose
 
 int  x_window_unset_wall_picture( x_window_t *  win , int  do_expose) ;
 
-#if  defined(USE_WIN32GUI) || defined(USE_FRAMEBUFFER)
-#define  x_window_has_wall_picture( win)  ((win)->wall_picture != None)
-#else
+#ifdef  USE_XLIB
 #define  x_window_has_wall_picture( win)  ((win)->wall_picture_is_set)
+#else
+#define  x_window_has_wall_picture( win)  ((win)->wall_picture != None)
 #endif
 
 int  x_window_set_transparent( x_window_t *  win , x_picture_modifier_ptr_t  pic_mod) ;
@@ -294,7 +302,7 @@ int  x_window_receive_event( x_window_t *  win , XEvent *  event) ;
 size_t  x_window_get_str( x_window_t *  win , u_char *  seq , size_t  seq_len ,
 	mkf_parser_t **  parser , KeySym *  keysym , XKeyEvent *  event) ;
 
-#ifdef  USE_FRAMEBUFFER
+#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
 int  x_window_is_scrollable( x_window_t *  win) ;
 #else
 #define  x_window_is_scrollable(win)  ((win)->is_scrollable)
@@ -323,6 +331,10 @@ int  x_window_scroll_rightward( x_window_t *  win , u_int  width) ;
 int  x_window_copy_area( x_window_t *  win , Pixmap  src , PixmapMask  mask ,
 	int  src_x , int  src_y , u_int  width , u_int  height , int  dst_x , int  dst_y) ;
 
+void  x_window_set_clip( x_window_t *  win , int  x , int  y , u_int  width , u_int  height) ;
+
+void  x_window_unset_clip( x_window_t *  win) ;
+
 int  x_window_draw_decsp_string( x_window_t *  win , x_font_t *  font ,
 	x_color_t *  fg_color , int  x , int  y , u_char *  str , u_int  len) ;
 
@@ -351,6 +363,16 @@ int  x_window_draw_image_string16( x_window_t *  win , x_font_t *  font ,
 	XChar2b *  str , u_int  len) ;
 #endif
 
+#ifdef  USE_CONSOLE
+int  x_window_console_draw_string( x_window_t *  win , x_font_t *  font ,
+	x_color_t *  fg_color , x_color_t *  bg_color , int  x , int  y ,
+	u_char *  str , u_int  len , int  underline_style) ;
+
+int  x_window_console_draw_string16( x_window_t *  win , x_font_t *  font ,
+	x_color_t *  fg_color , x_color_t *  bg_color , int  x , int  y ,
+	XChar2b *  str , u_int  len , int  underline_style) ;
+#endif
+
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 int  x_window_ft_draw_string8( x_window_t *  win , x_font_t *  font ,
 	x_color_t *  fg_color , int  x , int  y , u_char *  str , size_t  len) ;
@@ -401,7 +423,7 @@ u_int  x_window_get_mod_ignore_mask( x_window_t *  win , KeySym *  keysyms) ;
 
 u_int  x_window_get_mod_meta_mask( x_window_t *  win , char *  mod_key) ;
 
-#ifdef  USE_FRAMEBUFFER
+#if  defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)
 #define  x_set_use_urgent_bell( use)  (0)
 #else
 int  x_set_use_urgent_bell( int  use) ;
