@@ -14,8 +14,7 @@
 #include  <mkf/mkf_ucs_property.h>
 #include  <ml_char_encoding.h>	/* ml_is_msb_set */
 
-#include  "../x_type_loader.h"
-
+#include  "x_type_loader.h"
 #include  "x_decsp_font.h"
 
 
@@ -87,7 +86,6 @@ static cs_info_t  cs_info_table[] =
 	{ ISCII_MALAYALAM , { NULL , NULL , } , } ,
 	{ ISCII_ORIYA , { NULL , NULL , } , } ,
 	{ ISCII_PUNJABI , { NULL , NULL , } , } ,
-	{ ISCII_ROMAN , { NULL , NULL , } , } ,
 	{ ISCII_TAMIL , { NULL , NULL , } , } ,
 	{ ISCII_TELUGU , { NULL , NULL , } , } ,
 	{ VISCII , { "viscii-1" , NULL , } , } ,
@@ -868,11 +866,60 @@ xft_set_font(
 			aa_opt , use_point_size_for_fc , dpi_for_fc) ;
 }
 
+static int
+xft_set_ot_font(
+	x_font_t *  font
+	)
+{
+	int (*func)( x_font_t *) ;
+
+	if( ! ( func = x_load_type_xft_func( X_SET_OT_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font) ;
+}
+
+static u_int
+xft_convert_text_to_glyphs(
+	x_font_t *  font ,
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+	int (*func)( x_font_t * , u_int32_t * , u_int , int8_t * , u_int8_t * ,
+		u_int32_t * , u_int32_t * , u_int , const char * , const char *) ;
+
+	if( ! ( func = x_load_type_xft_func( X_CONVERT_TEXT_TO_GLYPHS)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , shaped , shaped_len , offsets , widths , cmapped ,
+			src , src_len , script , features) ;
+}
+
 #elif  defined(USE_TYPE_XFT)
 u_int  xft_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
 int  xft_set_font( x_font_t *  font , const char *  fontname , u_int  fontsize ,
 	u_int  col_width , u_int  letter_space , int  aa_opt ,
 	int  use_point_size_for_fc , double  dpi_for_fc) ;
+int  xft_set_ot_font( x_font_t *  font) ;
+#define  xft_convert_text_to_glyphs( font , shaped , shaped_len , offsets , widths , cmapped , \
+		src , src_len , script , features) \
+	ft_convert_text_to_glyphs( font , shaped , shaped_len , offsets , widths , cmapped , \
+		src , src_len , script , features)
+u_int  ft_convert_text_to_glyphs( x_font_t *  font , u_int32_t *  shaped , int8_t * , u_int8_t * ,
+	u_int  shaped_len , u_int32_t *  cmapped , u_int32_t *  src , u_int  src_len ,
+	const char *  script , const char *  features) ;
 #endif
 
 
@@ -917,11 +964,62 @@ cairo_set_font(
 			aa_opt , use_point_size_for_fc , dpi_for_fc) ;
 }
 
+static int
+cairo_set_ot_font(
+	x_font_t *  font
+	)
+{
+	int (*func)( x_font_t *) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_SET_OT_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font) ;
+}
+
+static u_int
+cairo_convert_text_to_glyphs(
+	x_font_t *  font ,
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+	int (*func)( x_font_t * , u_int32_t * , u_int , int8_t * , u_int8_t * , u_int32_t * ,
+		u_int32_t * , u_int , const char * , const char *) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_CONVERT_TEXT_TO_GLYPHS)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , shaped , shaped_len , offsets , widths , cmapped ,
+			src , src_len , script , features) ;
+}
+
 #elif  defined(USE_TYPE_CAIRO)
 u_int  cairo_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
 int  cairo_set_font( x_font_t *  font , const char *  fontname , u_int  fontsize ,
 	u_int  col_width , u_int  letter_space , int  aa_opt , int  use_point_size_for_fc ,
 	double  dpi_for_fc) ;
+int  cairo_set_ot_font( x_font_t *  font) ;
+#define  cairo_convert_text_to_glyphs( font , shaped , shaped_len , offsets , widths , cmapped , \
+		src , src_len , script , features) \
+	ft_convert_text_to_glyphs( font , shaped , shaped_len , offsets , widths , cmapped , \
+		src , src_len , script , features)
+#ifndef  USE_TYPE_XFT
+u_int  ft_convert_text_to_glyphs( x_font_t *  font , u_int32_t *  shaped , int8_t *  offsets ,
+	u_int8_t *  widths , u_int  shaped_len , u_int32_t *  cmapped , u_int32_t *  src ,
+	u_int  src_len , const char *  script , const char *  features) ;
+#endif
 #endif
 
 static u_int
@@ -990,6 +1088,7 @@ x_font_t *
 x_font_new(
 	Display *  display ,
 	ml_font_t  id ,
+	int  size_attr ,
 	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present , /* FONT_VAR_WIDTH is never set if FONT_VERTICAL is set. */
 	const char *  fontname ,
@@ -1001,7 +1100,7 @@ x_font_new(
 {
 	x_font_t *  font ;
 
-	if( ( font = malloc( sizeof( x_font_t))) == NULL)
+	if( ( font = calloc( 1 , sizeof( x_font_t))) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
@@ -1022,34 +1121,29 @@ x_font_new(
 		font->cols = 1 ;
 	}
 
-	if( ( font_present & FONT_VAR_WIDTH) || IS_ISCII(FONT_CS(font->id)))
+	if( font_present & FONT_VAR_WIDTH)
 	{
 		font->is_var_col_width = 1 ;
 	}
-	else
+	else if( IS_ISCII(FONT_CS(font->id)))
 	{
-		font->is_var_col_width = 0 ;
+		/*
+		 * For exampe, 'W' width and 'l' width of OR-TTSarala font for ISCII_ORIYA
+		 * are the same by chance, though it is actually a proportional font.
+		 */
+		font->is_var_col_width = font->is_proportional = 1 ;
 	}
 
 	if( font_present & FONT_VERTICAL)
 	{
 		font->is_vertical = 1 ;
 	}
-	else
-	{
-		font->is_vertical = 0 ;
-	}
 
-#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
-	font->xfont = NULL ;
-#endif
-#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
-	font->xft_font = NULL ;
-#endif
-#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
-	font->cairo_font = NULL ;
-#endif
-	font->decsp_font = NULL ;
+	if( size_attr >= DOUBLE_HEIGHT_TOP)
+	{
+		fontsize *= 2 ;
+		col_width *= 2 ;
+	}
 
 	switch( type_engine)
 	{
@@ -1117,6 +1211,23 @@ x_font_new(
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 end:
 #endif
+	if( size_attr == DOUBLE_WIDTH)
+	{
+		if( type_engine == TYPE_CAIRO)
+		{
+			font->x_off *= 2 ;
+		}
+		else
+		{
+			font->x_off += (font->width / 2) ;
+			font->is_proportional = 1 ;
+			font->is_var_col_width = 0 ;
+		}
+		font->width *= 2 ;
+	}
+
+	font->size_attr = size_attr ;
+
 	if( font->is_proportional && ! font->is_var_col_width)
 	{
 		kik_msg_printf(
@@ -1220,14 +1331,19 @@ x_calculate_char_width(
 			*draw_alone = 1 ;
 		}
 	}
-	else if( draw_alone && cs == ISO10646_UCS4_1)
+	else if( draw_alone &&
+	         cs == ISO10646_UCS4_1 /* ISO10646_UCS4_1_V is always proportional */
+	#ifdef  USE_OT_LAYOUT
+		 && ( ! font->use_ot_layout /* || ! font->ot_font */)
+	#endif
+		 )
 	{
 		if( (( mkf_get_ucs_property( ch) & MKF_AWIDTH) ||
 		    /*
 		     * The width of U+2590 and U+2591 is narrow in EastAsianWidth-6.3.0
 		     * but the glyphs in GNU Unifont are full-width unexpectedly.
 		     */
-		    ch == 0x2590 || ch == 2591))
+		    ch == 0x2590 || ch == 0x2591))
 		{
 			if( calculate_char_width( font , ch , cs) != font->width)
 			{
@@ -1258,10 +1374,83 @@ x_font_get_encoding_names(
 
 void
 x_font_use_point_size(
-	int  bool
+	int  use
 	)
 {
-	use_point_size_for_fc = bool ;
+	use_point_size_for_fc = use ;
+}
+
+int
+x_font_has_ot_layout_table(
+	x_font_t *  font
+	)
+{
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( font->cairo_font)
+	{
+	#ifdef  USE_OT_LAYOUT
+		if( ! font->ot_font)
+		{
+			if( font->ot_font_not_found || ! cairo_set_ot_font( font))
+			{
+				return  0 ;
+			}
+		}
+	#endif
+
+		return  1 ;
+	}
+#endif
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+	if( font->xft_font)
+	{
+	#ifdef  USE_OT_LAYOUT
+		if( ! font->ot_font)
+		{
+			if( font->ot_font_not_found || ! xft_set_ot_font( font))
+			{
+				return  0 ;
+			}
+		}
+	#endif
+
+		return  1 ;
+	}
+#endif
+
+	return  0 ;
+}
+
+u_int
+x_convert_text_to_glyphs(
+	x_font_t *  font ,	/* always has ot_font */
+	u_int32_t *  shaped ,
+	u_int  shaped_len ,
+	int8_t *  offsets ,
+	u_int8_t *  widths ,
+	u_int32_t *  cmapped ,
+	u_int32_t *  src ,
+	u_int  src_len ,
+	const char *  script ,
+	const char *  features
+	)
+{
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( font->cairo_font)
+	{
+		return  cairo_convert_text_to_glyphs( font , shaped , shaped_len , offsets ,
+				widths , cmapped , src , src_len , script , features) ;
+	}
+#endif
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+	if( font->xft_font)
+	{
+		return  xft_convert_text_to_glyphs( font , shaped , shaped_len , offsets ,
+				widths , cmapped , src , src_len , script , features) ;
+	}
+#endif
+
+	return  0 ;
 }
 
 /* For mlterm-libvte */
@@ -1283,7 +1472,7 @@ convert_to_ucs4(
 	mkf_charset_t  cs
 	)
 {
-	if( cs == ISO10646_UCS4_1 /* || cs == ISO10646_UCS2_1 */)
+	if( IS_ISO10646_UCS4(cs) /* || cs == ISO10646_UCS2_1 */)
 	{
 		return  ch ;
 	}
@@ -1316,8 +1505,6 @@ convert_to_ucs4(
 			return  0 ;
 		}
 	}
-
-	return  1 ;
 }
 
 int
