@@ -24,7 +24,7 @@
 #ifndef  DISABLE_XDND
 #include  "../x_dnd.h"
 #endif
-#include  "../x_type_loader.h"
+#include  "x_type_loader.h"
 
 #include  "x_display.h"		/* x_display_get_cursor */
 #include  "x_decsp_font.h"
@@ -924,6 +924,40 @@ x_window_xft_draw_string32(
 	return  (*func)( win , font , fg_color , x , y , str , len) ;
 }
 
+static void
+xft_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	void (*func)( x_window_t * , int , int , u_int , u_int) ;
+
+	if( ! ( func = x_load_type_xft_func( X_WINDOW_SET_CLIP)))
+	{
+		return ;
+	}
+
+	(*func)( win , x , y , width , height) ;
+}
+
+static void
+xft_unset_clip(
+	x_window_t *  win
+	)
+{
+	void (*func)( x_window_t *) ;
+
+	if( ! ( func = x_load_type_xft_func( X_WINDOW_UNSET_CLIP)))
+	{
+		return ;
+	}
+
+	(*func)( win) ;
+}
+
 static int
 x_window_set_use_cairo(
 	x_window_t *  win ,
@@ -998,6 +1032,40 @@ cairo_resize(
 	return  (*func)( win) ;
 }
 
+static void
+cairo_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+	void (*func)( x_window_t * , int , int , u_int , u_int) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_SET_CLIP)))
+	{
+		return ;
+	}
+
+	(*func)( win , x , y , width , height) ;
+}
+
+static void
+cairo_unset_clip(
+	x_window_t *  win
+	)
+{
+	void (*func)( x_window_t *) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_UNSET_CLIP)))
+	{
+		return ;
+	}
+
+	(*func)( win) ;
+}
+
 #else  /* NO_DYNAMIC_LOAD_TYPE */
 #ifdef  USE_TYPE_XFT
 int  x_window_set_use_xft( x_window_t *  win , int  use_xft) ;
@@ -1005,6 +1073,8 @@ int  x_window_xft_draw_string8( x_window_t *  win , x_font_t *  font , x_color_t
 	int  x , int  y , u_char *  str , size_t  len) ;
 int  x_window_xft_draw_string32( x_window_t *  win , x_font_t *  font , x_color_t *  fg_color ,
 	int  x , int  y , u_int32_t *  str , size_t  len) ;
+void  xft_set_clip( x_window_t *  win , int  x , int  y , u_int  width , u_int  height) ;
+void  xft_unset_clip( x_window_t *  win) ;
 #endif
 #ifdef  USE_TYPE_CAIRO
 int  x_window_set_use_cairo( x_window_t *  win , int  use_cairo) ;
@@ -1013,6 +1083,8 @@ int  x_window_cairo_draw_string8( x_window_t *  win , x_font_t *  font , x_color
 int  x_window_cairo_draw_string32( x_window_t *  win , x_font_t *  font , x_color_t *  fg_color ,
 	int  x , int  y , u_int32_t *  str , size_t  len) ;
 int  cairo_resize( x_window_t *  win) ;
+void  cairo_set_clip( x_window_t *  win , int  x , int  y , u_int  width , u_int  height) ;
+void  cairo_unset_clip( x_window_t *  win) ;
 #endif
 #endif /* NO_DYNAMIC_LOAD_TYPE */
 
@@ -1153,36 +1225,6 @@ x_window_set_type_engine(
 	return  1 ;
 }
 
-int
-x_window_init_event_mask(
-	x_window_t *  win ,
-	long  event_mask
-	)
-{
-	if( win->my_window)
-	{
-		/*
-		 * Don't use this function after x_window_show().
-		 * After x_window_show(), use x_window_{add|remove}_event_mask().
-		 */
-
-		return  0 ;
-	}
-
-#if  0
-	if( event_mask & ButtonMotionMask)
-	{
-		event_mask &= ~ButtonMotionMask ;
-		event_mask |= ( Button1MotionMask | Button2MotionMask |
-			Button3MotionMask | Button4MotionMask | Button5MotionMask) ;
-	}
-#endif
-
-	win->event_mask |= event_mask ;
-
-	return  1 ;
-}
-
 int
 x_window_add_event_mask(
 	x_window_t *  win ,
@@ -1200,7 +1242,10 @@ x_window_add_event_mask(
 
 	win->event_mask |= event_mask ;
 
-	XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
+	if( win->my_window)
+	{
+		XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
+	}
 
 	return  1 ;
 }
@@ -1222,7 +1267,10 @@ x_window_remove_event_mask(
 
 	win->event_mask &= ~event_mask ;
 
-	XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
+	if( win->my_window)
+	{
+		XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
+	}
 
 	return  1 ;
 }
@@ -1517,8 +1565,8 @@ x_window_add_child(
 	win->children = p ;
 
 	child->parent = win ;
-	child->x = x ;
-	child->y = y ;
+	child->x = x + win->hmargin ;
+	child->y = y + win->vmargin ;
 
 	if( ! ( child->is_mapped = map) && child->inputtable > 0)
 	{
@@ -1696,8 +1744,8 @@ x_window_show(
 
 		size_hints.width_inc = total_width_inc( win) ;
 		size_hints.height_inc = total_height_inc( win) ;
-		size_hints.min_width = total_min_width( win) + size_hints.width_inc ;
-		size_hints.min_height = total_min_height( win) + size_hints.height_inc ;
+		size_hints.min_width = total_min_width( win) ;
+		size_hints.min_height = total_min_height( win) ;
 		size_hints.base_width = size_hints.min_width > size_hints.width_inc ?
 					size_hints.min_width - size_hints.width_inc : 0 ;
 		size_hints.base_height = size_hints.min_height > size_hints.height_inc ?
@@ -1971,10 +2019,12 @@ x_window_set_normal_hints(
 	 */
 	size_hints.width_inc = total_width_inc( root) ;
 	size_hints.height_inc = total_height_inc( root) ;
-	size_hints.min_width = total_min_width( root) + size_hints.width_inc ;
-	size_hints.min_height = total_min_height( root) + size_hints.height_inc ;
-	size_hints.base_width = size_hints.min_width - size_hints.width_inc ;
-	size_hints.base_height = size_hints.min_height - size_hints.height_inc ;
+	size_hints.min_width = total_min_width( root) ;
+	size_hints.min_height = total_min_height( root) ;
+	size_hints.base_width = size_hints.min_width > size_hints.width_inc ?
+				size_hints.min_width - size_hints.width_inc : 0 ;
+	size_hints.base_height = size_hints.min_height > size_hints.height_inc ?
+				size_hints.min_height - size_hints.height_inc : 0 ;
 	size_hints.flags = PMinSize | PResizeInc | PBaseSize ;
 
 #ifdef  DEBUG
@@ -2087,6 +2137,12 @@ x_window_move(
 	int  y
 	)
 {
+	if( win->parent)
+	{
+		x += win->parent->hmargin ;
+		y += win->parent->vmargin ;
+	}
+
 	if( win->x == x && win->y == y)
 	{
 		return  0 ;
@@ -2095,7 +2151,7 @@ x_window_move(
 	win->x = x ;
 	win->y = y ;
 
-	XMoveWindow( win->disp->display , win->my_window , x , y) ;
+	XMoveWindow( win->disp->display , win->my_window , win->x , win->y) ;
 
 	if( ! win->configure_root && win->parent)
 	{
@@ -3059,7 +3115,6 @@ x_window_receive_event(
 				exit(0) ;
 			}
 		}
-		
 #if  0
 		else if( event->xclient.format == 32 &&
 			event->xclient.data.l[0] == XA_TAKE_FOCUS( win->disp->display))
@@ -3383,6 +3438,66 @@ x_window_copy_area(
 	return  1 ;
 }
 
+void
+x_window_set_clip(
+	x_window_t *  win ,
+	int  x ,
+	int  y ,
+	u_int  width ,
+	u_int  height
+	)
+{
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( win->cairo_draw)
+	{
+		cairo_set_clip( win , x + win->hmargin , y + win->vmargin , width , height) ;
+	}
+	else
+#endif
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+	if( win->xft_draw)
+	{
+		xft_set_clip( win , x + win->hmargin , y + win->vmargin , width , height) ;
+	}
+	else
+#endif
+	{
+		XRectangle  rect ;
+
+		rect.x = 0 ;
+		rect.y = 0 ;
+		rect.width = width ;
+		rect.height = height ;
+
+		XSetClipRectangles( win->disp->display , win->gc->gc ,
+			x + win->hmargin , y + win->vmargin , &rect , 1 , YSorted) ;
+	}
+}
+
+void
+x_window_unset_clip(
+	x_window_t *  win
+	)
+{
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( win->cairo_draw)
+	{
+		cairo_unset_clip( win) ;
+	}
+	else
+#endif
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+	if( win->xft_draw)
+	{
+		xft_unset_clip( win) ;
+	}
+	else
+#endif
+	{
+		XSetClipMask( win->disp->display , win->gc->gc , None) ;
+	}
+}
+
 int
 x_window_draw_decsp_string(
 	x_window_t *  win ,
